/****************************************************************************
** Meta object code from reading C++ file 'abstractformwindow.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../../qttools/src/designer/src/lib/sdk/abstractformwindow.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'abstractformwindow.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN28QDesignerFormWindowInterfaceE_t {};
} // unnamed namespace

template <> constexpr inline auto QDesignerFormWindowInterface::qt_create_metaobjectdata<qt_meta_tag_ZN28QDesignerFormWindowInterfaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QDesignerFormWindowInterface",
        "mainContainerChanged",
        "",
        "QWidget*",
        "mainContainer",
        "toolChanged",
        "toolIndex",
        "fileNameChanged",
        "fileName",
        "featureChanged",
        "Feature",
        "f",
        "selectionChanged",
        "geometryChanged",
        "resourceFilesChanged",
        "widgetManaged",
        "widget",
        "widgetUnmanaged",
        "aboutToUnmanageWidget",
        "activated",
        "changed",
        "widgetRemoved",
        "w",
        "objectRemoved",
        "o",
        "manageWidget",
        "unmanageWidget",
        "setFeatures",
        "setDirty",
        "dirty",
        "clearSelection",
        "changePropertyDisplay",
        "selectWidget",
        "select",
        "setGrid",
        "QPoint",
        "grid",
        "setFileName",
        "setContents",
        "contents",
        "editWidgets",
        "activateResourceFilePaths",
        "paths",
        "int*",
        "errorCount",
        "QString*",
        "errorMessages",
        "ResourceFileSaveMode",
        "SaveAllResourceFiles",
        "SaveOnlyUsedResourceFiles",
        "DontSaveResourceFiles"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'mainContainerChanged'
        QtMocHelpers::SignalData<void(QWidget *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'toolChanged'
        QtMocHelpers::SignalData<void(int)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 6 },
        }}),
        // Signal 'fileNameChanged'
        QtMocHelpers::SignalData<void(const QString &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 8 },
        }}),
        // Signal 'featureChanged'
        QtMocHelpers::SignalData<void(Feature)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Signal 'selectionChanged'
        QtMocHelpers::SignalData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'geometryChanged'
        QtMocHelpers::SignalData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'resourceFilesChanged'
        QtMocHelpers::SignalData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'widgetManaged'
        QtMocHelpers::SignalData<void(QWidget *)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 16 },
        }}),
        // Signal 'widgetUnmanaged'
        QtMocHelpers::SignalData<void(QWidget *)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 16 },
        }}),
        // Signal 'aboutToUnmanageWidget'
        QtMocHelpers::SignalData<void(QWidget *)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 16 },
        }}),
        // Signal 'activated'
        QtMocHelpers::SignalData<void(QWidget *)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 16 },
        }}),
        // Signal 'changed'
        QtMocHelpers::SignalData<void()>(20, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'widgetRemoved'
        QtMocHelpers::SignalData<void(QWidget *)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 22 },
        }}),
        // Signal 'objectRemoved'
        QtMocHelpers::SignalData<void(QObject *)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QObjectStar, 24 },
        }}),
        // Slot 'manageWidget'
        QtMocHelpers::SlotData<void(QWidget *)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 16 },
        }}),
        // Slot 'unmanageWidget'
        QtMocHelpers::SlotData<void(QWidget *)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 16 },
        }}),
        // Slot 'setFeatures'
        QtMocHelpers::SlotData<void(Feature)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Slot 'setDirty'
        QtMocHelpers::SlotData<void(bool)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 29 },
        }}),
        // Slot 'clearSelection'
        QtMocHelpers::SlotData<void(bool)>(30, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 31 },
        }}),
        // Slot 'clearSelection'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'selectWidget'
        QtMocHelpers::SlotData<void(QWidget *, bool)>(32, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 22 }, { QMetaType::Bool, 33 },
        }}),
        // Slot 'selectWidget'
        QtMocHelpers::SlotData<void(QWidget *)>(32, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 3, 22 },
        }}),
        // Slot 'setGrid'
        QtMocHelpers::SlotData<void(const QPoint &)>(34, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 35, 36 },
        }}),
        // Slot 'setFileName'
        QtMocHelpers::SlotData<void(const QString &)>(37, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 8 },
        }}),
        // Slot 'setContents'
        QtMocHelpers::SlotData<bool(const QString &)>(38, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 39 },
        }}),
        // Slot 'editWidgets'
        QtMocHelpers::SlotData<void()>(40, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'activateResourceFilePaths'
        QtMocHelpers::SlotData<void(const QStringList &, int *, QString *)>(41, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 42 }, { 0x80000000 | 43, 44 }, { 0x80000000 | 45, 46 },
        }}),
        // Slot 'activateResourceFilePaths'
        QtMocHelpers::SlotData<void(const QStringList &, int *)>(41, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QStringList, 42 }, { 0x80000000 | 43, 44 },
        }}),
        // Slot 'activateResourceFilePaths'
        QtMocHelpers::SlotData<void(const QStringList &)>(41, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QStringList, 42 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'ResourceFileSaveMode'
        QtMocHelpers::EnumData<enum ResourceFileSaveMode>(47, 47, QMC::EnumFlags{}).add({
            {   48, ResourceFileSaveMode::SaveAllResourceFiles },
            {   49, ResourceFileSaveMode::SaveOnlyUsedResourceFiles },
            {   50, ResourceFileSaveMode::DontSaveResourceFiles },
        }),
    };
    return QtMocHelpers::metaObjectData<QDesignerFormWindowInterface, qt_meta_tag_ZN28QDesignerFormWindowInterfaceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QDesignerFormWindowInterface::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN28QDesignerFormWindowInterfaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN28QDesignerFormWindowInterfaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN28QDesignerFormWindowInterfaceE_t>.metaTypes,
    nullptr
} };

void QDesignerFormWindowInterface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QDesignerFormWindowInterface *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->mainContainerChanged((*reinterpret_cast<std::add_pointer_t<QWidget*>>(_a[1]))); break;
        case 1: _t->toolChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 2: _t->fileNameChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 3: _t->featureChanged((*reinterpret_cast<std::add_pointer_t<Feature>>(_a[1]))); break;
        case 4: _t->selectionChanged(); break;
        case 5: _t->geometryChanged(); break;
        case 6: _t->resourceFilesChanged(); break;
        case 7: _t->widgetManaged((*reinterpret_cast<std::add_pointer_t<QWidget*>>(_a[1]))); break;
        case 8: _t->widgetUnmanaged((*reinterpret_cast<std::add_pointer_t<QWidget*>>(_a[1]))); break;
        case 9: _t->aboutToUnmanageWidget((*reinterpret_cast<std::add_pointer_t<QWidget*>>(_a[1]))); break;
        case 10: _t->activated((*reinterpret_cast<std::add_pointer_t<QWidget*>>(_a[1]))); break;
        case 11: _t->changed(); break;
        case 12: _t->widgetRemoved((*reinterpret_cast<std::add_pointer_t<QWidget*>>(_a[1]))); break;
        case 13: _t->objectRemoved((*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[1]))); break;
        case 14: _t->manageWidget((*reinterpret_cast<std::add_pointer_t<QWidget*>>(_a[1]))); break;
        case 15: _t->unmanageWidget((*reinterpret_cast<std::add_pointer_t<QWidget*>>(_a[1]))); break;
        case 16: _t->setFeatures((*reinterpret_cast<std::add_pointer_t<Feature>>(_a[1]))); break;
        case 17: _t->setDirty((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 18: _t->clearSelection((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 19: _t->clearSelection(); break;
        case 20: _t->selectWidget((*reinterpret_cast<std::add_pointer_t<QWidget*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 21: _t->selectWidget((*reinterpret_cast<std::add_pointer_t<QWidget*>>(_a[1]))); break;
        case 22: _t->setGrid((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 23: _t->setFileName((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 24: { bool _r = _t->setContents((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 25: _t->editWidgets(); break;
        case 26: _t->activateResourceFilePaths((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int*>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString*>>(_a[3]))); break;
        case 27: _t->activateResourceFilePaths((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int*>>(_a[2]))); break;
        case 28: _t->activateResourceFilePaths((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QWidget* >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QWidget* >(); break;
            }
            break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QWidget* >(); break;
            }
            break;
        case 9:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QWidget* >(); break;
            }
            break;
        case 10:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QWidget* >(); break;
            }
            break;
        case 12:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QWidget* >(); break;
            }
            break;
        case 14:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QWidget* >(); break;
            }
            break;
        case 15:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QWidget* >(); break;
            }
            break;
        case 20:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QWidget* >(); break;
            }
            break;
        case 21:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QWidget* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QDesignerFormWindowInterface::*)(QWidget * )>(_a, &QDesignerFormWindowInterface::mainContainerChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDesignerFormWindowInterface::*)(int )>(_a, &QDesignerFormWindowInterface::toolChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDesignerFormWindowInterface::*)(const QString & )>(_a, &QDesignerFormWindowInterface::fileNameChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDesignerFormWindowInterface::*)(Feature )>(_a, &QDesignerFormWindowInterface::featureChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDesignerFormWindowInterface::*)()>(_a, &QDesignerFormWindowInterface::selectionChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDesignerFormWindowInterface::*)()>(_a, &QDesignerFormWindowInterface::geometryChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDesignerFormWindowInterface::*)()>(_a, &QDesignerFormWindowInterface::resourceFilesChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDesignerFormWindowInterface::*)(QWidget * )>(_a, &QDesignerFormWindowInterface::widgetManaged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDesignerFormWindowInterface::*)(QWidget * )>(_a, &QDesignerFormWindowInterface::widgetUnmanaged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDesignerFormWindowInterface::*)(QWidget * )>(_a, &QDesignerFormWindowInterface::aboutToUnmanageWidget, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDesignerFormWindowInterface::*)(QWidget * )>(_a, &QDesignerFormWindowInterface::activated, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDesignerFormWindowInterface::*)()>(_a, &QDesignerFormWindowInterface::changed, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDesignerFormWindowInterface::*)(QWidget * )>(_a, &QDesignerFormWindowInterface::widgetRemoved, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDesignerFormWindowInterface::*)(QObject * )>(_a, &QDesignerFormWindowInterface::objectRemoved, 13))
            return;
    }
}

const QMetaObject *QDesignerFormWindowInterface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QDesignerFormWindowInterface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN28QDesignerFormWindowInterfaceE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int QDesignerFormWindowInterface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 29)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 29;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 29)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 29;
    }
    return _id;
}

// SIGNAL 0
void QDesignerFormWindowInterface::mainContainerChanged(QWidget * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void QDesignerFormWindowInterface::toolChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void QDesignerFormWindowInterface::fileNameChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void QDesignerFormWindowInterface::featureChanged(Feature _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void QDesignerFormWindowInterface::selectionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QDesignerFormWindowInterface::geometryChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void QDesignerFormWindowInterface::resourceFilesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void QDesignerFormWindowInterface::widgetManaged(QWidget * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void QDesignerFormWindowInterface::widgetUnmanaged(QWidget * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void QDesignerFormWindowInterface::aboutToUnmanageWidget(QWidget * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void QDesignerFormWindowInterface::activated(QWidget * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}

// SIGNAL 11
void QDesignerFormWindowInterface::changed()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}

// SIGNAL 12
void QDesignerFormWindowInterface::widgetRemoved(QWidget * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1);
}

// SIGNAL 13
void QDesignerFormWindowInterface::objectRemoved(QObject * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 13, nullptr, _t1);
}
QT_WARNING_POP
