/****************************************************************************
** Meta object code from reading C++ file 'openpagesmanager.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../qttools/src/assistant/assistant/openpagesmanager.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'openpagesmanager.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN16OpenPagesManagerE_t {};
} // unnamed namespace

template <> constexpr inline auto OpenPagesManager::qt_create_metaobjectdata<qt_meta_tag_ZN16OpenPagesManagerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OpenPagesManager",
        "aboutToAddPage",
        "",
        "pageAdded",
        "index",
        "pageClosed",
        "aboutToClosePage",
        "createPage",
        "HelpViewer*",
        "QUrl",
        "url",
        "fromSearch",
        "createNewPageFromSearch",
        "createBlankPage",
        "closeCurrentPage",
        "nextPage",
        "nextPageWithSwitcher",
        "previousPage",
        "previousPageWithSwitcher",
        "closePage",
        "page",
        "setCurrentPage",
        "QModelIndex",
        "closePagesExcept"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'aboutToAddPage'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'pageAdded'
        QtMocHelpers::SignalData<void(int)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 4 },
        }}),
        // Signal 'pageClosed'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'aboutToClosePage'
        QtMocHelpers::SignalData<void(int)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 4 },
        }}),
        // Slot 'createPage'
        QtMocHelpers::SlotData<HelpViewer *(const QUrl &, bool)>(7, 2, QMC::AccessPublic, 0x80000000 | 8, {{
            { 0x80000000 | 9, 10 }, { QMetaType::Bool, 11 },
        }}),
        // Slot 'createPage'
        QtMocHelpers::SlotData<HelpViewer *(const QUrl &)>(7, 2, QMC::AccessPublic | QMC::MethodCloned, 0x80000000 | 8, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Slot 'createNewPageFromSearch'
        QtMocHelpers::SlotData<HelpViewer *(const QUrl &)>(12, 2, QMC::AccessPublic, 0x80000000 | 8, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Slot 'createBlankPage'
        QtMocHelpers::SlotData<HelpViewer *()>(13, 2, QMC::AccessPublic, 0x80000000 | 8),
        // Slot 'closeCurrentPage'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'nextPage'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'nextPageWithSwitcher'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'previousPage'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'previousPageWithSwitcher'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'closePage'
        QtMocHelpers::SlotData<void(HelpViewer *)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 20 },
        }}),
        // Slot 'setCurrentPage'
        QtMocHelpers::SlotData<void(HelpViewer *)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 20 },
        }}),
        // Slot 'setCurrentPage'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(21, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 22, 4 },
        }}),
        // Slot 'closePage'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(19, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 22, 4 },
        }}),
        // Slot 'closePagesExcept'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(23, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 22, 4 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<OpenPagesManager, qt_meta_tag_ZN16OpenPagesManagerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OpenPagesManager::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16OpenPagesManagerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16OpenPagesManagerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16OpenPagesManagerE_t>.metaTypes,
    nullptr
} };

void OpenPagesManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<OpenPagesManager *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->aboutToAddPage(); break;
        case 1: _t->pageAdded((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 2: _t->pageClosed(); break;
        case 3: _t->aboutToClosePage((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 4: { HelpViewer* _r = _t->createPage((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])));
            if (_a[0]) *reinterpret_cast<HelpViewer**>(_a[0]) = std::move(_r); }  break;
        case 5: { HelpViewer* _r = _t->createPage((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])));
            if (_a[0]) *reinterpret_cast<HelpViewer**>(_a[0]) = std::move(_r); }  break;
        case 6: { HelpViewer* _r = _t->createNewPageFromSearch((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])));
            if (_a[0]) *reinterpret_cast<HelpViewer**>(_a[0]) = std::move(_r); }  break;
        case 7: { HelpViewer* _r = _t->createBlankPage();
            if (_a[0]) *reinterpret_cast<HelpViewer**>(_a[0]) = std::move(_r); }  break;
        case 8: _t->closeCurrentPage(); break;
        case 9: _t->nextPage(); break;
        case 10: _t->nextPageWithSwitcher(); break;
        case 11: _t->previousPage(); break;
        case 12: _t->previousPageWithSwitcher(); break;
        case 13: _t->closePage((*reinterpret_cast<std::add_pointer_t<HelpViewer*>>(_a[1]))); break;
        case 14: _t->setCurrentPage((*reinterpret_cast<std::add_pointer_t<HelpViewer*>>(_a[1]))); break;
        case 15: _t->setCurrentPage((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 16: _t->closePage((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 17: _t->closePagesExcept((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (OpenPagesManager::*)()>(_a, &OpenPagesManager::aboutToAddPage, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (OpenPagesManager::*)(int )>(_a, &OpenPagesManager::pageAdded, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (OpenPagesManager::*)()>(_a, &OpenPagesManager::pageClosed, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (OpenPagesManager::*)(int )>(_a, &OpenPagesManager::aboutToClosePage, 3))
            return;
    }
}

const QMetaObject *OpenPagesManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OpenPagesManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16OpenPagesManagerE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int OpenPagesManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 18)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 18;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 18)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 18;
    }
    return _id;
}

// SIGNAL 0
void OpenPagesManager::aboutToAddPage()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void OpenPagesManager::pageAdded(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void OpenPagesManager::pageClosed()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void OpenPagesManager::aboutToClosePage(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}
QT_WARNING_POP
