/****************************************************************************
** Meta object code from reading C++ file 'messagemodel.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../qttools/src/linguist/linguist/messagemodel.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'messagemodel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN9DataModelE_t {};
} // unnamed namespace

template <> constexpr inline auto DataModel::qt_create_metaobjectdata<qt_meta_tag_ZN9DataModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DataModel",
        "statsChanged",
        "",
        "StatisticalData",
        "newStats",
        "progressChanged",
        "finishedCount",
        "oldFinishedCount",
        "languageChanged",
        "modifiedChanged"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'statsChanged'
        QtMocHelpers::SignalData<void(const StatisticalData &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'progressChanged'
        QtMocHelpers::SignalData<void(int, int)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 6 }, { QMetaType::Int, 7 },
        }}),
        // Signal 'languageChanged'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'modifiedChanged'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DataModel, qt_meta_tag_ZN9DataModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject DataModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9DataModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9DataModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN9DataModelE_t>.metaTypes,
    nullptr
} };

void DataModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DataModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->statsChanged((*reinterpret_cast<std::add_pointer_t<StatisticalData>>(_a[1]))); break;
        case 1: _t->progressChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 2: _t->languageChanged(); break;
        case 3: _t->modifiedChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DataModel::*)(const StatisticalData & )>(_a, &DataModel::statsChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DataModel::*)(int , int )>(_a, &DataModel::progressChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (DataModel::*)()>(_a, &DataModel::languageChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (DataModel::*)()>(_a, &DataModel::modifiedChanged, 3))
            return;
    }
}

const QMetaObject *DataModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DataModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9DataModelE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int DataModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void DataModel::statsChanged(const StatisticalData & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void DataModel::progressChanged(int _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void DataModel::languageChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void DataModel::modifiedChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}
namespace {
struct qt_meta_tag_ZN14MultiDataModelE_t {};
} // unnamed namespace

template <> constexpr inline auto MultiDataModel::qt_create_metaobjectdata<qt_meta_tag_ZN14MultiDataModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "MultiDataModel",
        "modelAppended",
        "",
        "modelDeleted",
        "model",
        "allModelsDeleted",
        "languageChanged",
        "statsChanged",
        "StatisticalData",
        "newStats",
        "modifiedChanged",
        "multiGroupDataChanged",
        "MultiDataIndex",
        "index",
        "groupDataChanged",
        "messageDataChanged",
        "translationChanged",
        "onModifiedChanged",
        "onLanguageChanged"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'modelAppended'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'modelDeleted'
        QtMocHelpers::SignalData<void(int)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 4 },
        }}),
        // Signal 'allModelsDeleted'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'languageChanged'
        QtMocHelpers::SignalData<void(int)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 4 },
        }}),
        // Signal 'statsChanged'
        QtMocHelpers::SignalData<void(const StatisticalData &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Signal 'modifiedChanged'
        QtMocHelpers::SignalData<void(bool)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Signal 'multiGroupDataChanged'
        QtMocHelpers::SignalData<void(const MultiDataIndex &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Signal 'groupDataChanged'
        QtMocHelpers::SignalData<void(const MultiDataIndex &)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Signal 'messageDataChanged'
        QtMocHelpers::SignalData<void(const MultiDataIndex &)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Signal 'translationChanged'
        QtMocHelpers::SignalData<void(const MultiDataIndex &)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Slot 'onModifiedChanged'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onLanguageChanged'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<MultiDataModel, qt_meta_tag_ZN14MultiDataModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject MultiDataModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14MultiDataModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14MultiDataModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14MultiDataModelE_t>.metaTypes,
    nullptr
} };

void MultiDataModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<MultiDataModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->modelAppended(); break;
        case 1: _t->modelDeleted((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 2: _t->allModelsDeleted(); break;
        case 3: _t->languageChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 4: _t->statsChanged((*reinterpret_cast<std::add_pointer_t<StatisticalData>>(_a[1]))); break;
        case 5: _t->modifiedChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 6: _t->multiGroupDataChanged((*reinterpret_cast<std::add_pointer_t<MultiDataIndex>>(_a[1]))); break;
        case 7: _t->groupDataChanged((*reinterpret_cast<std::add_pointer_t<MultiDataIndex>>(_a[1]))); break;
        case 8: _t->messageDataChanged((*reinterpret_cast<std::add_pointer_t<MultiDataIndex>>(_a[1]))); break;
        case 9: _t->translationChanged((*reinterpret_cast<std::add_pointer_t<MultiDataIndex>>(_a[1]))); break;
        case 10: _t->onModifiedChanged(); break;
        case 11: _t->onLanguageChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (MultiDataModel::*)()>(_a, &MultiDataModel::modelAppended, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (MultiDataModel::*)(int )>(_a, &MultiDataModel::modelDeleted, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (MultiDataModel::*)()>(_a, &MultiDataModel::allModelsDeleted, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (MultiDataModel::*)(int )>(_a, &MultiDataModel::languageChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (MultiDataModel::*)(const StatisticalData & )>(_a, &MultiDataModel::statsChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (MultiDataModel::*)(bool )>(_a, &MultiDataModel::modifiedChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (MultiDataModel::*)(const MultiDataIndex & )>(_a, &MultiDataModel::multiGroupDataChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (MultiDataModel::*)(const MultiDataIndex & )>(_a, &MultiDataModel::groupDataChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (MultiDataModel::*)(const MultiDataIndex & )>(_a, &MultiDataModel::messageDataChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (MultiDataModel::*)(const MultiDataIndex & )>(_a, &MultiDataModel::translationChanged, 9))
            return;
    }
}

const QMetaObject *MultiDataModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MultiDataModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14MultiDataModelE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int MultiDataModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 12;
    }
    return _id;
}

// SIGNAL 0
void MultiDataModel::modelAppended()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void MultiDataModel::modelDeleted(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void MultiDataModel::allModelsDeleted()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void MultiDataModel::languageChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void MultiDataModel::statsChanged(const StatisticalData & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void MultiDataModel::modifiedChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void MultiDataModel::multiGroupDataChanged(const MultiDataIndex & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void MultiDataModel::groupDataChanged(const MultiDataIndex & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void MultiDataModel::messageDataChanged(const MultiDataIndex & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void MultiDataModel::translationChanged(const MultiDataIndex & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN12MessageModelE_t {};
} // unnamed namespace

template <> constexpr inline auto MessageModel::qt_create_metaobjectdata<qt_meta_tag_ZN12MessageModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "MessageModel",
        "multiGroupItemChanged",
        "",
        "MultiDataIndex",
        "index",
        "groupItemChanged",
        "messageItemChanged"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'multiGroupItemChanged'
        QtMocHelpers::SlotData<void(const MultiDataIndex &)>(1, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'groupItemChanged'
        QtMocHelpers::SlotData<void(const MultiDataIndex &)>(5, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'messageItemChanged'
        QtMocHelpers::SlotData<void(const MultiDataIndex &)>(6, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<MessageModel, qt_meta_tag_ZN12MessageModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject MessageModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractItemModel::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12MessageModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12MessageModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12MessageModelE_t>.metaTypes,
    nullptr
} };

void MessageModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<MessageModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->multiGroupItemChanged((*reinterpret_cast<std::add_pointer_t<MultiDataIndex>>(_a[1]))); break;
        case 1: _t->groupItemChanged((*reinterpret_cast<std::add_pointer_t<MultiDataIndex>>(_a[1]))); break;
        case 2: _t->messageItemChanged((*reinterpret_cast<std::add_pointer_t<MultiDataIndex>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *MessageModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MessageModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12MessageModelE_t>.strings))
        return static_cast<void*>(this);
    return QAbstractItemModel::qt_metacast(_clname);
}

int MessageModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractItemModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 3;
    }
    return _id;
}
QT_WARNING_POP
