/****************************************************************************
** Meta object code from reading C++ file 'qdesigner_propertyeditor_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../../qttools/src/designer/src/lib/shared/qdesigner_propertyeditor_p.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qdesigner_propertyeditor_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN18qdesigner_internal23QDesignerPropertyEditorE_t {};
} // unnamed namespace

template <> constexpr inline auto qdesigner_internal::QDesignerPropertyEditor::qt_create_metaobjectdata<qt_meta_tag_ZN18qdesigner_internal23QDesignerPropertyEditorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "qdesigner_internal::QDesignerPropertyEditor",
        "propertyValueChanged",
        "",
        "name",
        "QVariant",
        "value",
        "enableSubPropertyHandling",
        "resetProperty",
        "addDynamicProperty",
        "removeDynamicProperty",
        "editorOpened",
        "editorClosed",
        "updatePropertySheet",
        "reloadResourceProperties",
        "slotPropertyChanged"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'propertyValueChanged'
        QtMocHelpers::SignalData<void(const QString &, const QVariant &, bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { 0x80000000 | 4, 5 }, { QMetaType::Bool, 6 },
        }}),
        // Signal 'resetProperty'
        QtMocHelpers::SignalData<void(const QString &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'addDynamicProperty'
        QtMocHelpers::SignalData<void(const QString &, const QVariant &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { 0x80000000 | 4, 5 },
        }}),
        // Signal 'removeDynamicProperty'
        QtMocHelpers::SignalData<void(const QString &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'editorOpened'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'editorClosed'
        QtMocHelpers::SignalData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'updatePropertySheet'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'reloadResourceProperties'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotPropertyChanged'
        QtMocHelpers::SlotData<void(const QString &, const QVariant &)>(14, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { 0x80000000 | 4, 5 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QDesignerPropertyEditor, qt_meta_tag_ZN18qdesigner_internal23QDesignerPropertyEditorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject qdesigner_internal::QDesignerPropertyEditor::staticMetaObject = { {
    QMetaObject::SuperData::link<QDesignerPropertyEditorInterface::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18qdesigner_internal23QDesignerPropertyEditorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18qdesigner_internal23QDesignerPropertyEditorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN18qdesigner_internal23QDesignerPropertyEditorE_t>.metaTypes,
    nullptr
} };

void qdesigner_internal::QDesignerPropertyEditor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QDesignerPropertyEditor *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->propertyValueChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 1: _t->resetProperty((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->addDynamicProperty((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 3: _t->removeDynamicProperty((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 4: _t->editorOpened(); break;
        case 5: _t->editorClosed(); break;
        case 6: _t->updatePropertySheet(); break;
        case 7: _t->reloadResourceProperties(); break;
        case 8: _t->slotPropertyChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QDesignerPropertyEditor::*)(const QString & , const QVariant & , bool )>(_a, &QDesignerPropertyEditor::propertyValueChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDesignerPropertyEditor::*)(const QString & )>(_a, &QDesignerPropertyEditor::resetProperty, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDesignerPropertyEditor::*)(const QString & , const QVariant & )>(_a, &QDesignerPropertyEditor::addDynamicProperty, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDesignerPropertyEditor::*)(const QString & )>(_a, &QDesignerPropertyEditor::removeDynamicProperty, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDesignerPropertyEditor::*)()>(_a, &QDesignerPropertyEditor::editorOpened, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDesignerPropertyEditor::*)()>(_a, &QDesignerPropertyEditor::editorClosed, 5))
            return;
    }
}

const QMetaObject *qdesigner_internal::QDesignerPropertyEditor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *qdesigner_internal::QDesignerPropertyEditor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18qdesigner_internal23QDesignerPropertyEditorE_t>.strings))
        return static_cast<void*>(this);
    return QDesignerPropertyEditorInterface::qt_metacast(_clname);
}

int qdesigner_internal::QDesignerPropertyEditor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDesignerPropertyEditorInterface::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 9;
    }
    return _id;
}

// SIGNAL 0
void qdesigner_internal::QDesignerPropertyEditor::propertyValueChanged(const QString & _t1, const QVariant & _t2, bool _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2, _t3);
}

// SIGNAL 1
void qdesigner_internal::QDesignerPropertyEditor::resetProperty(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void qdesigner_internal::QDesignerPropertyEditor::addDynamicProperty(const QString & _t1, const QVariant & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void qdesigner_internal::QDesignerPropertyEditor::removeDynamicProperty(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void qdesigner_internal::QDesignerPropertyEditor::editorOpened()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void qdesigner_internal::QDesignerPropertyEditor::editorClosed()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}
QT_WARNING_POP
