/*

* Copyright (C) 2016 The Qt Company Ltd.
* SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0

*/

/********************************************************************************
** Form generated from reading UI file 'mainwindow.ui'
**
** Created by: Qt User Interface Compiler version 6.12.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_MAINWINDOW_H
#define UI_MAINWINDOW_H

#include <QtCore/QVariant>
#include <QtGui/QAction>
#include <QtWidgets/QApplication>
#include <QtWidgets/QMainWindow>
#include <QtWidgets/QMenu>
#include <QtWidgets/QMenuBar>
#include <QtWidgets/QStatusBar>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_MainWindow
{
public:
    QAction *actionOpen;
    QAction *actionExit;
    QAction *actionSave;
    QAction *actionSaveAs;
    QAction *actionRelease;
    QAction *actionPrint;
    QAction *actionUndo;
    QAction *actionRedo;
    QAction *actionCut;
    QAction *actionCopy;
    QAction *actionPaste;
    QAction *actionSelectAll;
    QAction *actionFind;
    QAction *actionFindNext;
    QAction *actionPrevUnfinished;
    QAction *actionNextUnfinished;
    QAction *actionPrev;
    QAction *actionNext;
    QAction *actionDoneAndNext;
    QAction *actionBeginFromSource;
    QAction *actionAccelerators;
    QAction *actionSurroundingWhitespace;
    QAction *actionEndingPunctuation;
    QAction *actionPhraseMatches;
    QAction *actionPlaceMarkerMatches;
    QAction *actionNewPhraseBook;
    QAction *actionOpenPhraseBook;
    QAction *actionResetSorting;
    QAction *actionDisplayGuesses;
    QAction *actionStatistics;
    QAction *actionQmlPreview;
    QAction *actionManual;
    QAction *actionAbout;
    QAction *actionAboutQt;
    QAction *actionWhatsThis;
    QAction *actionSearchAndTranslate;
    QAction *actionBatchTranslation;
    QAction *actionReleaseAs;
    QAction *actionFile;
    QAction *actionEdit;
    QAction *actionTranslation;
    QAction *actionValidation;
    QAction *actionHelp;
    QAction *actionPreviewForm;
    QAction *actionTranslationFileSettings;
    QAction *actionAddToPhraseBook;
    QAction *actionOpenAux;
    QAction *actionSaveAll;
    QAction *actionReleaseAll;
    QAction *actionClose;
    QAction *actionCloseAll;
    QAction *actionLengthVariants;
    QAction *actionVisualizeWhitespace;
    QAction *actionIncreaseZoom;
    QAction *actionDecreaseZoom;
    QAction *actionResetZoomToDefault;
    QAction *actionShowMoreGuesses;
    QAction *actionShowFewerGuesses;
    QAction *actionResetGuessesToDefault;
    QAction *actionDone;
    QAction *actionFindPrev;
    QAction *actionAuto_Translation;
    QWidget *centralwidget;
    QMenuBar *menubar;
    QMenu *menuPhrases;
    QMenu *menuClosePhraseBook;
    QMenu *menuEditPhraseBook;
    QMenu *menuPrintPhraseBook;
    QMenu *menuValidation;
    QMenu *menuView;
    QMenu *menuViewViews;
    QMenu *menuToolbars;
    QMenu *menuZoom;
    QMenu *menuGuesses;
    QMenu *menuHelp;
    QMenu *menuTranslation;
    QMenu *menuFile;
    QMenu *menuRecentlyOpenedFiles;
    QMenu *menuEdit;
    QStatusBar *statusbar;

    void setupUi(QMainWindow *MainWindow)
    {
        if (MainWindow->objectName().isEmpty())
            MainWindow->setObjectName("MainWindow");
        MainWindow->resize(673, 461);
        actionOpen = new QAction(MainWindow);
        actionOpen->setObjectName("actionOpen");
        actionExit = new QAction(MainWindow);
        actionExit->setObjectName("actionExit");
        actionExit->setMenuRole(QAction::MenuRole::QuitRole);
        actionSave = new QAction(MainWindow);
        actionSave->setObjectName("actionSave");
        actionSaveAs = new QAction(MainWindow);
        actionSaveAs->setObjectName("actionSaveAs");
        actionRelease = new QAction(MainWindow);
        actionRelease->setObjectName("actionRelease");
        actionPrint = new QAction(MainWindow);
        actionPrint->setObjectName("actionPrint");
        actionUndo = new QAction(MainWindow);
        actionUndo->setObjectName("actionUndo");
        actionUndo->setEnabled(false);
        actionUndo->setMenuRole(QAction::MenuRole::NoRole);
        actionRedo = new QAction(MainWindow);
        actionRedo->setObjectName("actionRedo");
        actionRedo->setEnabled(false);
        actionRedo->setMenuRole(QAction::MenuRole::NoRole);
        actionCut = new QAction(MainWindow);
        actionCut->setObjectName("actionCut");
        actionCut->setEnabled(false);
        actionCopy = new QAction(MainWindow);
        actionCopy->setObjectName("actionCopy");
        actionCopy->setEnabled(false);
        actionPaste = new QAction(MainWindow);
        actionPaste->setObjectName("actionPaste");
        actionPaste->setEnabled(false);
        actionSelectAll = new QAction(MainWindow);
        actionSelectAll->setObjectName("actionSelectAll");
        actionSelectAll->setEnabled(false);
        actionFind = new QAction(MainWindow);
        actionFind->setObjectName("actionFind");
        actionFind->setEnabled(false);
        actionFind->setMenuRole(QAction::MenuRole::NoRole);
        actionFindNext = new QAction(MainWindow);
        actionFindNext->setObjectName("actionFindNext");
        actionFindNext->setEnabled(false);
        actionFindNext->setMenuRole(QAction::MenuRole::NoRole);
        actionPrevUnfinished = new QAction(MainWindow);
        actionPrevUnfinished->setObjectName("actionPrevUnfinished");
        actionPrevUnfinished->setMenuRole(QAction::MenuRole::NoRole);
        actionNextUnfinished = new QAction(MainWindow);
        actionNextUnfinished->setObjectName("actionNextUnfinished");
        actionNextUnfinished->setMenuRole(QAction::MenuRole::NoRole);
        actionPrev = new QAction(MainWindow);
        actionPrev->setObjectName("actionPrev");
        actionPrev->setMenuRole(QAction::MenuRole::NoRole);
        actionNext = new QAction(MainWindow);
        actionNext->setObjectName("actionNext");
        actionNext->setMenuRole(QAction::MenuRole::NoRole);
        actionDoneAndNext = new QAction(MainWindow);
        actionDoneAndNext->setObjectName("actionDoneAndNext");
        actionDoneAndNext->setEnabled(false);
        actionDoneAndNext->setMenuRole(QAction::MenuRole::NoRole);
        actionBeginFromSource = new QAction(MainWindow);
        actionBeginFromSource->setObjectName("actionBeginFromSource");
        actionBeginFromSource->setEnabled(true);
        actionBeginFromSource->setMenuRole(QAction::MenuRole::NoRole);
        actionAccelerators = new QAction(MainWindow);
        actionAccelerators->setObjectName("actionAccelerators");
        actionAccelerators->setCheckable(true);
        actionAccelerators->setMenuRole(QAction::MenuRole::NoRole);
        actionSurroundingWhitespace = new QAction(MainWindow);
        actionSurroundingWhitespace->setObjectName("actionSurroundingWhitespace");
        actionSurroundingWhitespace->setCheckable(true);
        actionSurroundingWhitespace->setMenuRole(QAction::MenuRole::NoRole);
        actionEndingPunctuation = new QAction(MainWindow);
        actionEndingPunctuation->setObjectName("actionEndingPunctuation");
        actionEndingPunctuation->setCheckable(true);
        actionEndingPunctuation->setMenuRole(QAction::MenuRole::NoRole);
        actionPhraseMatches = new QAction(MainWindow);
        actionPhraseMatches->setObjectName("actionPhraseMatches");
        actionPhraseMatches->setCheckable(true);
        actionPhraseMatches->setMenuRole(QAction::MenuRole::NoRole);
        actionPlaceMarkerMatches = new QAction(MainWindow);
        actionPlaceMarkerMatches->setObjectName("actionPlaceMarkerMatches");
        actionPlaceMarkerMatches->setCheckable(true);
        actionPlaceMarkerMatches->setMenuRole(QAction::MenuRole::NoRole);
        actionNewPhraseBook = new QAction(MainWindow);
        actionNewPhraseBook->setObjectName("actionNewPhraseBook");
        actionNewPhraseBook->setMenuRole(QAction::MenuRole::NoRole);
        actionOpenPhraseBook = new QAction(MainWindow);
        actionOpenPhraseBook->setObjectName("actionOpenPhraseBook");
        actionOpenPhraseBook->setMenuRole(QAction::MenuRole::NoRole);
        actionResetSorting = new QAction(MainWindow);
        actionResetSorting->setObjectName("actionResetSorting");
        actionResetSorting->setCheckable(false);
        actionResetSorting->setChecked(false);
        actionResetSorting->setMenuRole(QAction::MenuRole::NoRole);
        actionDisplayGuesses = new QAction(MainWindow);
        actionDisplayGuesses->setObjectName("actionDisplayGuesses");
        actionDisplayGuesses->setCheckable(true);
        actionDisplayGuesses->setChecked(true);
        actionDisplayGuesses->setMenuRole(QAction::MenuRole::NoRole);
        actionStatistics = new QAction(MainWindow);
        actionStatistics->setObjectName("actionStatistics");
        actionStatistics->setCheckable(false);
        actionStatistics->setChecked(false);
        actionStatistics->setMenuRole(QAction::MenuRole::NoRole);
        actionQmlPreview = new QAction(MainWindow);
        actionQmlPreview->setObjectName("actionQmlPreview");
        actionQmlPreview->setCheckable(true);
        actionQmlPreview->setChecked(true);
        actionQmlPreview->setMenuRole(QAction::MenuRole::NoRole);
        actionManual = new QAction(MainWindow);
        actionManual->setObjectName("actionManual");
        actionManual->setMenuRole(QAction::MenuRole::NoRole);
        actionAbout = new QAction(MainWindow);
        actionAbout->setObjectName("actionAbout");
        actionAbout->setMenuRole(QAction::MenuRole::AboutRole);
        actionAboutQt = new QAction(MainWindow);
        actionAboutQt->setObjectName("actionAboutQt");
        actionAboutQt->setMenuRole(QAction::MenuRole::AboutQtRole);
        actionWhatsThis = new QAction(MainWindow);
        actionWhatsThis->setObjectName("actionWhatsThis");
        actionWhatsThis->setCheckable(false);
        actionWhatsThis->setChecked(false);
        actionWhatsThis->setMenuRole(QAction::MenuRole::NoRole);
        actionSearchAndTranslate = new QAction(MainWindow);
        actionSearchAndTranslate->setObjectName("actionSearchAndTranslate");
        actionSearchAndTranslate->setEnabled(false);
        actionSearchAndTranslate->setMenuRole(QAction::MenuRole::NoRole);
        actionBatchTranslation = new QAction(MainWindow);
        actionBatchTranslation->setObjectName("actionBatchTranslation");
        actionBatchTranslation->setEnabled(false);
        actionBatchTranslation->setMenuRole(QAction::MenuRole::NoRole);
        actionReleaseAs = new QAction(MainWindow);
        actionReleaseAs->setObjectName("actionReleaseAs");
        actionFile = new QAction(MainWindow);
        actionFile->setObjectName("actionFile");
        actionFile->setCheckable(true);
        actionFile->setChecked(true);
        actionEdit = new QAction(MainWindow);
        actionEdit->setObjectName("actionEdit");
        actionEdit->setCheckable(true);
        actionEdit->setChecked(true);
        actionTranslation = new QAction(MainWindow);
        actionTranslation->setObjectName("actionTranslation");
        actionTranslation->setCheckable(true);
        actionTranslation->setChecked(true);
        actionValidation = new QAction(MainWindow);
        actionValidation->setObjectName("actionValidation");
        actionValidation->setCheckable(true);
        actionValidation->setChecked(true);
        actionHelp = new QAction(MainWindow);
        actionHelp->setObjectName("actionHelp");
        actionHelp->setCheckable(true);
        actionHelp->setChecked(true);
        actionPreviewForm = new QAction(MainWindow);
        actionPreviewForm->setObjectName("actionPreviewForm");
        actionPreviewForm->setEnabled(false);
        actionPreviewForm->setMenuRole(QAction::MenuRole::NoRole);
        actionTranslationFileSettings = new QAction(MainWindow);
        actionTranslationFileSettings->setObjectName("actionTranslationFileSettings");
        actionTranslationFileSettings->setEnabled(false);
        actionTranslationFileSettings->setMenuRole(QAction::MenuRole::NoRole);
        actionAddToPhraseBook = new QAction(MainWindow);
        actionAddToPhraseBook->setObjectName("actionAddToPhraseBook");
        actionAddToPhraseBook->setEnabled(false);
        actionOpenAux = new QAction(MainWindow);
        actionOpenAux->setObjectName("actionOpenAux");
        actionSaveAll = new QAction(MainWindow);
        actionSaveAll->setObjectName("actionSaveAll");
        actionReleaseAll = new QAction(MainWindow);
        actionReleaseAll->setObjectName("actionReleaseAll");
        actionClose = new QAction(MainWindow);
        actionClose->setObjectName("actionClose");
        actionCloseAll = new QAction(MainWindow);
        actionCloseAll->setObjectName("actionCloseAll");
        actionLengthVariants = new QAction(MainWindow);
        actionLengthVariants->setObjectName("actionLengthVariants");
        actionLengthVariants->setCheckable(true);
        actionVisualizeWhitespace = new QAction(MainWindow);
        actionVisualizeWhitespace->setObjectName("actionVisualizeWhitespace");
        actionVisualizeWhitespace->setCheckable(true);
        actionIncreaseZoom = new QAction(MainWindow);
        actionIncreaseZoom->setObjectName("actionIncreaseZoom");
        actionDecreaseZoom = new QAction(MainWindow);
        actionDecreaseZoom->setObjectName("actionDecreaseZoom");
        actionResetZoomToDefault = new QAction(MainWindow);
        actionResetZoomToDefault->setObjectName("actionResetZoomToDefault");
        actionShowMoreGuesses = new QAction(MainWindow);
        actionShowMoreGuesses->setObjectName("actionShowMoreGuesses");
        actionShowFewerGuesses = new QAction(MainWindow);
        actionShowFewerGuesses->setObjectName("actionShowFewerGuesses");
        actionResetGuessesToDefault = new QAction(MainWindow);
        actionResetGuessesToDefault->setObjectName("actionResetGuessesToDefault");
        actionDone = new QAction(MainWindow);
        actionDone->setObjectName("actionDone");
        actionDone->setEnabled(false);
        actionDone->setMenuRole(QAction::MenuRole::NoRole);
        actionFindPrev = new QAction(MainWindow);
        actionFindPrev->setObjectName("actionFindPrev");
        actionFindPrev->setEnabled(false);
        actionAuto_Translation = new QAction(MainWindow);
        actionAuto_Translation->setObjectName("actionAuto_Translation");
        centralwidget = new QWidget(MainWindow);
        centralwidget->setObjectName("centralwidget");
        MainWindow->setCentralWidget(centralwidget);
        menubar = new QMenuBar(MainWindow);
        menubar->setObjectName("menubar");
        menubar->setGeometry(QRect(0, 0, 673, 24));
        menuPhrases = new QMenu(menubar);
        menuPhrases->setObjectName("menuPhrases");
        menuClosePhraseBook = new QMenu(menuPhrases);
        menuClosePhraseBook->setObjectName("menuClosePhraseBook");
        menuEditPhraseBook = new QMenu(menuPhrases);
        menuEditPhraseBook->setObjectName("menuEditPhraseBook");
        menuPrintPhraseBook = new QMenu(menuPhrases);
        menuPrintPhraseBook->setObjectName("menuPrintPhraseBook");
        menuValidation = new QMenu(menubar);
        menuValidation->setObjectName("menuValidation");
        menuView = new QMenu(menubar);
        menuView->setObjectName("menuView");
        menuViewViews = new QMenu(menuView);
        menuViewViews->setObjectName("menuViewViews");
        menuToolbars = new QMenu(menuView);
        menuToolbars->setObjectName("menuToolbars");
        menuZoom = new QMenu(menuView);
        menuZoom->setObjectName("menuZoom");
        menuGuesses = new QMenu(menuView);
        menuGuesses->setObjectName("menuGuesses");
        menuHelp = new QMenu(menubar);
        menuHelp->setObjectName("menuHelp");
        menuTranslation = new QMenu(menubar);
        menuTranslation->setObjectName("menuTranslation");
        menuFile = new QMenu(menubar);
        menuFile->setObjectName("menuFile");
        menuRecentlyOpenedFiles = new QMenu(menuFile);
        menuRecentlyOpenedFiles->setObjectName("menuRecentlyOpenedFiles");
        menuEdit = new QMenu(menubar);
        menuEdit->setObjectName("menuEdit");
        MainWindow->setMenuBar(menubar);
        statusbar = new QStatusBar(MainWindow);
        statusbar->setObjectName("statusbar");
        MainWindow->setStatusBar(statusbar);

        menubar->addAction(menuFile->menuAction());
        menubar->addAction(menuEdit->menuAction());
        menubar->addAction(menuTranslation->menuAction());
        menubar->addAction(menuValidation->menuAction());
        menubar->addAction(menuPhrases->menuAction());
        menubar->addAction(menuView->menuAction());
        menubar->addAction(menuHelp->menuAction());
        menuPhrases->addAction(actionNewPhraseBook);
        menuPhrases->addAction(actionOpenPhraseBook);
        menuPhrases->addAction(menuClosePhraseBook->menuAction());
        menuPhrases->addSeparator();
        menuPhrases->addAction(menuEditPhraseBook->menuAction());
        menuPhrases->addAction(menuPrintPhraseBook->menuAction());
        menuPhrases->addAction(actionAddToPhraseBook);
        menuValidation->addAction(actionAccelerators);
        menuValidation->addAction(actionSurroundingWhitespace);
        menuValidation->addAction(actionEndingPunctuation);
        menuValidation->addAction(actionPhraseMatches);
        menuValidation->addAction(actionPlaceMarkerMatches);
        menuView->addAction(actionResetSorting);
        menuView->addAction(actionDisplayGuesses);
        menuView->addAction(actionQmlPreview);
        menuView->addAction(actionLengthVariants);
        menuView->addAction(actionVisualizeWhitespace);
        menuView->addSeparator();
        menuView->addAction(menuZoom->menuAction());
        menuView->addAction(menuGuesses->menuAction());
        menuView->addSeparator();
        menuView->addAction(menuToolbars->menuAction());
        menuView->addAction(menuViewViews->menuAction());
        menuView->addAction(actionStatistics);
        menuZoom->addAction(actionIncreaseZoom);
        menuZoom->addAction(actionDecreaseZoom);
        menuZoom->addSeparator();
        menuZoom->addAction(actionResetZoomToDefault);
        menuGuesses->addAction(actionShowMoreGuesses);
        menuGuesses->addAction(actionShowFewerGuesses);
        menuGuesses->addSeparator();
        menuGuesses->addAction(actionResetGuessesToDefault);
        menuHelp->addAction(actionManual);
        menuHelp->addAction(actionAbout);
        menuHelp->addAction(actionAboutQt);
        menuHelp->addAction(actionWhatsThis);
        menuTranslation->addAction(actionAuto_Translation);
        menuTranslation->addSeparator();
        menuTranslation->addAction(actionPrevUnfinished);
        menuTranslation->addAction(actionNextUnfinished);
        menuTranslation->addAction(actionPrev);
        menuTranslation->addAction(actionNext);
        menuTranslation->addAction(actionDone);
        menuTranslation->addAction(actionDoneAndNext);
        menuTranslation->addAction(actionBeginFromSource);
        menuFile->addAction(actionOpen);
        menuFile->addAction(actionOpenAux);
        menuFile->addAction(menuRecentlyOpenedFiles->menuAction());
        menuFile->addSeparator();
        menuFile->addAction(actionSaveAll);
        menuFile->addAction(actionSave);
        menuFile->addAction(actionSaveAs);
        menuFile->addAction(actionReleaseAll);
        menuFile->addAction(actionRelease);
        menuFile->addAction(actionReleaseAs);
        menuFile->addSeparator();
        menuFile->addAction(actionPrint);
        menuFile->addSeparator();
        menuFile->addAction(actionCloseAll);
        menuFile->addAction(actionClose);
        menuFile->addSeparator();
        menuFile->addAction(actionExit);
        menuEdit->addAction(actionUndo);
        menuEdit->addAction(actionRedo);
        menuEdit->addSeparator();
        menuEdit->addAction(actionCut);
        menuEdit->addAction(actionCopy);
        menuEdit->addAction(actionPaste);
        menuEdit->addAction(actionSelectAll);
        menuEdit->addSeparator();
        menuEdit->addAction(actionFind);
        menuEdit->addAction(actionFindPrev);
        menuEdit->addAction(actionFindNext);
        menuEdit->addAction(actionSearchAndTranslate);
        menuEdit->addAction(actionBatchTranslation);
        menuEdit->addAction(actionTranslationFileSettings);

        retranslateUi(MainWindow);

        QMetaObject::connectSlotsByName(MainWindow);
    } // setupUi

    void retranslateUi(QMainWindow *MainWindow)
    {
        MainWindow->setWindowTitle(QCoreApplication::translate("MainWindow", "MainWindow", nullptr));
        actionOpen->setText(QCoreApplication::translate("MainWindow", "&Open...", nullptr));
#if QT_CONFIG(whatsthis)
        actionOpen->setWhatsThis(QCoreApplication::translate("MainWindow", "Open a Qt translation source file (TS file) for editing", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(shortcut)
        actionOpen->setShortcut(QCoreApplication::translate("MainWindow", "Ctrl+O", nullptr));
#endif // QT_CONFIG(shortcut)
        actionExit->setText(QCoreApplication::translate("MainWindow", "E&xit", nullptr));
#if QT_CONFIG(statustip)
        actionExit->setStatusTip(QString());
#endif // QT_CONFIG(statustip)
#if QT_CONFIG(whatsthis)
        actionExit->setWhatsThis(QCoreApplication::translate("MainWindow", "Close this window and exit.", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(shortcut)
        actionExit->setShortcut(QCoreApplication::translate("MainWindow", "Ctrl+Q", nullptr));
#endif // QT_CONFIG(shortcut)
        actionSave->setText(QCoreApplication::translate("MainWindow", "Save", nullptr));
#if QT_CONFIG(whatsthis)
        actionSave->setWhatsThis(QCoreApplication::translate("MainWindow", "Save changes made to this Qt translation source file", nullptr));
#endif // QT_CONFIG(whatsthis)
        actionSaveAs->setText(QCoreApplication::translate("MainWindow", "Save &As...", nullptr));
        actionSaveAs->setIconText(QCoreApplication::translate("MainWindow", "Save As...", nullptr));
#if QT_CONFIG(whatsthis)
        actionSaveAs->setWhatsThis(QCoreApplication::translate("MainWindow", "Save changes made to this Qt translation source file into a new file.", nullptr));
#endif // QT_CONFIG(whatsthis)
        actionRelease->setText(QCoreApplication::translate("MainWindow", "Release", nullptr));
#if QT_CONFIG(whatsthis)
        actionRelease->setWhatsThis(QCoreApplication::translate("MainWindow", "Create a Qt message file suitable for released applications from the current message file.", nullptr));
#endif // QT_CONFIG(whatsthis)
        actionPrint->setText(QCoreApplication::translate("MainWindow", "&Print...", nullptr));
#if QT_CONFIG(whatsthis)
        actionPrint->setWhatsThis(QCoreApplication::translate("MainWindow", "Print a list of all the translation units in the current translation source file.", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(shortcut)
        actionPrint->setShortcut(QCoreApplication::translate("MainWindow", "Ctrl+P", nullptr));
#endif // QT_CONFIG(shortcut)
        actionUndo->setText(QCoreApplication::translate("MainWindow", "&Undo", nullptr));
#if QT_CONFIG(whatsthis)
        actionUndo->setWhatsThis(QCoreApplication::translate("MainWindow", "Undo the last editing operation performed on the current translation.", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(shortcut)
        actionUndo->setShortcut(QCoreApplication::translate("MainWindow", "Ctrl+Z", nullptr));
#endif // QT_CONFIG(shortcut)
        actionRedo->setText(QCoreApplication::translate("MainWindow", "&Redo", nullptr));
#if QT_CONFIG(whatsthis)
        actionRedo->setWhatsThis(QCoreApplication::translate("MainWindow", "Redo an undone editing operation performed on the translation.", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(shortcut)
        actionRedo->setShortcut(QCoreApplication::translate("MainWindow", "Ctrl+Y", nullptr));
#endif // QT_CONFIG(shortcut)
        actionCut->setText(QCoreApplication::translate("MainWindow", "Cu&t", nullptr));
#if QT_CONFIG(whatsthis)
        actionCut->setWhatsThis(QCoreApplication::translate("MainWindow", "Copy the selected translation text to the clipboard and deletes it.", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(shortcut)
        actionCut->setShortcut(QCoreApplication::translate("MainWindow", "Ctrl+X", nullptr));
#endif // QT_CONFIG(shortcut)
        actionCopy->setText(QCoreApplication::translate("MainWindow", "&Copy", nullptr));
#if QT_CONFIG(whatsthis)
        actionCopy->setWhatsThis(QCoreApplication::translate("MainWindow", "Copy the selected translation text to the clipboard.", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(shortcut)
        actionCopy->setShortcut(QCoreApplication::translate("MainWindow", "Ctrl+C", nullptr));
#endif // QT_CONFIG(shortcut)
        actionPaste->setText(QCoreApplication::translate("MainWindow", "&Paste", nullptr));
#if QT_CONFIG(whatsthis)
        actionPaste->setWhatsThis(QCoreApplication::translate("MainWindow", "Paste the clipboard text into the translation.", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(shortcut)
        actionPaste->setShortcut(QCoreApplication::translate("MainWindow", "Ctrl+V", nullptr));
#endif // QT_CONFIG(shortcut)
        actionSelectAll->setText(QCoreApplication::translate("MainWindow", "Select &All", nullptr));
#if QT_CONFIG(whatsthis)
        actionSelectAll->setWhatsThis(QCoreApplication::translate("MainWindow", "Select the whole translation text.", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(shortcut)
        actionSelectAll->setShortcut(QCoreApplication::translate("MainWindow", "Ctrl+A", nullptr));
#endif // QT_CONFIG(shortcut)
        actionFind->setText(QCoreApplication::translate("MainWindow", "&Find...", nullptr));
#if QT_CONFIG(whatsthis)
        actionFind->setWhatsThis(QCoreApplication::translate("MainWindow", "Search for some text in the translation source file.", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(shortcut)
        actionFind->setShortcut(QCoreApplication::translate("MainWindow", "Ctrl+F", nullptr));
#endif // QT_CONFIG(shortcut)
        actionFindNext->setText(QCoreApplication::translate("MainWindow", "Find &Next", nullptr));
#if QT_CONFIG(whatsthis)
        actionFindNext->setWhatsThis(QCoreApplication::translate("MainWindow", "Continue the search where it was left.", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(shortcut)
        actionFindNext->setShortcut(QCoreApplication::translate("MainWindow", "F3", nullptr));
#endif // QT_CONFIG(shortcut)
        actionPrevUnfinished->setText(QCoreApplication::translate("MainWindow", "&Prev Unfinished", nullptr));
#if QT_CONFIG(tooltip)
        actionPrevUnfinished->setToolTip(QCoreApplication::translate("MainWindow", "Previous unfinished item", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        actionPrevUnfinished->setWhatsThis(QCoreApplication::translate("MainWindow", "Move to the previous unfinished item.", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(shortcut)
        actionPrevUnfinished->setShortcut(QCoreApplication::translate("MainWindow", "Ctrl+K", nullptr));
#endif // QT_CONFIG(shortcut)
        actionNextUnfinished->setText(QCoreApplication::translate("MainWindow", "&Next Unfinished", nullptr));
#if QT_CONFIG(tooltip)
        actionNextUnfinished->setToolTip(QCoreApplication::translate("MainWindow", "Next unfinished item", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        actionNextUnfinished->setWhatsThis(QCoreApplication::translate("MainWindow", "Move to the next unfinished item.", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(shortcut)
        actionNextUnfinished->setShortcut(QCoreApplication::translate("MainWindow", "Ctrl+J", nullptr));
#endif // QT_CONFIG(shortcut)
        actionPrev->setText(QCoreApplication::translate("MainWindow", "P&rev", nullptr));
#if QT_CONFIG(tooltip)
        actionPrev->setToolTip(QCoreApplication::translate("MainWindow", "Move to previous item", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        actionPrev->setWhatsThis(QCoreApplication::translate("MainWindow", "Move to the previous item.", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(shortcut)
        actionPrev->setShortcut(QCoreApplication::translate("MainWindow", "Ctrl+Shift+K", nullptr));
#endif // QT_CONFIG(shortcut)
        actionNext->setText(QCoreApplication::translate("MainWindow", "Ne&xt", nullptr));
#if QT_CONFIG(tooltip)
        actionNext->setToolTip(QCoreApplication::translate("MainWindow", "Next item", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        actionNext->setWhatsThis(QCoreApplication::translate("MainWindow", "Move to the next item.", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(shortcut)
        actionNext->setShortcut(QCoreApplication::translate("MainWindow", "Ctrl+Shift+J", nullptr));
#endif // QT_CONFIG(shortcut)
        actionDoneAndNext->setText(QCoreApplication::translate("MainWindow", "&Done and Next", nullptr));
#if QT_CONFIG(tooltip)
        actionDoneAndNext->setToolTip(QCoreApplication::translate("MainWindow", "Mark item as done and move to the next unfinished item", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        actionDoneAndNext->setWhatsThis(QCoreApplication::translate("MainWindow", "Mark this item as done and move to the next unfinished item.", nullptr));
#endif // QT_CONFIG(whatsthis)
        actionBeginFromSource->setText(QCoreApplication::translate("MainWindow", "Copy from source text", nullptr));
        actionBeginFromSource->setIconText(QCoreApplication::translate("MainWindow", "Copy from source text", nullptr));
#if QT_CONFIG(tooltip)
        actionBeginFromSource->setToolTip(QCoreApplication::translate("MainWindow", "Copies the source text into the translation field", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        actionBeginFromSource->setWhatsThis(QCoreApplication::translate("MainWindow", "Copies the source text into the translation field.", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(shortcut)
        actionBeginFromSource->setShortcut(QCoreApplication::translate("MainWindow", "Ctrl+B", nullptr));
#endif // QT_CONFIG(shortcut)
        actionAccelerators->setText(QCoreApplication::translate("MainWindow", "&Accelerators", nullptr));
#if QT_CONFIG(tooltip)
        actionAccelerators->setToolTip(QCoreApplication::translate("MainWindow", "Toggles the validity check of accelerators", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        actionAccelerators->setWhatsThis(QCoreApplication::translate("MainWindow", "Toggles the validity check of accelerators, i.e. whether the number of ampersands in the source and translation text is the same. If the check fails, a message is shown in the warnings window.", nullptr));
#endif // QT_CONFIG(whatsthis)
        actionSurroundingWhitespace->setText(QCoreApplication::translate("MainWindow", "Surrounding &Whitespace", nullptr));
#if QT_CONFIG(tooltip)
        actionSurroundingWhitespace->setToolTip(QCoreApplication::translate("MainWindow", "Toggles the validity check of surrounding whitespace.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        actionSurroundingWhitespace->setWhatsThis(QCoreApplication::translate("MainWindow", "Toggles the validity check of surrounding whitespace. If the check fails, a message is shown in the warnings window.", nullptr));
#endif // QT_CONFIG(whatsthis)
        actionEndingPunctuation->setText(QCoreApplication::translate("MainWindow", "&Ending Punctuation", nullptr));
#if QT_CONFIG(tooltip)
        actionEndingPunctuation->setToolTip(QCoreApplication::translate("MainWindow", "Toggles the validity check of ending punctuation", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        actionEndingPunctuation->setWhatsThis(QCoreApplication::translate("MainWindow", "Toggles the validity check of ending punctuation. If the check fails, a message is shown in the warnings window.", nullptr));
#endif // QT_CONFIG(whatsthis)
        actionPhraseMatches->setText(QCoreApplication::translate("MainWindow", "&Phrase matches", nullptr));
#if QT_CONFIG(tooltip)
        actionPhraseMatches->setToolTip(QCoreApplication::translate("MainWindow", "Toggles checking that phrase suggestions are used", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        actionPhraseMatches->setWhatsThis(QCoreApplication::translate("MainWindow", "Toggles checking that phrase suggestions are used. If the check fails, a message is shown in the warnings window.", nullptr));
#endif // QT_CONFIG(whatsthis)
        actionPlaceMarkerMatches->setText(QCoreApplication::translate("MainWindow", "Place &Marker Matches", nullptr));
#if QT_CONFIG(tooltip)
        actionPlaceMarkerMatches->setToolTip(QCoreApplication::translate("MainWindow", "Toggles the validity check of place markers", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        actionPlaceMarkerMatches->setWhatsThis(QCoreApplication::translate("MainWindow", "Toggles the validity check of place markers, i.e. whether %1, %2, ... are used consistently in the source text and translation text. If the check fails, a message is shown in the warnings window.", nullptr));
#endif // QT_CONFIG(whatsthis)
        actionNewPhraseBook->setText(QCoreApplication::translate("MainWindow", "&New Phrase Book...", nullptr));
#if QT_CONFIG(whatsthis)
        actionNewPhraseBook->setWhatsThis(QCoreApplication::translate("MainWindow", "Create a new phrase book.", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(shortcut)
        actionNewPhraseBook->setShortcut(QCoreApplication::translate("MainWindow", "Ctrl+N", nullptr));
#endif // QT_CONFIG(shortcut)
        actionOpenPhraseBook->setText(QCoreApplication::translate("MainWindow", "&Open Phrase Book...", nullptr));
#if QT_CONFIG(whatsthis)
        actionOpenPhraseBook->setWhatsThis(QCoreApplication::translate("MainWindow", "Open a phrase book to assist translation.", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(shortcut)
        actionOpenPhraseBook->setShortcut(QCoreApplication::translate("MainWindow", "Ctrl+H", nullptr));
#endif // QT_CONFIG(shortcut)
        actionResetSorting->setText(QCoreApplication::translate("MainWindow", "&Reset Sorting", nullptr));
#if QT_CONFIG(whatsthis)
        actionResetSorting->setWhatsThis(QCoreApplication::translate("MainWindow", "Sort the items back in the same order as in the message file.", nullptr));
#endif // QT_CONFIG(whatsthis)
        actionDisplayGuesses->setText(QCoreApplication::translate("MainWindow", "&Display guesses", nullptr));
#if QT_CONFIG(whatsthis)
        actionDisplayGuesses->setWhatsThis(QCoreApplication::translate("MainWindow", "Set whether or not to display translation guesses.", nullptr));
#endif // QT_CONFIG(whatsthis)
        actionStatistics->setText(QCoreApplication::translate("MainWindow", "&Statistics", nullptr));
#if QT_CONFIG(whatsthis)
        actionStatistics->setWhatsThis(QCoreApplication::translate("MainWindow", "Display translation statistics.", nullptr));
#endif // QT_CONFIG(whatsthis)
        actionQmlPreview->setText(QCoreApplication::translate("MainWindow", "&QML preview", nullptr));
#if QT_CONFIG(tooltip)
        actionQmlPreview->setToolTip(QCoreApplication::translate("MainWindow", "Displays a preview of QML documents.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        actionQmlPreview->setWhatsThis(QString());
#endif // QT_CONFIG(whatsthis)
        actionManual->setText(QCoreApplication::translate("MainWindow", "&Manual", nullptr));
#if QT_CONFIG(whatsthis)
        actionManual->setWhatsThis(QString());
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(shortcut)
        actionManual->setShortcut(QCoreApplication::translate("MainWindow", "F1", nullptr));
#endif // QT_CONFIG(shortcut)
        actionAbout->setText(QCoreApplication::translate("MainWindow", "About Qt Linguist", nullptr));
        actionAboutQt->setText(QCoreApplication::translate("MainWindow", "About Qt", nullptr));
#if QT_CONFIG(whatsthis)
        actionAboutQt->setWhatsThis(QCoreApplication::translate("MainWindow", "Display information about the Qt toolkit by Digia.", nullptr));
#endif // QT_CONFIG(whatsthis)
        actionWhatsThis->setText(QCoreApplication::translate("MainWindow", "&What's This?", nullptr));
        actionWhatsThis->setIconText(QCoreApplication::translate("MainWindow", "What's This?", nullptr));
#if QT_CONFIG(tooltip)
        actionWhatsThis->setToolTip(QCoreApplication::translate("MainWindow", "What's This?", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        actionWhatsThis->setWhatsThis(QCoreApplication::translate("MainWindow", "Enter What's This? mode.", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(shortcut)
        actionWhatsThis->setShortcut(QCoreApplication::translate("MainWindow", "Shift+F1", nullptr));
#endif // QT_CONFIG(shortcut)
        actionSearchAndTranslate->setText(QCoreApplication::translate("MainWindow", "&Search And Translate...", nullptr));
#if QT_CONFIG(whatsthis)
        actionSearchAndTranslate->setWhatsThis(QCoreApplication::translate("MainWindow", "Replace the translation on all entries that matches the search source text.", nullptr));
#endif // QT_CONFIG(whatsthis)
        actionBatchTranslation->setText(QCoreApplication::translate("MainWindow", "&Batch Translation...", nullptr));
#if QT_CONFIG(whatsthis)
        actionBatchTranslation->setWhatsThis(QCoreApplication::translate("MainWindow", "Batch translate all entries using the information in the phrase books.", nullptr));
#endif // QT_CONFIG(whatsthis)
        actionReleaseAs->setText(QCoreApplication::translate("MainWindow", "Release As...", nullptr));
        actionReleaseAs->setIconText(QCoreApplication::translate("MainWindow", "Release As...", nullptr));
#if QT_CONFIG(whatsthis)
        actionReleaseAs->setWhatsThis(QCoreApplication::translate("MainWindow", "Create a Qt message file suitable for released applications from the current message file. The filename will automatically be determined from the name of the TS file.", nullptr));
#endif // QT_CONFIG(whatsthis)
        actionFile->setText(QCoreApplication::translate("MainWindow", "File", nullptr));
        actionEdit->setText(QCoreApplication::translate("MainWindow", "Edit", nullptr));
        actionTranslation->setText(QCoreApplication::translate("MainWindow", "Translation", nullptr));
        actionValidation->setText(QCoreApplication::translate("MainWindow", "Validation", nullptr));
        actionHelp->setText(QCoreApplication::translate("MainWindow", "Help", nullptr));
        actionPreviewForm->setText(QCoreApplication::translate("MainWindow", "Open/Refresh Form &Preview", nullptr));
        actionPreviewForm->setIconText(QCoreApplication::translate("MainWindow", "Form Preview Tool", nullptr));
#if QT_CONFIG(tooltip)
        actionPreviewForm->setToolTip(QCoreApplication::translate("MainWindow", "Form Preview Tool", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(shortcut)
        actionPreviewForm->setShortcut(QCoreApplication::translate("MainWindow", "F5", nullptr));
#endif // QT_CONFIG(shortcut)
        actionTranslationFileSettings->setText(QCoreApplication::translate("MainWindow", "Translation File &Settings...", nullptr));
        actionAddToPhraseBook->setText(QCoreApplication::translate("MainWindow", "&Add to Phrase Book", nullptr));
#if QT_CONFIG(shortcut)
        actionAddToPhraseBook->setShortcut(QCoreApplication::translate("MainWindow", "Ctrl+T", nullptr));
#endif // QT_CONFIG(shortcut)
        actionOpenAux->setText(QCoreApplication::translate("MainWindow", "Open Read-O&nly...", nullptr));
        actionSaveAll->setText(QCoreApplication::translate("MainWindow", "&Save All", nullptr));
#if QT_CONFIG(shortcut)
        actionSaveAll->setShortcut(QCoreApplication::translate("MainWindow", "Ctrl+S", nullptr));
#endif // QT_CONFIG(shortcut)
        actionReleaseAll->setText(QCoreApplication::translate("MainWindow", "&Release All", nullptr));
        actionClose->setText(QCoreApplication::translate("MainWindow", "Close", nullptr));
        actionCloseAll->setText(QCoreApplication::translate("MainWindow", "&Close All", nullptr));
#if QT_CONFIG(shortcut)
        actionCloseAll->setShortcut(QCoreApplication::translate("MainWindow", "Ctrl+W", nullptr));
#endif // QT_CONFIG(shortcut)
        actionLengthVariants->setText(QCoreApplication::translate("MainWindow", "Length Variants", nullptr));
        actionVisualizeWhitespace->setText(QCoreApplication::translate("MainWindow", "Visualize whitespace", nullptr));
#if QT_CONFIG(tooltip)
        actionVisualizeWhitespace->setToolTip(QCoreApplication::translate("MainWindow", "Toggles visualize whitespace in editors", nullptr));
#endif // QT_CONFIG(tooltip)
        actionIncreaseZoom->setText(QCoreApplication::translate("MainWindow", "Increase", nullptr));
#if QT_CONFIG(shortcut)
        actionIncreaseZoom->setShortcut(QCoreApplication::translate("MainWindow", "Ctrl++", nullptr));
#endif // QT_CONFIG(shortcut)
        actionDecreaseZoom->setText(QCoreApplication::translate("MainWindow", "Decrease", nullptr));
#if QT_CONFIG(shortcut)
        actionDecreaseZoom->setShortcut(QCoreApplication::translate("MainWindow", "Ctrl+-", nullptr));
#endif // QT_CONFIG(shortcut)
        actionResetZoomToDefault->setText(QCoreApplication::translate("MainWindow", "Reset to default", nullptr));
#if QT_CONFIG(shortcut)
        actionResetZoomToDefault->setShortcut(QCoreApplication::translate("MainWindow", "Ctrl+0", nullptr));
#endif // QT_CONFIG(shortcut)
        actionShowMoreGuesses->setText(QCoreApplication::translate("MainWindow", "Show more", nullptr));
#if QT_CONFIG(shortcut)
        actionShowMoreGuesses->setShortcut(QCoreApplication::translate("MainWindow", "Alt++", nullptr));
#endif // QT_CONFIG(shortcut)
        actionShowFewerGuesses->setText(QCoreApplication::translate("MainWindow", "Show fewer", nullptr));
#if QT_CONFIG(shortcut)
        actionShowFewerGuesses->setShortcut(QCoreApplication::translate("MainWindow", "Alt+-", nullptr));
#endif // QT_CONFIG(shortcut)
        actionResetGuessesToDefault->setText(QCoreApplication::translate("MainWindow", "Reset to default", nullptr));
#if QT_CONFIG(shortcut)
        actionResetGuessesToDefault->setShortcut(QCoreApplication::translate("MainWindow", "Alt+0", nullptr));
#endif // QT_CONFIG(shortcut)
        actionDone->setText(QCoreApplication::translate("MainWindow", "D&one", nullptr));
#if QT_CONFIG(tooltip)
        actionDone->setToolTip(QCoreApplication::translate("MainWindow", "Mark item as done", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        actionDone->setWhatsThis(QCoreApplication::translate("MainWindow", "Mark this item as done.", nullptr));
#endif // QT_CONFIG(whatsthis)
        actionFindPrev->setText(QCoreApplication::translate("MainWindow", "Find P&revious", nullptr));
#if QT_CONFIG(shortcut)
        actionFindPrev->setShortcut(QCoreApplication::translate("MainWindow", "Shift+F3", nullptr));
#endif // QT_CONFIG(shortcut)
        actionAuto_Translation->setText(QCoreApplication::translate("MainWindow", "AI Translation...", nullptr));
        menuPhrases->setTitle(QCoreApplication::translate("MainWindow", "&Phrases", nullptr));
        menuClosePhraseBook->setTitle(QCoreApplication::translate("MainWindow", "&Close Phrase Book", nullptr));
        menuEditPhraseBook->setTitle(QCoreApplication::translate("MainWindow", "&Edit Phrase Book", nullptr));
        menuPrintPhraseBook->setTitle(QCoreApplication::translate("MainWindow", "&Print Phrase Book", nullptr));
        menuValidation->setTitle(QCoreApplication::translate("MainWindow", "V&alidation", nullptr));
        menuView->setTitle(QCoreApplication::translate("MainWindow", "&View", nullptr));
        menuViewViews->setTitle(QCoreApplication::translate("MainWindow", "Vie&ws", nullptr));
        menuToolbars->setTitle(QCoreApplication::translate("MainWindow", "&Toolbars", nullptr));
        menuZoom->setTitle(QCoreApplication::translate("MainWindow", "&Zoom", nullptr));
        menuGuesses->setTitle(QCoreApplication::translate("MainWindow", "Guesses", nullptr));
        menuHelp->setTitle(QCoreApplication::translate("MainWindow", "&Help", nullptr));
        menuTranslation->setTitle(QCoreApplication::translate("MainWindow", "&Translation", nullptr));
        menuFile->setTitle(QCoreApplication::translate("MainWindow", "&File", nullptr));
        menuRecentlyOpenedFiles->setTitle(QCoreApplication::translate("MainWindow", "Recently Opened &Files", nullptr));
        menuEdit->setTitle(QCoreApplication::translate("MainWindow", "&Edit", nullptr));
    } // retranslateUi

};

namespace Ui {
    class MainWindow: public Ui_MainWindow {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_MAINWINDOW_H
