/****************************************************************************
** Meta object code from reading C++ file 'abstractformwindowmanager.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../../qttools/src/designer/src/lib/sdk/abstractformwindowmanager.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'abstractformwindowmanager.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN35QDesignerFormWindowManagerInterfaceE_t {};
} // unnamed namespace

template <> constexpr inline auto QDesignerFormWindowManagerInterface::qt_create_metaobjectdata<qt_meta_tag_ZN35QDesignerFormWindowManagerInterfaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QDesignerFormWindowManagerInterface",
        "formWindowAdded",
        "",
        "QDesignerFormWindowInterface*",
        "formWindow",
        "formWindowRemoved",
        "activeFormWindowChanged",
        "formWindowSettingsChanged",
        "fw",
        "addFormWindow",
        "removeFormWindow",
        "setActiveFormWindow",
        "showPreview",
        "closeAllPreviews",
        "showPluginDialog",
        "Action",
        "CutAction",
        "CopyAction",
        "PasteAction",
        "DeleteAction",
        "SelectAllAction",
        "LowerAction",
        "RaiseAction",
        "UndoAction",
        "RedoAction",
        "HorizontalLayoutAction",
        "VerticalLayoutAction",
        "SplitHorizontalAction",
        "SplitVerticalAction",
        "GridLayoutAction",
        "FormLayoutAction",
        "BreakLayoutAction",
        "AdjustSizeAction",
        "SimplifyLayoutAction",
        "DefaultPreviewAction",
        "FormWindowSettingsDialogAction",
        "ActionGroup",
        "StyledPreviewActionGroup"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'formWindowAdded'
        QtMocHelpers::SignalData<void(QDesignerFormWindowInterface *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'formWindowRemoved'
        QtMocHelpers::SignalData<void(QDesignerFormWindowInterface *)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'activeFormWindowChanged'
        QtMocHelpers::SignalData<void(QDesignerFormWindowInterface *)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'formWindowSettingsChanged'
        QtMocHelpers::SignalData<void(QDesignerFormWindowInterface *)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 8 },
        }}),
        // Slot 'addFormWindow'
        QtMocHelpers::SlotData<void(QDesignerFormWindowInterface *)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'removeFormWindow'
        QtMocHelpers::SlotData<void(QDesignerFormWindowInterface *)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'setActiveFormWindow'
        QtMocHelpers::SlotData<void(QDesignerFormWindowInterface *)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'showPreview'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'closeAllPreviews'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showPluginDialog'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Action'
        QtMocHelpers::EnumData<enum Action>(15, 15, QMC::EnumFlags{}).add({
            {   16, Action::CutAction },
            {   17, Action::CopyAction },
            {   18, Action::PasteAction },
            {   19, Action::DeleteAction },
            {   20, Action::SelectAllAction },
            {   21, Action::LowerAction },
            {   22, Action::RaiseAction },
            {   23, Action::UndoAction },
            {   24, Action::RedoAction },
            {   25, Action::HorizontalLayoutAction },
            {   26, Action::VerticalLayoutAction },
            {   27, Action::SplitHorizontalAction },
            {   28, Action::SplitVerticalAction },
            {   29, Action::GridLayoutAction },
            {   30, Action::FormLayoutAction },
            {   31, Action::BreakLayoutAction },
            {   32, Action::AdjustSizeAction },
            {   33, Action::SimplifyLayoutAction },
            {   34, Action::DefaultPreviewAction },
            {   35, Action::FormWindowSettingsDialogAction },
        }),
        // enum 'ActionGroup'
        QtMocHelpers::EnumData<enum ActionGroup>(36, 36, QMC::EnumFlags{}).add({
            {   37, ActionGroup::StyledPreviewActionGroup },
        }),
    };
    return QtMocHelpers::metaObjectData<QDesignerFormWindowManagerInterface, qt_meta_tag_ZN35QDesignerFormWindowManagerInterfaceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QDesignerFormWindowManagerInterface::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN35QDesignerFormWindowManagerInterfaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN35QDesignerFormWindowManagerInterfaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN35QDesignerFormWindowManagerInterfaceE_t>.metaTypes,
    nullptr
} };

void QDesignerFormWindowManagerInterface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QDesignerFormWindowManagerInterface *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->formWindowAdded((*reinterpret_cast<std::add_pointer_t<QDesignerFormWindowInterface*>>(_a[1]))); break;
        case 1: _t->formWindowRemoved((*reinterpret_cast<std::add_pointer_t<QDesignerFormWindowInterface*>>(_a[1]))); break;
        case 2: _t->activeFormWindowChanged((*reinterpret_cast<std::add_pointer_t<QDesignerFormWindowInterface*>>(_a[1]))); break;
        case 3: _t->formWindowSettingsChanged((*reinterpret_cast<std::add_pointer_t<QDesignerFormWindowInterface*>>(_a[1]))); break;
        case 4: _t->addFormWindow((*reinterpret_cast<std::add_pointer_t<QDesignerFormWindowInterface*>>(_a[1]))); break;
        case 5: _t->removeFormWindow((*reinterpret_cast<std::add_pointer_t<QDesignerFormWindowInterface*>>(_a[1]))); break;
        case 6: _t->setActiveFormWindow((*reinterpret_cast<std::add_pointer_t<QDesignerFormWindowInterface*>>(_a[1]))); break;
        case 7: _t->showPreview(); break;
        case 8: _t->closeAllPreviews(); break;
        case 9: _t->showPluginDialog(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QDesignerFormWindowInterface* >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QDesignerFormWindowInterface* >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QDesignerFormWindowInterface* >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QDesignerFormWindowInterface* >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QDesignerFormWindowInterface* >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QDesignerFormWindowInterface* >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QDesignerFormWindowInterface* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QDesignerFormWindowManagerInterface::*)(QDesignerFormWindowInterface * )>(_a, &QDesignerFormWindowManagerInterface::formWindowAdded, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDesignerFormWindowManagerInterface::*)(QDesignerFormWindowInterface * )>(_a, &QDesignerFormWindowManagerInterface::formWindowRemoved, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDesignerFormWindowManagerInterface::*)(QDesignerFormWindowInterface * )>(_a, &QDesignerFormWindowManagerInterface::activeFormWindowChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDesignerFormWindowManagerInterface::*)(QDesignerFormWindowInterface * )>(_a, &QDesignerFormWindowManagerInterface::formWindowSettingsChanged, 3))
            return;
    }
}

const QMetaObject *QDesignerFormWindowManagerInterface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QDesignerFormWindowManagerInterface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN35QDesignerFormWindowManagerInterfaceE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QDesignerFormWindowManagerInterface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    return _id;
}

// SIGNAL 0
void QDesignerFormWindowManagerInterface::formWindowAdded(QDesignerFormWindowInterface * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void QDesignerFormWindowManagerInterface::formWindowRemoved(QDesignerFormWindowInterface * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void QDesignerFormWindowManagerInterface::activeFormWindowChanged(QDesignerFormWindowInterface * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void QDesignerFormWindowManagerInterface::formWindowSettingsChanged(QDesignerFormWindowInterface * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}
QT_WARNING_POP
