/****************************************************************************
** Meta object code from reading C++ file 'messageeditor.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../qttools/src/linguist/linguist/messageeditor.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'messageeditor.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13MessageEditorE_t {};
} // unnamed namespace

template <> constexpr inline auto MessageEditor::qt_create_metaobjectdata<qt_meta_tag_ZN13MessageEditorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "MessageEditor",
        "translationChanged",
        "",
        "translations",
        "translatorCommentChanged",
        "comment",
        "activeModelChanged",
        "model",
        "undoAvailable",
        "avail",
        "redoAvailable",
        "cutAvailable",
        "copyAvailable",
        "pasteAvailable",
        "undo",
        "redo",
        "cut",
        "copy",
        "paste",
        "selectAll",
        "beginFromSource",
        "setEditorFocus",
        "setTranslation",
        "latestModel",
        "translation",
        "setLengthVariants",
        "on",
        "increaseFontSize",
        "decreaseFontSize",
        "resetFontSize",
        "editorCreated",
        "QTextEdit*",
        "editorDestroyed",
        "selectionChanged",
        "toggleNcrMode",
        "resetHoverSelection",
        "emitTranslationChanged",
        "emitTranslatorCommentChanged",
        "updateCanPaste",
        "clipboardChanged",
        "messageModelAppended",
        "messageModelDeleted",
        "allModelsDeleted",
        "setTargetLanguage",
        "reallyFixTabOrder"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'translationChanged'
        QtMocHelpers::SignalData<void(const QStringList &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 3 },
        }}),
        // Signal 'translatorCommentChanged'
        QtMocHelpers::SignalData<void(const QString &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 5 },
        }}),
        // Signal 'activeModelChanged'
        QtMocHelpers::SignalData<void(int)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 7 },
        }}),
        // Signal 'undoAvailable'
        QtMocHelpers::SignalData<void(bool)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 9 },
        }}),
        // Signal 'redoAvailable'
        QtMocHelpers::SignalData<void(bool)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 9 },
        }}),
        // Signal 'cutAvailable'
        QtMocHelpers::SignalData<void(bool)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 9 },
        }}),
        // Signal 'copyAvailable'
        QtMocHelpers::SignalData<void(bool)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 9 },
        }}),
        // Signal 'pasteAvailable'
        QtMocHelpers::SignalData<void(bool)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 9 },
        }}),
        // Slot 'undo'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'redo'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'cut'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'copy'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'paste'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'selectAll'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'beginFromSource'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setEditorFocus'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setTranslation'
        QtMocHelpers::SlotData<void(int, const QString &)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 23 }, { QMetaType::QString, 24 },
        }}),
        // Slot 'setLengthVariants'
        QtMocHelpers::SlotData<void(bool)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 26 },
        }}),
        // Slot 'increaseFontSize'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'decreaseFontSize'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'resetFontSize'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'editorCreated'
        QtMocHelpers::SlotData<void(QTextEdit *)>(30, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 31, 2 },
        }}),
        // Slot 'editorDestroyed'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'selectionChanged'
        QtMocHelpers::SlotData<void(QTextEdit *)>(33, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 31, 2 },
        }}),
        // Slot 'toggleNcrMode'
        QtMocHelpers::SlotData<void()>(34, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'resetHoverSelection'
        QtMocHelpers::SlotData<void()>(35, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'emitTranslationChanged'
        QtMocHelpers::SlotData<void(QTextEdit *)>(36, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 31, 2 },
        }}),
        // Slot 'emitTranslatorCommentChanged'
        QtMocHelpers::SlotData<void(QTextEdit *)>(37, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 31, 2 },
        }}),
        // Slot 'updateCanPaste'
        QtMocHelpers::SlotData<void()>(38, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'clipboardChanged'
        QtMocHelpers::SlotData<void()>(39, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'messageModelAppended'
        QtMocHelpers::SlotData<void()>(40, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'messageModelDeleted'
        QtMocHelpers::SlotData<void(int)>(41, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 7 },
        }}),
        // Slot 'allModelsDeleted'
        QtMocHelpers::SlotData<void()>(42, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'setTargetLanguage'
        QtMocHelpers::SlotData<void(int)>(43, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 7 },
        }}),
        // Slot 'reallyFixTabOrder'
        QtMocHelpers::SlotData<void()>(44, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<MessageEditor, qt_meta_tag_ZN13MessageEditorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject MessageEditor::staticMetaObject = { {
    QMetaObject::SuperData::link<QScrollArea::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13MessageEditorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13MessageEditorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13MessageEditorE_t>.metaTypes,
    nullptr
} };

void MessageEditor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<MessageEditor *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->translationChanged((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 1: _t->translatorCommentChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->activeModelChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 3: _t->undoAvailable((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 4: _t->redoAvailable((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 5: _t->cutAvailable((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 6: _t->copyAvailable((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 7: _t->pasteAvailable((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 8: _t->undo(); break;
        case 9: _t->redo(); break;
        case 10: _t->cut(); break;
        case 11: _t->copy(); break;
        case 12: _t->paste(); break;
        case 13: _t->selectAll(); break;
        case 14: _t->beginFromSource(); break;
        case 15: _t->setEditorFocus(); break;
        case 16: _t->setTranslation((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 17: _t->setLengthVariants((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 18: _t->increaseFontSize(); break;
        case 19: _t->decreaseFontSize(); break;
        case 20: _t->resetFontSize(); break;
        case 21: _t->editorCreated((*reinterpret_cast<std::add_pointer_t<QTextEdit*>>(_a[1]))); break;
        case 22: _t->editorDestroyed(); break;
        case 23: _t->selectionChanged((*reinterpret_cast<std::add_pointer_t<QTextEdit*>>(_a[1]))); break;
        case 24: _t->toggleNcrMode(); break;
        case 25: _t->resetHoverSelection(); break;
        case 26: _t->emitTranslationChanged((*reinterpret_cast<std::add_pointer_t<QTextEdit*>>(_a[1]))); break;
        case 27: _t->emitTranslatorCommentChanged((*reinterpret_cast<std::add_pointer_t<QTextEdit*>>(_a[1]))); break;
        case 28: _t->updateCanPaste(); break;
        case 29: _t->clipboardChanged(); break;
        case 30: _t->messageModelAppended(); break;
        case 31: _t->messageModelDeleted((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 32: _t->allModelsDeleted(); break;
        case 33: _t->setTargetLanguage((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 34: _t->reallyFixTabOrder(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (MessageEditor::*)(const QStringList & )>(_a, &MessageEditor::translationChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (MessageEditor::*)(const QString & )>(_a, &MessageEditor::translatorCommentChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (MessageEditor::*)(int )>(_a, &MessageEditor::activeModelChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (MessageEditor::*)(bool )>(_a, &MessageEditor::undoAvailable, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (MessageEditor::*)(bool )>(_a, &MessageEditor::redoAvailable, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (MessageEditor::*)(bool )>(_a, &MessageEditor::cutAvailable, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (MessageEditor::*)(bool )>(_a, &MessageEditor::copyAvailable, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (MessageEditor::*)(bool )>(_a, &MessageEditor::pasteAvailable, 7))
            return;
    }
}

const QMetaObject *MessageEditor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MessageEditor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13MessageEditorE_t>.strings))
        return static_cast<void*>(this);
    return QScrollArea::qt_metacast(_clname);
}

int MessageEditor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QScrollArea::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 35)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 35;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 35)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 35;
    }
    return _id;
}

// SIGNAL 0
void MessageEditor::translationChanged(const QStringList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void MessageEditor::translatorCommentChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void MessageEditor::activeModelChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void MessageEditor::undoAvailable(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void MessageEditor::redoAvailable(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void MessageEditor::cutAvailable(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void MessageEditor::copyAvailable(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void MessageEditor::pasteAvailable(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}
QT_WARNING_POP
