/****************************************************************************
** Meta object code from reading C++ file 'qtgradientstopsmodel_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../../qttools/src/shared/qtgradienteditor/qtgradientstopsmodel_p.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qtgradientstopsmodel_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN20QtGradientStopsModelE_t {};
} // unnamed namespace

template <> constexpr inline auto QtGradientStopsModel::qt_create_metaobjectdata<qt_meta_tag_ZN20QtGradientStopsModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QtGradientStopsModel",
        "stopAdded",
        "",
        "QtGradientStop*",
        "stop",
        "stopRemoved",
        "stopMoved",
        "newPos",
        "stopsSwapped",
        "stop1",
        "stop2",
        "stopChanged",
        "QColor",
        "newColor",
        "stopSelected",
        "selected",
        "currentStopChanged"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'stopAdded'
        QtMocHelpers::SignalData<void(QtGradientStop *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'stopRemoved'
        QtMocHelpers::SignalData<void(QtGradientStop *)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'stopMoved'
        QtMocHelpers::SignalData<void(QtGradientStop *, qreal)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::QReal, 7 },
        }}),
        // Signal 'stopsSwapped'
        QtMocHelpers::SignalData<void(QtGradientStop *, QtGradientStop *)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 9 }, { 0x80000000 | 3, 10 },
        }}),
        // Signal 'stopChanged'
        QtMocHelpers::SignalData<void(QtGradientStop *, const QColor &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 12, 13 },
        }}),
        // Signal 'stopSelected'
        QtMocHelpers::SignalData<void(QtGradientStop *, bool)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::Bool, 15 },
        }}),
        // Signal 'currentStopChanged'
        QtMocHelpers::SignalData<void(QtGradientStop *)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QtGradientStopsModel, qt_meta_tag_ZN20QtGradientStopsModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QtGradientStopsModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20QtGradientStopsModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20QtGradientStopsModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN20QtGradientStopsModelE_t>.metaTypes,
    nullptr
} };

void QtGradientStopsModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QtGradientStopsModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->stopAdded((*reinterpret_cast<std::add_pointer_t<QtGradientStop*>>(_a[1]))); break;
        case 1: _t->stopRemoved((*reinterpret_cast<std::add_pointer_t<QtGradientStop*>>(_a[1]))); break;
        case 2: _t->stopMoved((*reinterpret_cast<std::add_pointer_t<QtGradientStop*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qreal>>(_a[2]))); break;
        case 3: _t->stopsSwapped((*reinterpret_cast<std::add_pointer_t<QtGradientStop*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QtGradientStop*>>(_a[2]))); break;
        case 4: _t->stopChanged((*reinterpret_cast<std::add_pointer_t<QtGradientStop*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QColor>>(_a[2]))); break;
        case 5: _t->stopSelected((*reinterpret_cast<std::add_pointer_t<QtGradientStop*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 6: _t->currentStopChanged((*reinterpret_cast<std::add_pointer_t<QtGradientStop*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QtGradientStopsModel::*)(QtGradientStop * )>(_a, &QtGradientStopsModel::stopAdded, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QtGradientStopsModel::*)(QtGradientStop * )>(_a, &QtGradientStopsModel::stopRemoved, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QtGradientStopsModel::*)(QtGradientStop * , qreal )>(_a, &QtGradientStopsModel::stopMoved, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QtGradientStopsModel::*)(QtGradientStop * , QtGradientStop * )>(_a, &QtGradientStopsModel::stopsSwapped, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QtGradientStopsModel::*)(QtGradientStop * , const QColor & )>(_a, &QtGradientStopsModel::stopChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QtGradientStopsModel::*)(QtGradientStop * , bool )>(_a, &QtGradientStopsModel::stopSelected, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (QtGradientStopsModel::*)(QtGradientStop * )>(_a, &QtGradientStopsModel::currentStopChanged, 6))
            return;
    }
}

const QMetaObject *QtGradientStopsModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QtGradientStopsModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20QtGradientStopsModelE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QtGradientStopsModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void QtGradientStopsModel::stopAdded(QtGradientStop * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void QtGradientStopsModel::stopRemoved(QtGradientStop * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void QtGradientStopsModel::stopMoved(QtGradientStop * _t1, qreal _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void QtGradientStopsModel::stopsSwapped(QtGradientStop * _t1, QtGradientStop * _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 4
void QtGradientStopsModel::stopChanged(QtGradientStop * _t1, const QColor & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2);
}

// SIGNAL 5
void QtGradientStopsModel::stopSelected(QtGradientStop * _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2);
}

// SIGNAL 6
void QtGradientStopsModel::currentStopChanged(QtGradientStop * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}
QT_WARNING_POP
