/********************************************************************************
** Form generated from reading UI file 'helpdocsettingswidget.ui'
**
** Created by: Qt User Interface Compiler version 6.12.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_HELPDOCSETTINGSWIDGET_H
#define UI_HELPDOCSETTINGSWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_HelpDocSettingsWidget
{
public:
    QGridLayout *gridLayout;
    QLabel *docLabel;
    QLineEdit *registeredDocsFilterLineEdit;
    QVBoxLayout *buttonLayout;
    QPushButton *docAddButton;
    QPushButton *docRemoveButton;
    QSpacerItem *buttonSpacer;
    QListWidget *registeredDocsListWidget;

    void setupUi(QWidget *HelpDocSettingsWidget)
    {
        if (HelpDocSettingsWidget->objectName().isEmpty())
            HelpDocSettingsWidget->setObjectName("HelpDocSettingsWidget");
        HelpDocSettingsWidget->resize(268, 128);
        gridLayout = new QGridLayout(HelpDocSettingsWidget);
        gridLayout->setObjectName("gridLayout");
        docLabel = new QLabel(HelpDocSettingsWidget);
        docLabel->setObjectName("docLabel");

        gridLayout->addWidget(docLabel, 0, 0, 1, 1);

        registeredDocsFilterLineEdit = new QLineEdit(HelpDocSettingsWidget);
        registeredDocsFilterLineEdit->setObjectName("registeredDocsFilterLineEdit");
        registeredDocsFilterLineEdit->setClearButtonEnabled(true);

        gridLayout->addWidget(registeredDocsFilterLineEdit, 1, 0, 1, 1);

        buttonLayout = new QVBoxLayout();
        buttonLayout->setObjectName("buttonLayout");
        docAddButton = new QPushButton(HelpDocSettingsWidget);
        docAddButton->setObjectName("docAddButton");

        buttonLayout->addWidget(docAddButton);

        docRemoveButton = new QPushButton(HelpDocSettingsWidget);
        docRemoveButton->setObjectName("docRemoveButton");

        buttonLayout->addWidget(docRemoveButton);

        buttonSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        buttonLayout->addItem(buttonSpacer);


        gridLayout->addLayout(buttonLayout, 1, 1, 2, 1);

        registeredDocsListWidget = new QListWidget(HelpDocSettingsWidget);
        registeredDocsListWidget->setObjectName("registeredDocsListWidget");
        registeredDocsListWidget->setSelectionMode(QAbstractItemView::ExtendedSelection);

        gridLayout->addWidget(registeredDocsListWidget, 2, 0, 1, 1);


        retranslateUi(HelpDocSettingsWidget);

        QMetaObject::connectSlotsByName(HelpDocSettingsWidget);
    } // setupUi

    void retranslateUi(QWidget *HelpDocSettingsWidget)
    {
        HelpDocSettingsWidget->setWindowTitle(QCoreApplication::translate("HelpDocSettingsWidget", "Form", nullptr));
        docLabel->setText(QCoreApplication::translate("HelpDocSettingsWidget", "Registered Documentation", nullptr));
        registeredDocsFilterLineEdit->setPlaceholderText(QCoreApplication::translate("HelpDocSettingsWidget", "<Filter>", nullptr));
        docAddButton->setText(QCoreApplication::translate("HelpDocSettingsWidget", "Add...", nullptr));
        docRemoveButton->setText(QCoreApplication::translate("HelpDocSettingsWidget", "Remove", nullptr));
    } // retranslateUi

};

namespace Ui {
    class HelpDocSettingsWidget: public Ui_HelpDocSettingsWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_HELPDOCSETTINGSWIDGET_H
