/****************************************************************************
** Meta object code from reading C++ file 'qhelpindexwidget.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../qttools/src/assistant/help/qhelpindexwidget.h"
#include <QtHelp/qhelplink.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qhelpindexwidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15QHelpIndexModelE_t {};
} // unnamed namespace

template <> constexpr inline auto QHelpIndexModel::qt_create_metaobjectdata<qt_meta_tag_ZN15QHelpIndexModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QHelpIndexModel",
        "indexCreationStarted",
        "",
        "indexCreated",
        "insertIndices"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'indexCreationStarted'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'indexCreated'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'insertIndices'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QHelpIndexModel, qt_meta_tag_ZN15QHelpIndexModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QHelpIndexModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QStringListModel::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15QHelpIndexModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15QHelpIndexModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15QHelpIndexModelE_t>.metaTypes,
    nullptr
} };

void QHelpIndexModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QHelpIndexModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->indexCreationStarted(); break;
        case 1: _t->indexCreated(); break;
        case 2: _t->insertIndices(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QHelpIndexModel::*)()>(_a, &QHelpIndexModel::indexCreationStarted, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QHelpIndexModel::*)()>(_a, &QHelpIndexModel::indexCreated, 1))
            return;
    }
}

const QMetaObject *QHelpIndexModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QHelpIndexModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15QHelpIndexModelE_t>.strings))
        return static_cast<void*>(this);
    return QStringListModel::qt_metacast(_clname);
}

int QHelpIndexModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QStringListModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void QHelpIndexModel::indexCreationStarted()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QHelpIndexModel::indexCreated()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
namespace {
struct qt_meta_tag_ZN16QHelpIndexWidgetE_t {};
} // unnamed namespace

template <> constexpr inline auto QHelpIndexWidget::qt_create_metaobjectdata<qt_meta_tag_ZN16QHelpIndexWidgetE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QHelpIndexWidget",
        "linkActivated",
        "",
        "QUrl",
        "link",
        "keyword",
        "linksActivated",
        "QMultiMap<QString,QUrl>",
        "links",
        "documentActivated",
        "QHelpLink",
        "document",
        "documentsActivated",
        "QList<QHelpLink>",
        "documents",
        "filterIndices",
        "filter",
        "wildcard",
        "activateCurrentItem",
        "showLink",
        "QModelIndex",
        "index"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'linkActivated'
        QtMocHelpers::SignalData<void(const QUrl &, const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::QString, 5 },
        }}),
        // Signal 'linksActivated'
        QtMocHelpers::SignalData<void(const QMultiMap<QString,QUrl> &, const QString &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 }, { QMetaType::QString, 5 },
        }}),
        // Signal 'documentActivated'
        QtMocHelpers::SignalData<void(const QHelpLink &, const QString &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 }, { QMetaType::QString, 5 },
        }}),
        // Signal 'documentsActivated'
        QtMocHelpers::SignalData<void(const QList<QHelpLink> &, const QString &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 13, 14 }, { QMetaType::QString, 5 },
        }}),
        // Slot 'filterIndices'
        QtMocHelpers::SlotData<void(const QString &, const QString &)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 16 }, { QMetaType::QString, 17 },
        }}),
        // Slot 'filterIndices'
        QtMocHelpers::SlotData<void(const QString &)>(15, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 16 },
        }}),
        // Slot 'activateCurrentItem'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showLink'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(19, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 20, 21 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QHelpIndexWidget, qt_meta_tag_ZN16QHelpIndexWidgetE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QHelpIndexWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QListView::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16QHelpIndexWidgetE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16QHelpIndexWidgetE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16QHelpIndexWidgetE_t>.metaTypes,
    nullptr
} };

void QHelpIndexWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QHelpIndexWidget *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->linkActivated((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 1: _t->linksActivated((*reinterpret_cast<std::add_pointer_t<QMultiMap<QString,QUrl>>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 2: _t->documentActivated((*reinterpret_cast<std::add_pointer_t<QHelpLink>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 3: _t->documentsActivated((*reinterpret_cast<std::add_pointer_t<QList<QHelpLink>>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 4: _t->filterIndices((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 5: _t->filterIndices((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 6: _t->activateCurrentItem(); break;
        case 7: _t->showLink((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QHelpIndexWidget::*)(const QUrl & , const QString & )>(_a, &QHelpIndexWidget::linkActivated, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QHelpIndexWidget::*)(const QMultiMap<QString,QUrl> & , const QString & )>(_a, &QHelpIndexWidget::linksActivated, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QHelpIndexWidget::*)(const QHelpLink & , const QString & )>(_a, &QHelpIndexWidget::documentActivated, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QHelpIndexWidget::*)(const QList<QHelpLink> & , const QString & )>(_a, &QHelpIndexWidget::documentsActivated, 3))
            return;
    }
}

const QMetaObject *QHelpIndexWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QHelpIndexWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16QHelpIndexWidgetE_t>.strings))
        return static_cast<void*>(this);
    return QListView::qt_metacast(_clname);
}

int QHelpIndexWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QListView::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void QHelpIndexWidget::linkActivated(const QUrl & _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void QHelpIndexWidget::linksActivated(const QMultiMap<QString,QUrl> & _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void QHelpIndexWidget::documentActivated(const QHelpLink & _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void QHelpIndexWidget::documentsActivated(const QList<QHelpLink> & _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}
QT_WARNING_POP
