/****************************************************************************
** Meta object code from reading C++ file 'propertyeditor.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../../../qttools/src/designer/src/components/propertyeditor/propertyeditor.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'propertyeditor.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN18qdesigner_internal14PropertyEditorE_t {};
} // unnamed namespace

template <> constexpr inline auto qdesigner_internal::PropertyEditor::qt_create_metaobjectdata<qt_meta_tag_ZN18qdesigner_internal14PropertyEditorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "qdesigner_internal::PropertyEditor",
        "slotResetProperty",
        "",
        "QtProperty*",
        "property",
        "slotValueChanged",
        "QVariant",
        "value",
        "enableSubPropertyHandling",
        "slotViewTriggered",
        "QAction*",
        "action",
        "slotAddDynamicProperty",
        "slotRemoveDynamicProperty",
        "slotSorting",
        "sort",
        "slotColoring",
        "color",
        "slotCurrentItemChanged",
        "QtBrowserItem*",
        "setFilter",
        "pattern"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'slotResetProperty'
        QtMocHelpers::SlotData<void(QtProperty *)>(1, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'slotValueChanged'
        QtMocHelpers::SlotData<void(QtProperty *, const QVariant &, bool)>(5, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 6, 7 }, { QMetaType::Bool, 8 },
        }}),
        // Slot 'slotViewTriggered'
        QtMocHelpers::SlotData<void(QAction *)>(9, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Slot 'slotAddDynamicProperty'
        QtMocHelpers::SlotData<void(QAction *)>(12, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Slot 'slotRemoveDynamicProperty'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotSorting'
        QtMocHelpers::SlotData<void(bool)>(14, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 15 },
        }}),
        // Slot 'slotColoring'
        QtMocHelpers::SlotData<void(bool)>(16, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 17 },
        }}),
        // Slot 'slotCurrentItemChanged'
        QtMocHelpers::SlotData<void(QtBrowserItem *)>(18, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 19, 2 },
        }}),
        // Slot 'setFilter'
        QtMocHelpers::SlotData<void(const QString &)>(20, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 21 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<PropertyEditor, qt_meta_tag_ZN18qdesigner_internal14PropertyEditorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject qdesigner_internal::PropertyEditor::staticMetaObject = { {
    QMetaObject::SuperData::link<QDesignerPropertyEditor::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18qdesigner_internal14PropertyEditorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18qdesigner_internal14PropertyEditorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN18qdesigner_internal14PropertyEditorE_t>.metaTypes,
    nullptr
} };

void qdesigner_internal::PropertyEditor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PropertyEditor *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->slotResetProperty((*reinterpret_cast<std::add_pointer_t<QtProperty*>>(_a[1]))); break;
        case 1: _t->slotValueChanged((*reinterpret_cast<std::add_pointer_t<QtProperty*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 2: _t->slotViewTriggered((*reinterpret_cast<std::add_pointer_t<QAction*>>(_a[1]))); break;
        case 3: _t->slotAddDynamicProperty((*reinterpret_cast<std::add_pointer_t<QAction*>>(_a[1]))); break;
        case 4: _t->slotRemoveDynamicProperty(); break;
        case 5: _t->slotSorting((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 6: _t->slotColoring((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 7: _t->slotCurrentItemChanged((*reinterpret_cast<std::add_pointer_t<QtBrowserItem*>>(_a[1]))); break;
        case 8: _t->setFilter((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAction* >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAction* >(); break;
            }
            break;
        }
    }
}

const QMetaObject *qdesigner_internal::PropertyEditor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *qdesigner_internal::PropertyEditor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18qdesigner_internal14PropertyEditorE_t>.strings))
        return static_cast<void*>(this);
    return QDesignerPropertyEditor::qt_metacast(_clname);
}

int qdesigner_internal::PropertyEditor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDesignerPropertyEditor::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    return _id;
}
QT_WARNING_POP
