/****************************************************************************
** Meta object code from reading C++ file 'abstractintegration.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../../qttools/src/designer/src/lib/sdk/abstractintegration.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'abstractintegration.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN29QDesignerIntegrationInterfaceE_t {};
} // unnamed namespace

template <> constexpr inline auto QDesignerIntegrationInterface::qt_create_metaobjectdata<qt_meta_tag_ZN29QDesignerIntegrationInterfaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QDesignerIntegrationInterface",
        "propertyChanged",
        "",
        "QDesignerFormWindowInterface*",
        "formWindow",
        "name",
        "QVariant",
        "value",
        "objectNameChanged",
        "object",
        "newName",
        "oldName",
        "helpRequested",
        "manual",
        "document",
        "navigateToSlot",
        "objectName",
        "signalSignature",
        "parameterNames",
        "slotSignature",
        "setFeatures",
        "Feature",
        "f",
        "updateProperty",
        "enableSubPropertyHandling",
        "resetProperty",
        "addDynamicProperty",
        "removeDynamicProperty",
        "updateActiveFormWindow",
        "setupFormWindow",
        "updateSelection",
        "updateCustomWidgetPlugins",
        "headerSuffix",
        "headerLowercase",
        "qtVersion",
        "QVersionNumber",
        "ResourceFileWatcherBehaviour",
        "NoResourceFileWatcher",
        "ReloadResourceFileSilently",
        "PromptToReloadResourceFile"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'propertyChanged'
        QtMocHelpers::SignalData<void(QDesignerFormWindowInterface *, const QString &, const QVariant &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::QString, 5 }, { 0x80000000 | 6, 7 },
        }}),
        // Signal 'objectNameChanged'
        QtMocHelpers::SignalData<void(QDesignerFormWindowInterface *, QObject *, const QString &, const QString &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::QObjectStar, 9 }, { QMetaType::QString, 10 }, { QMetaType::QString, 11 },
        }}),
        // Signal 'helpRequested'
        QtMocHelpers::SignalData<void(const QString &, const QString &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 13 }, { QMetaType::QString, 14 },
        }}),
        // Signal 'navigateToSlot'
        QtMocHelpers::SignalData<void(const QString &, const QString &, const QStringList &)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 16 }, { QMetaType::QString, 17 }, { QMetaType::QStringList, 18 },
        }}),
        // Signal 'navigateToSlot'
        QtMocHelpers::SignalData<void(const QString &)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 19 },
        }}),
        // Slot 'setFeatures'
        QtMocHelpers::SlotData<void(Feature)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 21, 22 },
        }}),
        // Slot 'updateProperty'
        QtMocHelpers::SlotData<void(const QString &, const QVariant &, bool)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 5 }, { 0x80000000 | 6, 7 }, { QMetaType::Bool, 24 },
        }}),
        // Slot 'updateProperty'
        QtMocHelpers::SlotData<void(const QString &, const QVariant &)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 5 }, { 0x80000000 | 6, 7 },
        }}),
        // Slot 'resetProperty'
        QtMocHelpers::SlotData<void(const QString &)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 5 },
        }}),
        // Slot 'addDynamicProperty'
        QtMocHelpers::SlotData<void(const QString &, const QVariant &)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 5 }, { 0x80000000 | 6, 7 },
        }}),
        // Slot 'removeDynamicProperty'
        QtMocHelpers::SlotData<void(const QString &)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 5 },
        }}),
        // Slot 'updateActiveFormWindow'
        QtMocHelpers::SlotData<void(QDesignerFormWindowInterface *)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'setupFormWindow'
        QtMocHelpers::SlotData<void(QDesignerFormWindowInterface *)>(29, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'updateSelection'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'updateCustomWidgetPlugins'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'headerSuffix'
        QtMocHelpers::PropertyData<QString>(32, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'headerLowercase'
        QtMocHelpers::PropertyData<bool>(33, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'qtVersion'
        QtMocHelpers::PropertyData<QVersionNumber>(34, 0x80000000 | 35, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'ResourceFileWatcherBehaviour'
        QtMocHelpers::EnumData<enum ResourceFileWatcherBehaviour>(36, 36, QMC::EnumFlags{}).add({
            {   37, ResourceFileWatcherBehaviour::NoResourceFileWatcher },
            {   38, ResourceFileWatcherBehaviour::ReloadResourceFileSilently },
            {   39, ResourceFileWatcherBehaviour::PromptToReloadResourceFile },
        }),
    };
    return QtMocHelpers::metaObjectData<QDesignerIntegrationInterface, qt_meta_tag_ZN29QDesignerIntegrationInterfaceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QDesignerIntegrationInterface::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN29QDesignerIntegrationInterfaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN29QDesignerIntegrationInterfaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN29QDesignerIntegrationInterfaceE_t>.metaTypes,
    nullptr
} };

void QDesignerIntegrationInterface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QDesignerIntegrationInterface *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->propertyChanged((*reinterpret_cast<std::add_pointer_t<QDesignerFormWindowInterface*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[3]))); break;
        case 1: _t->objectNameChanged((*reinterpret_cast<std::add_pointer_t<QDesignerFormWindowInterface*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4]))); break;
        case 2: _t->helpRequested((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 3: _t->navigateToSlot((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[3]))); break;
        case 4: _t->navigateToSlot((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 5: _t->setFeatures((*reinterpret_cast<std::add_pointer_t<Feature>>(_a[1]))); break;
        case 6: _t->updateProperty((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 7: _t->updateProperty((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 8: _t->resetProperty((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 9: _t->addDynamicProperty((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 10: _t->removeDynamicProperty((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 11: _t->updateActiveFormWindow((*reinterpret_cast<std::add_pointer_t<QDesignerFormWindowInterface*>>(_a[1]))); break;
        case 12: _t->setupFormWindow((*reinterpret_cast<std::add_pointer_t<QDesignerFormWindowInterface*>>(_a[1]))); break;
        case 13: _t->updateSelection(); break;
        case 14: _t->updateCustomWidgetPlugins(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QDesignerIntegrationInterface::*)(QDesignerFormWindowInterface * , const QString & , const QVariant & )>(_a, &QDesignerIntegrationInterface::propertyChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDesignerIntegrationInterface::*)(QDesignerFormWindowInterface * , QObject * , const QString & , const QString & )>(_a, &QDesignerIntegrationInterface::objectNameChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDesignerIntegrationInterface::*)(const QString & , const QString & )>(_a, &QDesignerIntegrationInterface::helpRequested, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDesignerIntegrationInterface::*)(const QString & , const QString & , const QStringList & )>(_a, &QDesignerIntegrationInterface::navigateToSlot, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDesignerIntegrationInterface::*)(const QString & )>(_a, &QDesignerIntegrationInterface::navigateToSlot, 4))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 2:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QVersionNumber >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->headerSuffix(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->isHeaderLowercase(); break;
        case 2: *reinterpret_cast<QVersionNumber*>(_v) = _t->qtVersion(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setHeaderSuffix(*reinterpret_cast<QString*>(_v)); break;
        case 1: _t->setHeaderLowercase(*reinterpret_cast<bool*>(_v)); break;
        case 2: _t->setQtVersion(*reinterpret_cast<QVersionNumber*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QDesignerIntegrationInterface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QDesignerIntegrationInterface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN29QDesignerIntegrationInterfaceE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QDesignerIntegrationInterface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 15)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 15;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void QDesignerIntegrationInterface::propertyChanged(QDesignerFormWindowInterface * _t1, const QString & _t2, const QVariant & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2, _t3);
}

// SIGNAL 1
void QDesignerIntegrationInterface::objectNameChanged(QDesignerFormWindowInterface * _t1, QObject * _t2, const QString & _t3, const QString & _t4)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2, _t3, _t4);
}

// SIGNAL 2
void QDesignerIntegrationInterface::helpRequested(const QString & _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void QDesignerIntegrationInterface::navigateToSlot(const QString & _t1, const QString & _t2, const QStringList & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2, _t3);
}

// SIGNAL 4
void QDesignerIntegrationInterface::navigateToSlot(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN20QDesignerIntegrationE_t {};
} // unnamed namespace

template <> constexpr inline auto QDesignerIntegration::qt_create_metaobjectdata<qt_meta_tag_ZN20QDesignerIntegrationE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QDesignerIntegration"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QDesignerIntegration, qt_meta_tag_ZN20QDesignerIntegrationE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QDesignerIntegration::staticMetaObject = { {
    QMetaObject::SuperData::link<QDesignerIntegrationInterface::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20QDesignerIntegrationE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20QDesignerIntegrationE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN20QDesignerIntegrationE_t>.metaTypes,
    nullptr
} };

void QDesignerIntegration::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QDesignerIntegration *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *QDesignerIntegration::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QDesignerIntegration::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20QDesignerIntegrationE_t>.strings))
        return static_cast<void*>(this);
    return QDesignerIntegrationInterface::qt_metacast(_clname);
}

int QDesignerIntegration::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDesignerIntegrationInterface::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
