/****************************************************************************
** Meta object code from reading C++ file 'itemlisteditor.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../../../qttools/src/designer/src/components/taskmenu/itemlisteditor.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'itemlisteditor.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN18qdesigner_internal18AbstractItemEditorE_t {};
} // unnamed namespace

template <> constexpr inline auto qdesigner_internal::AbstractItemEditor::qt_create_metaobjectdata<qt_meta_tag_ZN18qdesigner_internal18AbstractItemEditorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "qdesigner_internal::AbstractItemEditor",
        "cacheReloaded",
        "",
        "propertyChanged",
        "QtProperty*",
        "property",
        "resetProperty"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'cacheReloaded'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'propertyChanged'
        QtMocHelpers::SlotData<void(QtProperty *)>(3, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 4, 5 },
        }}),
        // Slot 'resetProperty'
        QtMocHelpers::SlotData<void(QtProperty *)>(6, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 4, 5 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<AbstractItemEditor, qt_meta_tag_ZN18qdesigner_internal18AbstractItemEditorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject qdesigner_internal::AbstractItemEditor::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18qdesigner_internal18AbstractItemEditorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18qdesigner_internal18AbstractItemEditorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN18qdesigner_internal18AbstractItemEditorE_t>.metaTypes,
    nullptr
} };

void qdesigner_internal::AbstractItemEditor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<AbstractItemEditor *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->cacheReloaded(); break;
        case 1: _t->propertyChanged((*reinterpret_cast<std::add_pointer_t<QtProperty*>>(_a[1]))); break;
        case 2: _t->resetProperty((*reinterpret_cast<std::add_pointer_t<QtProperty*>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *qdesigner_internal::AbstractItemEditor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *qdesigner_internal::AbstractItemEditor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18qdesigner_internal18AbstractItemEditorE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int qdesigner_internal::AbstractItemEditor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 3;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN18qdesigner_internal14ItemListEditorE_t {};
} // unnamed namespace

template <> constexpr inline auto qdesigner_internal::ItemListEditor::qt_create_metaobjectdata<qt_meta_tag_ZN18qdesigner_internal14ItemListEditorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "qdesigner_internal::ItemListEditor",
        "indexChanged",
        "",
        "idx",
        "itemChanged",
        "role",
        "QVariant",
        "v",
        "itemInserted",
        "itemDeleted",
        "itemMovedUp",
        "itemMovedDown",
        "newListItemButtonClicked",
        "deleteListItemButtonClicked",
        "moveListItemUpButtonClicked",
        "moveListItemDownButtonClicked",
        "listWidgetCurrentRowChanged",
        "listWidgetItemChanged",
        "QListWidgetItem*",
        "item",
        "togglePropertyBrowser",
        "cacheReloaded"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'indexChanged'
        QtMocHelpers::SignalData<void(int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Signal 'itemChanged'
        QtMocHelpers::SignalData<void(int, int, const QVariant &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 }, { QMetaType::Int, 5 }, { 0x80000000 | 6, 7 },
        }}),
        // Signal 'itemInserted'
        QtMocHelpers::SignalData<void(int)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Signal 'itemDeleted'
        QtMocHelpers::SignalData<void(int)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Signal 'itemMovedUp'
        QtMocHelpers::SignalData<void(int)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Signal 'itemMovedDown'
        QtMocHelpers::SignalData<void(int)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Slot 'newListItemButtonClicked'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'deleteListItemButtonClicked'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'moveListItemUpButtonClicked'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'moveListItemDownButtonClicked'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'listWidgetCurrentRowChanged'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'listWidgetItemChanged'
        QtMocHelpers::SlotData<void(QListWidgetItem *)>(17, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 18, 19 },
        }}),
        // Slot 'togglePropertyBrowser'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'cacheReloaded'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ItemListEditor, qt_meta_tag_ZN18qdesigner_internal14ItemListEditorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject qdesigner_internal::ItemListEditor::staticMetaObject = { {
    QMetaObject::SuperData::link<AbstractItemEditor::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18qdesigner_internal14ItemListEditorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18qdesigner_internal14ItemListEditorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN18qdesigner_internal14ItemListEditorE_t>.metaTypes,
    nullptr
} };

void qdesigner_internal::ItemListEditor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ItemListEditor *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->indexChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 1: _t->itemChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[3]))); break;
        case 2: _t->itemInserted((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 3: _t->itemDeleted((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 4: _t->itemMovedUp((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 5: _t->itemMovedDown((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 6: _t->newListItemButtonClicked(); break;
        case 7: _t->deleteListItemButtonClicked(); break;
        case 8: _t->moveListItemUpButtonClicked(); break;
        case 9: _t->moveListItemDownButtonClicked(); break;
        case 10: _t->listWidgetCurrentRowChanged(); break;
        case 11: _t->listWidgetItemChanged((*reinterpret_cast<std::add_pointer_t<QListWidgetItem*>>(_a[1]))); break;
        case 12: _t->togglePropertyBrowser(); break;
        case 13: _t->cacheReloaded(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ItemListEditor::*)(int )>(_a, &ItemListEditor::indexChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ItemListEditor::*)(int , int , const QVariant & )>(_a, &ItemListEditor::itemChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ItemListEditor::*)(int )>(_a, &ItemListEditor::itemInserted, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (ItemListEditor::*)(int )>(_a, &ItemListEditor::itemDeleted, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (ItemListEditor::*)(int )>(_a, &ItemListEditor::itemMovedUp, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (ItemListEditor::*)(int )>(_a, &ItemListEditor::itemMovedDown, 5))
            return;
    }
}

const QMetaObject *qdesigner_internal::ItemListEditor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *qdesigner_internal::ItemListEditor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18qdesigner_internal14ItemListEditorE_t>.strings))
        return static_cast<void*>(this);
    return AbstractItemEditor::qt_metacast(_clname);
}

int qdesigner_internal::ItemListEditor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = AbstractItemEditor::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 14)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 14;
    }
    return _id;
}

// SIGNAL 0
void qdesigner_internal::ItemListEditor::indexChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void qdesigner_internal::ItemListEditor::itemChanged(int _t1, int _t2, const QVariant & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2, _t3);
}

// SIGNAL 2
void qdesigner_internal::ItemListEditor::itemInserted(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void qdesigner_internal::ItemListEditor::itemDeleted(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void qdesigner_internal::ItemListEditor::itemMovedUp(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void qdesigner_internal::ItemListEditor::itemMovedDown(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}
QT_WARNING_POP
