/********************************************************************************
** Form generated from reading UI file 'machinetranslationdialog.ui'
**
** Created by: Qt User Interface Compiler version 6.12.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_MACHINETRANSLATIONDIALOG_H
#define UI_MACHINETRANSLATIONDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPlainTextEdit>
#include <QtWidgets/QProgressBar>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QTextEdit>
#include <QtWidgets/QToolBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_MachineTranslationDialog
{
public:
    QVBoxLayout *verticalLayout_8;
    QToolBox *toolBox;
    QWidget *configPage;
    QVBoxLayout *verticalLayout_2;
    QGroupBox *configGroup;
    QVBoxLayout *verticalLayout_4;
    QGridLayout *configLayout;
    QSpacerItem *verticalSpacer_5;
    QLabel *contextLabel;
    QLabel *apiTypeLabel;
    QComboBox *modelComboBox;
    QSpacerItem *verticalSpacer;
    QPlainTextEdit *contextEdit;
    QWidget *advancedSettingsWidget;
    QGridLayout *advancedSettingsLayout;
    QLabel *maxRetriesLabel;
    QSpinBox *maxRetriesSpinBox;
    QLabel *maxConcurrentBatchesLabel;
    QSpinBox *maxConcurrentBatchesSpinBox;
    QLabel *transferTimeoutLabel;
    QSpinBox *transferTimeoutSpinBox;
    QLabel *maxBatchSizeLabel;
    QSpinBox *maxBatchSizeSpinBox;
    QLabel *temperatureLabel;
    QDoubleSpinBox *temperatureSpinBox;
    QLabel *maxJsonFormatTriesLabel;
    QSpinBox *maxJsonFormatTriesSpinBox;
    QLabel *ollamaWakeUpTimeoutLabel;
    QSpinBox *ollamaWakeUpTimeoutSpinBox;
    QLabel *settingsWarningLabel;
    QPushButton *applySettingsButton;
    QPushButton *resetSettingsButton;
    QPushButton *advancedSettingsToggle;
    QPushButton *nextButton;
    QLabel *connectionStatusLabel;
    QComboBox *apiTypeComboBox;
    QLabel *serverLabel;
    QSpacerItem *verticalSpacer_2;
    QLineEdit *serverText;
    QLabel *serverLabel_2;
    QPushButton *connectButton;
    QLabel *apiKeyLabel;
    QLineEdit *apiKeyEdit;
    QWidget *selectionPage;
    QVBoxLayout *verticalLayout_5;
    QGroupBox *selectionGroup;
    QHBoxLayout *horizontalLayout_3;
    QGridLayout *selectionLayout;
    QComboBox *filesComboBox;
    QListWidget *groupListWidget;
    QComboBox *filterComboBox;
    QLabel *selectionLabel;
    QPushButton *translateButton;
    QSpacerItem *verticalSpacer_4;
    QLabel *groupLabel;
    QSpacerItem *verticalSpacer_3;
    QLabel *filterLabel;
    QLabel *filesLabel;
    QWidget *progressPage;
    QVBoxLayout *verticalLayout_6;
    QGroupBox *progressGroup;
    QVBoxLayout *verticalLayout;
    QGridLayout *gridLayout;
    QTextEdit *translationLog;
    QProgressBar *progressBar;
    QLabel *statusLabel;
    QPushButton *stopButton;
    QPushButton *logButton;
    QHBoxLayout *horizontalLayout_2;
    QPushButton *applyButton;
    QSpacerItem *horizontalSpacer;
    QPushButton *cancelButton;
    QPushButton *doneButton;

    void setupUi(QDialog *MachineTranslationDialog)
    {
        if (MachineTranslationDialog->objectName().isEmpty())
            MachineTranslationDialog->setObjectName("MachineTranslationDialog");
        MachineTranslationDialog->resize(517, 672);
        verticalLayout_8 = new QVBoxLayout(MachineTranslationDialog);
        verticalLayout_8->setObjectName("verticalLayout_8");
        toolBox = new QToolBox(MachineTranslationDialog);
        toolBox->setObjectName("toolBox");
        toolBox->setFrameShape(QFrame::Shape::NoFrame);
        toolBox->setFrameShadow(QFrame::Shadow::Plain);
        configPage = new QWidget();
        configPage->setObjectName("configPage");
        configPage->setGeometry(QRect(0, 0, 493, 498));
        verticalLayout_2 = new QVBoxLayout(configPage);
        verticalLayout_2->setObjectName("verticalLayout_2");
        configGroup = new QGroupBox(configPage);
        configGroup->setObjectName("configGroup");
        configGroup->setFlat(false);
        configGroup->setCheckable(false);
        verticalLayout_4 = new QVBoxLayout(configGroup);
        verticalLayout_4->setObjectName("verticalLayout_4");
        configLayout = new QGridLayout();
        configLayout->setObjectName("configLayout");
        verticalSpacer_5 = new QSpacerItem(20, 10, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        configLayout->addItem(verticalSpacer_5, 2, 0, 1, 1);

        contextLabel = new QLabel(configGroup);
        contextLabel->setObjectName("contextLabel");
        contextLabel->setMaximumSize(QSize(16777215, 16777215));

        configLayout->addWidget(contextLabel, 12, 0, 1, 1);

        apiTypeLabel = new QLabel(configGroup);
        apiTypeLabel->setObjectName("apiTypeLabel");
        apiTypeLabel->setMaximumSize(QSize(16777215, 16777215));

        configLayout->addWidget(apiTypeLabel, 0, 0, 1, 1);

        modelComboBox = new QComboBox(configGroup);
        modelComboBox->setObjectName("modelComboBox");
        modelComboBox->setMinimumSize(QSize(150, 0));

        configLayout->addWidget(modelComboBox, 10, 0, 1, 2);

        verticalSpacer = new QSpacerItem(20, 10, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        configLayout->addItem(verticalSpacer, 11, 0, 1, 1);

        contextEdit = new QPlainTextEdit(configGroup);
        contextEdit->setObjectName("contextEdit");
        contextEdit->setMaximumSize(QSize(16777215, 16777215));

        configLayout->addWidget(contextEdit, 13, 0, 1, 2);

        advancedSettingsWidget = new QWidget(configGroup);
        advancedSettingsWidget->setObjectName("advancedSettingsWidget");
        advancedSettingsWidget->setVisible(false);
        advancedSettingsLayout = new QGridLayout(advancedSettingsWidget);
        advancedSettingsLayout->setObjectName("advancedSettingsLayout");
        maxRetriesLabel = new QLabel(advancedSettingsWidget);
        maxRetriesLabel->setObjectName("maxRetriesLabel");

        advancedSettingsLayout->addWidget(maxRetriesLabel, 0, 0, 1, 1);

        maxRetriesSpinBox = new QSpinBox(advancedSettingsWidget);
        maxRetriesSpinBox->setObjectName("maxRetriesSpinBox");
        maxRetriesSpinBox->setMinimum(1);
        maxRetriesSpinBox->setMaximum(100);

        advancedSettingsLayout->addWidget(maxRetriesSpinBox, 0, 1, 1, 1);

        maxConcurrentBatchesLabel = new QLabel(advancedSettingsWidget);
        maxConcurrentBatchesLabel->setObjectName("maxConcurrentBatchesLabel");

        advancedSettingsLayout->addWidget(maxConcurrentBatchesLabel, 1, 0, 1, 1);

        maxConcurrentBatchesSpinBox = new QSpinBox(advancedSettingsWidget);
        maxConcurrentBatchesSpinBox->setObjectName("maxConcurrentBatchesSpinBox");
        maxConcurrentBatchesSpinBox->setMinimum(1);
        maxConcurrentBatchesSpinBox->setMaximum(20);

        advancedSettingsLayout->addWidget(maxConcurrentBatchesSpinBox, 1, 1, 1, 1);

        transferTimeoutLabel = new QLabel(advancedSettingsWidget);
        transferTimeoutLabel->setObjectName("transferTimeoutLabel");

        advancedSettingsLayout->addWidget(transferTimeoutLabel, 2, 0, 1, 1);

        transferTimeoutSpinBox = new QSpinBox(advancedSettingsWidget);
        transferTimeoutSpinBox->setObjectName("transferTimeoutSpinBox");
        transferTimeoutSpinBox->setMinimum(10);
        transferTimeoutSpinBox->setMaximum(3600);

        advancedSettingsLayout->addWidget(transferTimeoutSpinBox, 2, 1, 1, 1);

        maxBatchSizeLabel = new QLabel(advancedSettingsWidget);
        maxBatchSizeLabel->setObjectName("maxBatchSizeLabel");

        advancedSettingsLayout->addWidget(maxBatchSizeLabel, 3, 0, 1, 1);

        maxBatchSizeSpinBox = new QSpinBox(advancedSettingsWidget);
        maxBatchSizeSpinBox->setObjectName("maxBatchSizeSpinBox");
        maxBatchSizeSpinBox->setMinimum(1);
        maxBatchSizeSpinBox->setMaximum(100);

        advancedSettingsLayout->addWidget(maxBatchSizeSpinBox, 3, 1, 1, 1);

        temperatureLabel = new QLabel(advancedSettingsWidget);
        temperatureLabel->setObjectName("temperatureLabel");

        advancedSettingsLayout->addWidget(temperatureLabel, 4, 0, 1, 1);

        temperatureSpinBox = new QDoubleSpinBox(advancedSettingsWidget);
        temperatureSpinBox->setObjectName("temperatureSpinBox");
        temperatureSpinBox->setMaximum(2.000000000000000);
        temperatureSpinBox->setSingleStep(0.050000000000000);

        advancedSettingsLayout->addWidget(temperatureSpinBox, 4, 1, 1, 1);

        maxJsonFormatTriesLabel = new QLabel(advancedSettingsWidget);
        maxJsonFormatTriesLabel->setObjectName("maxJsonFormatTriesLabel");

        advancedSettingsLayout->addWidget(maxJsonFormatTriesLabel, 5, 0, 1, 1);

        maxJsonFormatTriesSpinBox = new QSpinBox(advancedSettingsWidget);
        maxJsonFormatTriesSpinBox->setObjectName("maxJsonFormatTriesSpinBox");
        maxJsonFormatTriesSpinBox->setMinimum(1);
        maxJsonFormatTriesSpinBox->setMaximum(20);

        advancedSettingsLayout->addWidget(maxJsonFormatTriesSpinBox, 5, 1, 1, 1);

        ollamaWakeUpTimeoutLabel = new QLabel(advancedSettingsWidget);
        ollamaWakeUpTimeoutLabel->setObjectName("ollamaWakeUpTimeoutLabel");

        advancedSettingsLayout->addWidget(ollamaWakeUpTimeoutLabel, 6, 0, 1, 1);

        ollamaWakeUpTimeoutSpinBox = new QSpinBox(advancedSettingsWidget);
        ollamaWakeUpTimeoutSpinBox->setObjectName("ollamaWakeUpTimeoutSpinBox");
        ollamaWakeUpTimeoutSpinBox->setMinimum(30);
        ollamaWakeUpTimeoutSpinBox->setMaximum(600);

        advancedSettingsLayout->addWidget(ollamaWakeUpTimeoutSpinBox, 6, 1, 1, 1);

        settingsWarningLabel = new QLabel(advancedSettingsWidget);
        settingsWarningLabel->setObjectName("settingsWarningLabel");
        settingsWarningLabel->setWordWrap(true);

        advancedSettingsLayout->addWidget(settingsWarningLabel, 7, 0, 1, 2);

        applySettingsButton = new QPushButton(advancedSettingsWidget);
        applySettingsButton->setObjectName("applySettingsButton");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Maximum, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(applySettingsButton->sizePolicy().hasHeightForWidth());
        applySettingsButton->setSizePolicy(sizePolicy);

        advancedSettingsLayout->addWidget(applySettingsButton, 8, 0, 1, 1, Qt::AlignmentFlag::AlignRight|Qt::AlignmentFlag::AlignVCenter);

        resetSettingsButton = new QPushButton(advancedSettingsWidget);
        resetSettingsButton->setObjectName("resetSettingsButton");
        sizePolicy.setHeightForWidth(resetSettingsButton->sizePolicy().hasHeightForWidth());
        resetSettingsButton->setSizePolicy(sizePolicy);

        advancedSettingsLayout->addWidget(resetSettingsButton, 8, 1, 1, 1);


        configLayout->addWidget(advancedSettingsWidget, 14, 0, 1, 2);

        advancedSettingsToggle = new QPushButton(configGroup);
        advancedSettingsToggle->setObjectName("advancedSettingsToggle");
        sizePolicy.setHeightForWidth(advancedSettingsToggle->sizePolicy().hasHeightForWidth());
        advancedSettingsToggle->setSizePolicy(sizePolicy);
        advancedSettingsToggle->setCheckable(true);

        configLayout->addWidget(advancedSettingsToggle, 15, 0, 1, 1, Qt::AlignmentFlag::AlignRight|Qt::AlignmentFlag::AlignVCenter);

        nextButton = new QPushButton(configGroup);
        nextButton->setObjectName("nextButton");
        sizePolicy.setHeightForWidth(nextButton->sizePolicy().hasHeightForWidth());
        nextButton->setSizePolicy(sizePolicy);

        configLayout->addWidget(nextButton, 15, 1, 1, 1);

        connectionStatusLabel = new QLabel(configGroup);
        connectionStatusLabel->setObjectName("connectionStatusLabel");
        connectionStatusLabel->setAlignment(Qt::AlignmentFlag::AlignLeading|Qt::AlignmentFlag::AlignLeft|Qt::AlignmentFlag::AlignVCenter);

        configLayout->addWidget(connectionStatusLabel, 7, 0, 1, 2);

        apiTypeComboBox = new QComboBox(configGroup);
        apiTypeComboBox->setObjectName("apiTypeComboBox");
        apiTypeComboBox->setMinimumSize(QSize(150, 0));

        configLayout->addWidget(apiTypeComboBox, 1, 0, 1, 2);

        serverLabel = new QLabel(configGroup);
        serverLabel->setObjectName("serverLabel");
        serverLabel->setMaximumSize(QSize(16777215, 16777215));

        configLayout->addWidget(serverLabel, 3, 0, 1, 1);

        verticalSpacer_2 = new QSpacerItem(20, 10, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        configLayout->addItem(verticalSpacer_2, 8, 0, 1, 1);

        serverText = new QLineEdit(configGroup);
        serverText->setObjectName("serverText");
        serverText->setText(QString::fromUtf8("http://localhost:11434"));

        configLayout->addWidget(serverText, 4, 0, 1, 1);

        serverLabel_2 = new QLabel(configGroup);
        serverLabel_2->setObjectName("serverLabel_2");
        serverLabel_2->setMaximumSize(QSize(16777215, 16777215));

        configLayout->addWidget(serverLabel_2, 9, 0, 1, 1);

        connectButton = new QPushButton(configGroup);
        connectButton->setObjectName("connectButton");
        connectButton->setAutoDefault(false);

        configLayout->addWidget(connectButton, 4, 1, 1, 1);

        apiKeyLabel = new QLabel(configGroup);
        apiKeyLabel->setObjectName("apiKeyLabel");

        configLayout->addWidget(apiKeyLabel, 5, 0, 1, 1);

        apiKeyEdit = new QLineEdit(configGroup);
        apiKeyEdit->setObjectName("apiKeyEdit");
        apiKeyEdit->setEchoMode(QLineEdit::EchoMode::Password);

        configLayout->addWidget(apiKeyEdit, 6, 0, 1, 2);


        verticalLayout_4->addLayout(configLayout);


        verticalLayout_2->addWidget(configGroup);

        toolBox->addItem(configPage, QString::fromUtf8("Page"));
        selectionPage = new QWidget();
        selectionPage->setObjectName("selectionPage");
        selectionPage->setGeometry(QRect(0, 0, 493, 498));
        verticalLayout_5 = new QVBoxLayout(selectionPage);
        verticalLayout_5->setObjectName("verticalLayout_5");
        selectionGroup = new QGroupBox(selectionPage);
        selectionGroup->setObjectName("selectionGroup");
        horizontalLayout_3 = new QHBoxLayout(selectionGroup);
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        selectionLayout = new QGridLayout();
        selectionLayout->setObjectName("selectionLayout");
        selectionLayout->setSizeConstraint(QLayout::SizeConstraint::SetDefaultConstraint);
        filesComboBox = new QComboBox(selectionGroup);
        filesComboBox->setObjectName("filesComboBox");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Maximum);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(filesComboBox->sizePolicy().hasHeightForWidth());
        filesComboBox->setSizePolicy(sizePolicy1);

        selectionLayout->addWidget(filesComboBox, 2, 0, 1, 2);

        groupListWidget = new QListWidget(selectionGroup);
        groupListWidget->setObjectName("groupListWidget");
        groupListWidget->setEnabled(false);
        groupListWidget->setMaximumSize(QSize(16777215, 16777215));
        groupListWidget->setSelectionMode(QAbstractItemView::SelectionMode::ExtendedSelection);

        selectionLayout->addWidget(groupListWidget, 8, 0, 1, 2);

        filterComboBox = new QComboBox(selectionGroup);
        filterComboBox->addItem(QString());
        filterComboBox->addItem(QString());
        filterComboBox->addItem(QString());
        filterComboBox->setObjectName("filterComboBox");
        filterComboBox->setDuplicatesEnabled(false);

        selectionLayout->addWidget(filterComboBox, 5, 0, 1, 2);

        selectionLabel = new QLabel(selectionGroup);
        selectionLabel->setObjectName("selectionLabel");

        selectionLayout->addWidget(selectionLabel, 9, 0, 1, 2);

        translateButton = new QPushButton(selectionGroup);
        translateButton->setObjectName("translateButton");

        selectionLayout->addWidget(translateButton, 10, 0, 1, 2);

        verticalSpacer_4 = new QSpacerItem(20, 10, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        selectionLayout->addItem(verticalSpacer_4, 6, 0, 1, 2);

        groupLabel = new QLabel(selectionGroup);
        groupLabel->setObjectName("groupLabel");
        groupLabel->setEnabled(false);
        groupLabel->setAlignment(Qt::AlignmentFlag::AlignLeading|Qt::AlignmentFlag::AlignLeft|Qt::AlignmentFlag::AlignTop);

        selectionLayout->addWidget(groupLabel, 7, 0, 1, 2);

        verticalSpacer_3 = new QSpacerItem(20, 10, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        selectionLayout->addItem(verticalSpacer_3, 3, 0, 1, 2);

        filterLabel = new QLabel(selectionGroup);
        filterLabel->setObjectName("filterLabel");

        selectionLayout->addWidget(filterLabel, 4, 0, 1, 2);

        filesLabel = new QLabel(selectionGroup);
        filesLabel->setObjectName("filesLabel");

        selectionLayout->addWidget(filesLabel, 1, 0, 1, 2);

        selectionLayout->setRowStretch(8, 1);
        selectionLayout->setColumnStretch(0, 1);

        horizontalLayout_3->addLayout(selectionLayout);


        verticalLayout_5->addWidget(selectionGroup);

        toolBox->addItem(selectionPage, QString::fromUtf8("Page"));
        progressPage = new QWidget();
        progressPage->setObjectName("progressPage");
        progressPage->setGeometry(QRect(0, 0, 493, 498));
        verticalLayout_6 = new QVBoxLayout(progressPage);
        verticalLayout_6->setObjectName("verticalLayout_6");
        progressGroup = new QGroupBox(progressPage);
        progressGroup->setObjectName("progressGroup");
        verticalLayout = new QVBoxLayout(progressGroup);
        verticalLayout->setObjectName("verticalLayout");
        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        translationLog = new QTextEdit(progressGroup);
        translationLog->setObjectName("translationLog");
        translationLog->setMinimumSize(QSize(200, 300));
        translationLog->setReadOnly(true);

        gridLayout->addWidget(translationLog, 1, 0, 1, 3);

        progressBar = new QProgressBar(progressGroup);
        progressBar->setObjectName("progressBar");
        progressBar->setEnabled(true);
        progressBar->setValue(0);

        gridLayout->addWidget(progressBar, 3, 0, 1, 1);

        statusLabel = new QLabel(progressGroup);
        statusLabel->setObjectName("statusLabel");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(statusLabel->sizePolicy().hasHeightForWidth());
        statusLabel->setSizePolicy(sizePolicy2);

        gridLayout->addWidget(statusLabel, 0, 0, 1, 3);

        stopButton = new QPushButton(progressGroup);
        stopButton->setObjectName("stopButton");
        stopButton->setMinimumSize(QSize(70, 0));
        stopButton->setMaximumSize(QSize(70, 16777215));

        gridLayout->addWidget(stopButton, 3, 2, 1, 1);

        logButton = new QPushButton(progressGroup);
        logButton->setObjectName("logButton");
        logButton->setMinimumSize(QSize(150, 0));
        logButton->setMaximumSize(QSize(150, 16777215));
        logButton->setCheckable(true);
        logButton->setChecked(false);

        gridLayout->addWidget(logButton, 3, 1, 1, 1);


        verticalLayout->addLayout(gridLayout);


        verticalLayout_6->addWidget(progressGroup);

        toolBox->addItem(progressPage, QString::fromUtf8("Page"));

        verticalLayout_8->addWidget(toolBox);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        applyButton = new QPushButton(MachineTranslationDialog);
        applyButton->setObjectName("applyButton");
        applyButton->setEnabled(false);

        horizontalLayout_2->addWidget(applyButton);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer);

        cancelButton = new QPushButton(MachineTranslationDialog);
        cancelButton->setObjectName("cancelButton");

        horizontalLayout_2->addWidget(cancelButton);

        doneButton = new QPushButton(MachineTranslationDialog);
        doneButton->setObjectName("doneButton");
        QSizePolicy sizePolicy3(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(doneButton->sizePolicy().hasHeightForWidth());
        doneButton->setSizePolicy(sizePolicy3);
        doneButton->setMaximumSize(QSize(16777215, 16777215));

        horizontalLayout_2->addWidget(doneButton);


        verticalLayout_8->addLayout(horizontalLayout_2);

#if QT_CONFIG(shortcut)
        contextLabel->setBuddy(contextEdit);
        maxRetriesLabel->setBuddy(maxRetriesSpinBox);
        maxConcurrentBatchesLabel->setBuddy(maxConcurrentBatchesSpinBox);
        transferTimeoutLabel->setBuddy(transferTimeoutSpinBox);
        maxBatchSizeLabel->setBuddy(maxBatchSizeSpinBox);
        temperatureLabel->setBuddy(temperatureSpinBox);
        maxJsonFormatTriesLabel->setBuddy(maxJsonFormatTriesSpinBox);
        ollamaWakeUpTimeoutLabel->setBuddy(ollamaWakeUpTimeoutSpinBox);
        apiKeyLabel->setBuddy(apiKeyEdit);
        filesLabel->setBuddy(filesComboBox);
#endif // QT_CONFIG(shortcut)

        retranslateUi(MachineTranslationDialog);

        toolBox->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(MachineTranslationDialog);
    } // setupUi

    void retranslateUi(QDialog *MachineTranslationDialog)
    {
        MachineTranslationDialog->setWindowTitle(QCoreApplication::translate("MachineTranslationDialog", "Auto Translation", nullptr));
        configGroup->setTitle(QString());
        contextLabel->setText(QCoreApplication::translate("MachineTranslationDialog", "Context", nullptr));
        apiTypeLabel->setText(QCoreApplication::translate("MachineTranslationDialog", "API Type", nullptr));
        modelComboBox->setCurrentText(QString());
        contextEdit->setPlaceholderText(QCoreApplication::translate("MachineTranslationDialog", "Optional: Provide additional context about your application (e.g., medical software, game, financial app) for more accurate translations.", nullptr));
#if QT_CONFIG(tooltip)
        maxRetriesLabel->setToolTip(QCoreApplication::translate("MachineTranslationDialog", "Maximum number of retry attempts for failed translation requests", nullptr));
#endif // QT_CONFIG(tooltip)
        maxRetriesLabel->setText(QCoreApplication::translate("MachineTranslationDialog", "Maximum Retries", nullptr));
#if QT_CONFIG(tooltip)
        maxConcurrentBatchesLabel->setToolTip(QCoreApplication::translate("MachineTranslationDialog", "Maximum number of translation batches to process in parallel", nullptr));
#endif // QT_CONFIG(tooltip)
        maxConcurrentBatchesLabel->setText(QCoreApplication::translate("MachineTranslationDialog", "Maximum Concurrent Batches", nullptr));
#if QT_CONFIG(tooltip)
        transferTimeoutLabel->setToolTip(QCoreApplication::translate("MachineTranslationDialog", "Timeout for network requests in seconds", nullptr));
#endif // QT_CONFIG(tooltip)
        transferTimeoutLabel->setText(QCoreApplication::translate("MachineTranslationDialog", "Transfer Timeout (seconds)", nullptr));
#if QT_CONFIG(tooltip)
        maxBatchSizeLabel->setToolTip(QCoreApplication::translate("MachineTranslationDialog", "Maximum number of source texts to include in each translation batch", nullptr));
#endif // QT_CONFIG(tooltip)
        maxBatchSizeLabel->setText(QCoreApplication::translate("MachineTranslationDialog", "Maximum Batch Size", nullptr));
#if QT_CONFIG(tooltip)
        temperatureLabel->setToolTip(QCoreApplication::translate("MachineTranslationDialog", "LLM temperature (lower = more deterministic, higher = more creative)", nullptr));
#endif // QT_CONFIG(tooltip)
        temperatureLabel->setText(QCoreApplication::translate("MachineTranslationDialog", "Temperature", nullptr));
#if QT_CONFIG(tooltip)
        maxJsonFormatTriesLabel->setToolTip(QCoreApplication::translate("MachineTranslationDialog", "Number of attempts of sending JSON-format requests before falling back to non-JSON format requests", nullptr));
#endif // QT_CONFIG(tooltip)
        maxJsonFormatTriesLabel->setText(QCoreApplication::translate("MachineTranslationDialog", "Maximum JSON Format Tries", nullptr));
#if QT_CONFIG(tooltip)
        ollamaWakeUpTimeoutLabel->setToolTip(QCoreApplication::translate("MachineTranslationDialog", "Timeout for waking up an idle Ollama server in seconds", nullptr));
#endif // QT_CONFIG(tooltip)
        ollamaWakeUpTimeoutLabel->setText(QCoreApplication::translate("MachineTranslationDialog", "Ollama Wake-up Timeout (seconds)", nullptr));
        settingsWarningLabel->setStyleSheet(QCoreApplication::translate("MachineTranslationDialog", "color: #b58900;", nullptr));
        applySettingsButton->setText(QCoreApplication::translate("MachineTranslationDialog", "Apply", nullptr));
        resetSettingsButton->setText(QCoreApplication::translate("MachineTranslationDialog", "Reset to Defaults", nullptr));
        nextButton->setText(QCoreApplication::translate("MachineTranslationDialog", "Next", nullptr));
        connectionStatusLabel->setText(QCoreApplication::translate("MachineTranslationDialog", "Not connected", nullptr));
        apiTypeComboBox->setCurrentText(QString());
        serverLabel->setText(QCoreApplication::translate("MachineTranslationDialog", "Server URL", nullptr));
        serverLabel_2->setText(QCoreApplication::translate("MachineTranslationDialog", "Model", nullptr));
        connectButton->setText(QCoreApplication::translate("MachineTranslationDialog", "Connect", nullptr));
        apiKeyLabel->setText(QCoreApplication::translate("MachineTranslationDialog", "API Key", nullptr));
        apiKeyEdit->setPlaceholderText(QCoreApplication::translate("MachineTranslationDialog", "Optional: Required only for cloud APIs (OpenAI, Anthropic, etc.)", nullptr));
        toolBox->setItemText(toolBox->indexOf(configPage), QCoreApplication::translate("MachineTranslationDialog", "Page", nullptr));
        selectionGroup->setTitle(QString());
        filterComboBox->setItemText(0, QCoreApplication::translate("MachineTranslationDialog", "All Items", nullptr));
        filterComboBox->setItemText(1, QCoreApplication::translate("MachineTranslationDialog", "Text Based", nullptr));
        filterComboBox->setItemText(2, QCoreApplication::translate("MachineTranslationDialog", "ID Based", nullptr));

        selectionLabel->setText(QCoreApplication::translate("MachineTranslationDialog", "No messages selected", nullptr));
        translateButton->setText(QCoreApplication::translate("MachineTranslationDialog", "Translate", nullptr));
        groupLabel->setText(QCoreApplication::translate("MachineTranslationDialog", "Groups", nullptr));
        filterLabel->setText(QCoreApplication::translate("MachineTranslationDialog", "Filter", nullptr));
        filesLabel->setText(QCoreApplication::translate("MachineTranslationDialog", "File", nullptr));
        toolBox->setItemText(toolBox->indexOf(selectionPage), QCoreApplication::translate("MachineTranslationDialog", "Page", nullptr));
        progressGroup->setTitle(QString());
        translationLog->setPlaceholderText(QCoreApplication::translate("MachineTranslationDialog", "Translation Log", nullptr));
        statusLabel->setText(QCoreApplication::translate("MachineTranslationDialog", "Translation status: -", nullptr));
        stopButton->setText(QCoreApplication::translate("MachineTranslationDialog", "Stop", nullptr));
        logButton->setText(QCoreApplication::translate("MachineTranslationDialog", "Log Raw Requests", nullptr));
        toolBox->setItemText(toolBox->indexOf(progressPage), QCoreApplication::translate("MachineTranslationDialog", "Page", nullptr));
        applyButton->setText(QCoreApplication::translate("MachineTranslationDialog", "Apply Translations", nullptr));
        cancelButton->setText(QCoreApplication::translate("MachineTranslationDialog", "Cancel", nullptr));
        doneButton->setText(QCoreApplication::translate("MachineTranslationDialog", "Done", nullptr));
    } // retranslateUi

};

namespace Ui {
    class MachineTranslationDialog: public Ui_MachineTranslationDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_MACHINETRANSLATIONDIALOG_H
