/****************************************************************************
** Meta object code from reading C++ file 'qdesigner_dockwidget_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../../qttools/src/designer/src/lib/shared/qdesigner_dockwidget_p.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qdesigner_dockwidget_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN19QDesignerDockWidgetE_t {};
} // unnamed namespace

template <> constexpr inline auto QDesignerDockWidget::qt_create_metaobjectdata<qt_meta_tag_ZN19QDesignerDockWidgetE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QDesignerDockWidget",
        "dockWidgetArea",
        "Qt::DockWidgetArea",
        "docked"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'dockWidgetArea'
        QtMocHelpers::PropertyData<Qt::DockWidgetArea>(1, 0x80000000 | 2, QMC::Readable | QMC::Writable | QMC::Designable | QMC::Scriptable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'docked'
        QtMocHelpers::PropertyData<bool>(3, QMetaType::Bool, QMC::Readable | QMC::Writable | QMC::Designable | QMC::Scriptable | QMC::StdCppSet),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QDesignerDockWidget, qt_meta_tag_ZN19QDesignerDockWidgetE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QDesignerDockWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QDockWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19QDesignerDockWidgetE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19QDesignerDockWidgetE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN19QDesignerDockWidgetE_t>.metaTypes,
    nullptr
} };

void QDesignerDockWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QDesignerDockWidget *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<Qt::DockWidgetArea*>(_v) = _t->dockWidgetArea(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->docked(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setDockWidgetArea(*reinterpret_cast<Qt::DockWidgetArea*>(_v)); break;
        case 1: _t->setDocked(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QDesignerDockWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QDesignerDockWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19QDesignerDockWidgetE_t>.strings))
        return static_cast<void*>(this);
    return QDockWidget::qt_metacast(_clname);
}

int QDesignerDockWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDockWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN24QDockWidgetPropertySheetE_t {};
} // unnamed namespace

template <> constexpr inline auto QDockWidgetPropertySheet::qt_create_metaobjectdata<qt_meta_tag_ZN24QDockWidgetPropertySheetE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QDockWidgetPropertySheet"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QDockWidgetPropertySheet, qt_meta_tag_ZN24QDockWidgetPropertySheetE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QDockWidgetPropertySheet::staticMetaObject = { {
    QMetaObject::SuperData::link<QDesignerPropertySheet::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24QDockWidgetPropertySheetE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24QDockWidgetPropertySheetE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN24QDockWidgetPropertySheetE_t>.metaTypes,
    nullptr
} };

void QDockWidgetPropertySheet::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QDockWidgetPropertySheet *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *QDockWidgetPropertySheet::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QDockWidgetPropertySheet::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24QDockWidgetPropertySheetE_t>.strings))
        return static_cast<void*>(this);
    return QDesignerPropertySheet::qt_metacast(_clname);
}

int QDockWidgetPropertySheet::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDesignerPropertySheet::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
