/****************************************************************************
** Meta object code from reading C++ file 'qdesigner_promotiondialog_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../../qttools/src/designer/src/lib/shared/qdesigner_promotiondialog_p.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qdesigner_promotiondialog_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN18qdesigner_internal21NewPromotedClassPanelE_t {};
} // unnamed namespace

template <> constexpr inline auto qdesigner_internal::NewPromotedClassPanel::qt_create_metaobjectdata<qt_meta_tag_ZN18qdesigner_internal21NewPromotedClassPanelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "qdesigner_internal::NewPromotedClassPanel",
        "newPromotedClass",
        "",
        "PromotionParameters",
        "bool*",
        "ok",
        "grabFocus",
        "chooseBaseClass",
        "slotNameChanged",
        "slotIncludeFileChanged",
        "slotAdd",
        "slotReset"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'newPromotedClass'
        QtMocHelpers::SignalData<void(const PromotionParameters &, bool *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 }, { 0x80000000 | 4, 5 },
        }}),
        // Slot 'grabFocus'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'chooseBaseClass'
        QtMocHelpers::SlotData<void(const QString &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Slot 'slotNameChanged'
        QtMocHelpers::SlotData<void(const QString &)>(8, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Slot 'slotIncludeFileChanged'
        QtMocHelpers::SlotData<void(const QString &)>(9, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Slot 'slotAdd'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotReset'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<NewPromotedClassPanel, qt_meta_tag_ZN18qdesigner_internal21NewPromotedClassPanelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject qdesigner_internal::NewPromotedClassPanel::staticMetaObject = { {
    QMetaObject::SuperData::link<QGroupBox::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18qdesigner_internal21NewPromotedClassPanelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18qdesigner_internal21NewPromotedClassPanelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN18qdesigner_internal21NewPromotedClassPanelE_t>.metaTypes,
    nullptr
} };

void qdesigner_internal::NewPromotedClassPanel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<NewPromotedClassPanel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->newPromotedClass((*reinterpret_cast<std::add_pointer_t<PromotionParameters>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool*>>(_a[2]))); break;
        case 1: _t->grabFocus(); break;
        case 2: _t->chooseBaseClass((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 3: _t->slotNameChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 4: _t->slotIncludeFileChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 5: _t->slotAdd(); break;
        case 6: _t->slotReset(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (NewPromotedClassPanel::*)(const PromotionParameters & , bool * )>(_a, &NewPromotedClassPanel::newPromotedClass, 0))
            return;
    }
}

const QMetaObject *qdesigner_internal::NewPromotedClassPanel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *qdesigner_internal::NewPromotedClassPanel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18qdesigner_internal21NewPromotedClassPanelE_t>.strings))
        return static_cast<void*>(this);
    return QGroupBox::qt_metacast(_clname);
}

int qdesigner_internal::NewPromotedClassPanel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QGroupBox::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void qdesigner_internal::NewPromotedClassPanel::newPromotedClass(const PromotionParameters & _t1, bool * _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}
namespace {
struct qt_meta_tag_ZN18qdesigner_internal24QDesignerPromotionDialogE_t {};
} // unnamed namespace

template <> constexpr inline auto qdesigner_internal::QDesignerPromotionDialog::qt_create_metaobjectdata<qt_meta_tag_ZN18qdesigner_internal24QDesignerPromotionDialogE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "qdesigner_internal::QDesignerPromotionDialog",
        "selectedBaseClassChanged",
        "",
        "slotRemove",
        "slotAcceptPromoteTo",
        "slotSelectionChanged",
        "QItemSelection",
        "slotNewPromotedClass",
        "PromotionParameters",
        "bool*",
        "ok",
        "slotIncludeFileChanged",
        "QDesignerWidgetDataBaseItemInterface*",
        "includeFile",
        "slotClassNameChanged",
        "newName",
        "slotUpdateFromWidgetDatabase",
        "slotTreeViewContextMenu",
        "QPoint",
        "slotEditSignalsSlots"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'selectedBaseClassChanged'
        QtMocHelpers::SignalData<void(const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Slot 'slotRemove'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotAcceptPromoteTo'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotSelectionChanged'
        QtMocHelpers::SlotData<void(const QItemSelection &, const QItemSelection &)>(5, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 6, 2 }, { 0x80000000 | 6, 2 },
        }}),
        // Slot 'slotNewPromotedClass'
        QtMocHelpers::SlotData<void(const PromotionParameters &, bool *)>(7, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 8, 2 }, { 0x80000000 | 9, 10 },
        }}),
        // Slot 'slotIncludeFileChanged'
        QtMocHelpers::SlotData<void(QDesignerWidgetDataBaseItemInterface *, const QString &)>(11, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 12, 2 }, { QMetaType::QString, 13 },
        }}),
        // Slot 'slotClassNameChanged'
        QtMocHelpers::SlotData<void(QDesignerWidgetDataBaseItemInterface *, const QString &)>(14, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 12, 2 }, { QMetaType::QString, 15 },
        }}),
        // Slot 'slotUpdateFromWidgetDatabase'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotTreeViewContextMenu'
        QtMocHelpers::SlotData<void(const QPoint &)>(17, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 18, 2 },
        }}),
        // Slot 'slotEditSignalsSlots'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QDesignerPromotionDialog, qt_meta_tag_ZN18qdesigner_internal24QDesignerPromotionDialogE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject qdesigner_internal::QDesignerPromotionDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18qdesigner_internal24QDesignerPromotionDialogE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18qdesigner_internal24QDesignerPromotionDialogE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN18qdesigner_internal24QDesignerPromotionDialogE_t>.metaTypes,
    nullptr
} };

void qdesigner_internal::QDesignerPromotionDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QDesignerPromotionDialog *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->selectedBaseClassChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->slotRemove(); break;
        case 2: _t->slotAcceptPromoteTo(); break;
        case 3: _t->slotSelectionChanged((*reinterpret_cast<std::add_pointer_t<QItemSelection>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QItemSelection>>(_a[2]))); break;
        case 4: _t->slotNewPromotedClass((*reinterpret_cast<std::add_pointer_t<PromotionParameters>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool*>>(_a[2]))); break;
        case 5: _t->slotIncludeFileChanged((*reinterpret_cast<std::add_pointer_t<QDesignerWidgetDataBaseItemInterface*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 6: _t->slotClassNameChanged((*reinterpret_cast<std::add_pointer_t<QDesignerWidgetDataBaseItemInterface*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 7: _t->slotUpdateFromWidgetDatabase(); break;
        case 8: _t->slotTreeViewContextMenu((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 9: _t->slotEditSignalsSlots(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QDesignerPromotionDialog::*)(const QString & )>(_a, &QDesignerPromotionDialog::selectedBaseClassChanged, 0))
            return;
    }
}

const QMetaObject *qdesigner_internal::QDesignerPromotionDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *qdesigner_internal::QDesignerPromotionDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18qdesigner_internal24QDesignerPromotionDialogE_t>.strings))
        return static_cast<void*>(this);
    return QDialog::qt_metacast(_clname);
}

int qdesigner_internal::QDesignerPromotionDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 10;
    }
    return _id;
}

// SIGNAL 0
void qdesigner_internal::QDesignerPromotionDialog::selectedBaseClassChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}
QT_WARNING_POP
