/****************************************************************************
** Meta object code from reading C++ file 'mainwindow.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../qttools/src/linguist/linguist/mainwindow.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'mainwindow.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10MainWindowE_t {};
} // unnamed namespace

template <> constexpr inline auto MainWindow::qt_create_metaobjectdata<qt_meta_tag_ZN10MainWindowE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "MainWindow",
        "done",
        "",
        "doneAndNext",
        "prev",
        "next",
        "recentFileActivated",
        "QAction*",
        "action",
        "setupRecentFilesMenu",
        "open",
        "openAux",
        "saveAll",
        "save",
        "saveAs",
        "releaseAll",
        "release",
        "releaseAs",
        "closeFile",
        "closeAll",
        "showTranslateDialog",
        "showBatchTranslateDialog",
        "showTranslationSettings",
        "updateTranslateHit",
        "bool&",
        "hit",
        "translate",
        "mode",
        "newPhraseBook",
        "openPhraseBook",
        "closePhraseBook",
        "editPhraseBook",
        "addToPhraseBook",
        "manual",
        "resetSorting",
        "about",
        "aboutQt",
        "fileAboutToShow",
        "editAboutToShow",
        "showContextDock",
        "showMessagesDock",
        "showPhrasesDock",
        "showSourceCodeDock",
        "showErrorDock",
        "setupPhrase",
        "maybeSaveAll",
        "maybeSave",
        "model",
        "updateProgress",
        "maybeUpdateStatistics",
        "MultiDataIndex",
        "translationChanged",
        "updateCaption",
        "updateLatestModel",
        "QModelIndex",
        "index",
        "selectedContextChanged",
        "sortedIndex",
        "oldIndex",
        "selectedLabelChanged",
        "selectedMessageChanged",
        "setCurrentMessageFromGuess",
        "modelIndex",
        "Candidate",
        "tm",
        "contextAndLabelTabChanged",
        "updateTranslation",
        "translations",
        "updateTranslatorComment",
        "comment",
        "updateVisibleColumns",
        "updateActiveModel",
        "refreshItemViews",
        "toggleFinished",
        "openMachineTranslateDialog",
        "prevUnfinished",
        "nextUnfinished",
        "findNext",
        "text",
        "DataModel::FindLocation",
        "where",
        "FindDialog::FindOptions",
        "options",
        "statusFilter",
        "revalidate",
        "showStatistics",
        "toggleQmlPreview",
        "toggleVisualizeWhitespace",
        "onWhatsThis",
        "updatePhraseDicts",
        "updatePhraseDict",
        "print",
        "printPhraseBook"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'done'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'doneAndNext'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'prev'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'next'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'recentFileActivated'
        QtMocHelpers::SlotData<void(QAction *)>(6, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Slot 'setupRecentFilesMenu'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'open'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'openAux'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'saveAll'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'save'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'saveAs'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'releaseAll'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'release'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'releaseAs'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'closeFile'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'closeAll'
        QtMocHelpers::SlotData<bool()>(19, 2, QMC::AccessPrivate, QMetaType::Bool),
        // Slot 'showTranslateDialog'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'showBatchTranslateDialog'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'showTranslationSettings'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateTranslateHit'
        QtMocHelpers::SlotData<void(bool &)>(23, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 24, 25 },
        }}),
        // Slot 'translate'
        QtMocHelpers::SlotData<void(int)>(26, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 27 },
        }}),
        // Slot 'newPhraseBook'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'openPhraseBook'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'closePhraseBook'
        QtMocHelpers::SlotData<void(QAction *)>(30, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Slot 'editPhraseBook'
        QtMocHelpers::SlotData<void(QAction *)>(31, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Slot 'addToPhraseBook'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'manual'
        QtMocHelpers::SlotData<void()>(33, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'resetSorting'
        QtMocHelpers::SlotData<void()>(34, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'about'
        QtMocHelpers::SlotData<void()>(35, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'aboutQt'
        QtMocHelpers::SlotData<void()>(36, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'fileAboutToShow'
        QtMocHelpers::SlotData<void()>(37, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'editAboutToShow'
        QtMocHelpers::SlotData<void()>(38, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'showContextDock'
        QtMocHelpers::SlotData<void()>(39, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'showMessagesDock'
        QtMocHelpers::SlotData<void()>(40, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'showPhrasesDock'
        QtMocHelpers::SlotData<void()>(41, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'showSourceCodeDock'
        QtMocHelpers::SlotData<void()>(42, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'showErrorDock'
        QtMocHelpers::SlotData<void()>(43, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'setupPhrase'
        QtMocHelpers::SlotData<void()>(44, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'maybeSaveAll'
        QtMocHelpers::SlotData<bool()>(45, 2, QMC::AccessPrivate, QMetaType::Bool),
        // Slot 'maybeSave'
        QtMocHelpers::SlotData<bool(int)>(46, 2, QMC::AccessPrivate, QMetaType::Bool, {{
            { QMetaType::Int, 47 },
        }}),
        // Slot 'updateProgress'
        QtMocHelpers::SlotData<void()>(48, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'maybeUpdateStatistics'
        QtMocHelpers::SlotData<void(const MultiDataIndex &)>(49, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 50, 2 },
        }}),
        // Slot 'translationChanged'
        QtMocHelpers::SlotData<void(const MultiDataIndex &)>(51, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 50, 2 },
        }}),
        // Slot 'updateCaption'
        QtMocHelpers::SlotData<void()>(52, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateLatestModel'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(53, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 54, 55 },
        }}),
        // Slot 'selectedContextChanged'
        QtMocHelpers::SlotData<void(const QModelIndex &, const QModelIndex &)>(56, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 54, 57 }, { 0x80000000 | 54, 58 },
        }}),
        // Slot 'selectedLabelChanged'
        QtMocHelpers::SlotData<void(const QModelIndex &, const QModelIndex &)>(59, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 54, 57 }, { 0x80000000 | 54, 58 },
        }}),
        // Slot 'selectedMessageChanged'
        QtMocHelpers::SlotData<void(const QModelIndex &, const QModelIndex &)>(60, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 54, 57 }, { 0x80000000 | 54, 58 },
        }}),
        // Slot 'setCurrentMessageFromGuess'
        QtMocHelpers::SlotData<void(int, const Candidate &)>(61, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 62 }, { 0x80000000 | 63, 64 },
        }}),
        // Slot 'contextAndLabelTabChanged'
        QtMocHelpers::SlotData<void()>(65, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateTranslation'
        QtMocHelpers::SlotData<void(const QStringList &)>(66, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QStringList, 67 },
        }}),
        // Slot 'updateTranslatorComment'
        QtMocHelpers::SlotData<void(const QString &)>(68, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 69 },
        }}),
        // Slot 'updateVisibleColumns'
        QtMocHelpers::SlotData<void()>(70, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateActiveModel'
        QtMocHelpers::SlotData<void(int)>(71, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'refreshItemViews'
        QtMocHelpers::SlotData<void()>(72, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'toggleFinished'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(73, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 54, 55 },
        }}),
        // Slot 'openMachineTranslateDialog'
        QtMocHelpers::SlotData<void()>(74, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'prevUnfinished'
        QtMocHelpers::SlotData<void()>(75, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'nextUnfinished'
        QtMocHelpers::SlotData<void()>(76, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'findNext'
        QtMocHelpers::SlotData<void(const QString &, DataModel::FindLocation, FindDialog::FindOptions, int)>(77, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 78 }, { 0x80000000 | 79, 80 }, { 0x80000000 | 81, 82 }, { QMetaType::Int, 83 },
        }}),
        // Slot 'revalidate'
        QtMocHelpers::SlotData<void()>(84, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'showStatistics'
        QtMocHelpers::SlotData<void()>(85, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'toggleQmlPreview'
        QtMocHelpers::SlotData<void()>(86, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'toggleVisualizeWhitespace'
        QtMocHelpers::SlotData<void()>(87, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onWhatsThis'
        QtMocHelpers::SlotData<void()>(88, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updatePhraseDicts'
        QtMocHelpers::SlotData<void()>(89, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updatePhraseDict'
        QtMocHelpers::SlotData<void(int)>(90, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 47 },
        }}),
        // Slot 'print'
        QtMocHelpers::SlotData<void()>(91, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'printPhraseBook'
        QtMocHelpers::SlotData<void(QAction *)>(92, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<MainWindow, qt_meta_tag_ZN10MainWindowE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject MainWindow::staticMetaObject = { {
    QMetaObject::SuperData::link<QMainWindow::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10MainWindowE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10MainWindowE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10MainWindowE_t>.metaTypes,
    nullptr
} };

void MainWindow::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<MainWindow *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->done(); break;
        case 1: _t->doneAndNext(); break;
        case 2: _t->prev(); break;
        case 3: _t->next(); break;
        case 4: _t->recentFileActivated((*reinterpret_cast<std::add_pointer_t<QAction*>>(_a[1]))); break;
        case 5: _t->setupRecentFilesMenu(); break;
        case 6: _t->open(); break;
        case 7: _t->openAux(); break;
        case 8: _t->saveAll(); break;
        case 9: _t->save(); break;
        case 10: _t->saveAs(); break;
        case 11: _t->releaseAll(); break;
        case 12: _t->release(); break;
        case 13: _t->releaseAs(); break;
        case 14: _t->closeFile(); break;
        case 15: { bool _r = _t->closeAll();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 16: _t->showTranslateDialog(); break;
        case 17: _t->showBatchTranslateDialog(); break;
        case 18: _t->showTranslationSettings(); break;
        case 19: _t->updateTranslateHit((*reinterpret_cast<std::add_pointer_t<bool&>>(_a[1]))); break;
        case 20: _t->translate((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 21: _t->newPhraseBook(); break;
        case 22: _t->openPhraseBook(); break;
        case 23: _t->closePhraseBook((*reinterpret_cast<std::add_pointer_t<QAction*>>(_a[1]))); break;
        case 24: _t->editPhraseBook((*reinterpret_cast<std::add_pointer_t<QAction*>>(_a[1]))); break;
        case 25: _t->addToPhraseBook(); break;
        case 26: _t->manual(); break;
        case 27: _t->resetSorting(); break;
        case 28: _t->about(); break;
        case 29: _t->aboutQt(); break;
        case 30: _t->fileAboutToShow(); break;
        case 31: _t->editAboutToShow(); break;
        case 32: _t->showContextDock(); break;
        case 33: _t->showMessagesDock(); break;
        case 34: _t->showPhrasesDock(); break;
        case 35: _t->showSourceCodeDock(); break;
        case 36: _t->showErrorDock(); break;
        case 37: _t->setupPhrase(); break;
        case 38: { bool _r = _t->maybeSaveAll();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 39: { bool _r = _t->maybeSave((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 40: _t->updateProgress(); break;
        case 41: _t->maybeUpdateStatistics((*reinterpret_cast<std::add_pointer_t<MultiDataIndex>>(_a[1]))); break;
        case 42: _t->translationChanged((*reinterpret_cast<std::add_pointer_t<MultiDataIndex>>(_a[1]))); break;
        case 43: _t->updateCaption(); break;
        case 44: _t->updateLatestModel((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 45: _t->selectedContextChanged((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[2]))); break;
        case 46: _t->selectedLabelChanged((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[2]))); break;
        case 47: _t->selectedMessageChanged((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[2]))); break;
        case 48: _t->setCurrentMessageFromGuess((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Candidate>>(_a[2]))); break;
        case 49: _t->contextAndLabelTabChanged(); break;
        case 50: _t->updateTranslation((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 51: _t->updateTranslatorComment((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 52: _t->updateVisibleColumns(); break;
        case 53: _t->updateActiveModel((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 54: _t->refreshItemViews(); break;
        case 55: _t->toggleFinished((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 56: _t->openMachineTranslateDialog(); break;
        case 57: _t->prevUnfinished(); break;
        case 58: _t->nextUnfinished(); break;
        case 59: _t->findNext((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<DataModel::FindLocation>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<FindDialog::FindOptions>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[4]))); break;
        case 60: _t->revalidate(); break;
        case 61: _t->showStatistics(); break;
        case 62: _t->toggleQmlPreview(); break;
        case 63: _t->toggleVisualizeWhitespace(); break;
        case 64: _t->onWhatsThis(); break;
        case 65: _t->updatePhraseDicts(); break;
        case 66: _t->updatePhraseDict((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 67: _t->print(); break;
        case 68: _t->printPhraseBook((*reinterpret_cast<std::add_pointer_t<QAction*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAction* >(); break;
            }
            break;
        case 23:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAction* >(); break;
            }
            break;
        case 24:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAction* >(); break;
            }
            break;
        case 68:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAction* >(); break;
            }
            break;
        }
    }
}

const QMetaObject *MainWindow::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MainWindow::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10MainWindowE_t>.strings))
        return static_cast<void*>(this);
    return QMainWindow::qt_metacast(_clname);
}

int MainWindow::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QMainWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 69)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 69;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 69)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 69;
    }
    return _id;
}
QT_WARNING_POP
