/****************************************************************************
** Meta object code from reading C++ file 'widgetboxcategorylistview.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../../../qttools/src/designer/src/components/widgetbox/widgetboxcategorylistview.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'widgetboxcategorylistview.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN18qdesigner_internal25WidgetBoxCategoryListViewE_t {};
} // unnamed namespace

template <> constexpr inline auto qdesigner_internal::WidgetBoxCategoryListView::qt_create_metaobjectdata<qt_meta_tag_ZN18qdesigner_internal25WidgetBoxCategoryListViewE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "qdesigner_internal::WidgetBoxCategoryListView",
        "scratchPadChanged",
        "",
        "widgetBoxPressed",
        "name",
        "xml",
        "QPoint",
        "globalPos",
        "itemRemoved",
        "lastItemRemoved",
        "filter",
        "needle",
        "Qt::CaseSensitivity",
        "caseSensitivity",
        "removeCurrentItem",
        "editCurrentItem",
        "slotPressed",
        "QModelIndex",
        "index"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'scratchPadChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'widgetBoxPressed'
        QtMocHelpers::SignalData<void(const QString &, const QString &, const QPoint &)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 4 }, { QMetaType::QString, 5 }, { 0x80000000 | 6, 7 },
        }}),
        // Signal 'itemRemoved'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'lastItemRemoved'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'filter'
        QtMocHelpers::SlotData<void(const QString &, Qt::CaseSensitivity)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 11 }, { 0x80000000 | 12, 13 },
        }}),
        // Slot 'removeCurrentItem'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'editCurrentItem'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotPressed'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(16, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 17, 18 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<WidgetBoxCategoryListView, qt_meta_tag_ZN18qdesigner_internal25WidgetBoxCategoryListViewE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject qdesigner_internal::WidgetBoxCategoryListView::staticMetaObject = { {
    QMetaObject::SuperData::link<QListView::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18qdesigner_internal25WidgetBoxCategoryListViewE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18qdesigner_internal25WidgetBoxCategoryListViewE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN18qdesigner_internal25WidgetBoxCategoryListViewE_t>.metaTypes,
    nullptr
} };

void qdesigner_internal::WidgetBoxCategoryListView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<WidgetBoxCategoryListView *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->scratchPadChanged(); break;
        case 1: _t->widgetBoxPressed((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[3]))); break;
        case 2: _t->itemRemoved(); break;
        case 3: _t->lastItemRemoved(); break;
        case 4: _t->filter((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Qt::CaseSensitivity>>(_a[2]))); break;
        case 5: _t->removeCurrentItem(); break;
        case 6: _t->editCurrentItem(); break;
        case 7: _t->slotPressed((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (WidgetBoxCategoryListView::*)()>(_a, &WidgetBoxCategoryListView::scratchPadChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (WidgetBoxCategoryListView::*)(const QString & , const QString & , const QPoint & )>(_a, &WidgetBoxCategoryListView::widgetBoxPressed, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (WidgetBoxCategoryListView::*)()>(_a, &WidgetBoxCategoryListView::itemRemoved, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (WidgetBoxCategoryListView::*)()>(_a, &WidgetBoxCategoryListView::lastItemRemoved, 3))
            return;
    }
}

const QMetaObject *qdesigner_internal::WidgetBoxCategoryListView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *qdesigner_internal::WidgetBoxCategoryListView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18qdesigner_internal25WidgetBoxCategoryListViewE_t>.strings))
        return static_cast<void*>(this);
    return QListView::qt_metacast(_clname);
}

int qdesigner_internal::WidgetBoxCategoryListView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QListView::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void qdesigner_internal::WidgetBoxCategoryListView::scratchPadChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void qdesigner_internal::WidgetBoxCategoryListView::widgetBoxPressed(const QString & _t1, const QString & _t2, const QPoint & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2, _t3);
}

// SIGNAL 2
void qdesigner_internal::WidgetBoxCategoryListView::itemRemoved()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void qdesigner_internal::WidgetBoxCategoryListView::lastItemRemoved()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}
QT_WARNING_POP
