/****************************************************************************
** Meta object code from reading C++ file 'qdesigner_actions.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../../qttools/src/designer/src/designer/qdesigner_actions.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qdesigner_actions.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN16QDesignerActionsE_t {};
} // unnamed namespace

template <> constexpr inline auto QDesignerActions::qt_create_metaobjectdata<qt_meta_tag_ZN16QDesignerActionsE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QDesignerActions",
        "useBigIcons",
        "",
        "activeFormWindowChanged",
        "QDesignerFormWindowInterface*",
        "formWindow",
        "createForm",
        "slotOpenForm",
        "helpRequested",
        "manual",
        "document",
        "saveForm",
        "saveFormAs",
        "saveAllForms",
        "saveFormAsTemplate",
        "notImplementedYet",
        "shutdown",
        "editWidgetsSlot",
        "openRecentForm",
        "clearRecentFiles",
        "closeForm",
        "showDesignerHelp",
        "aboutDesigner",
        "showWidgetSpecificHelp",
        "backupForms",
        "showNewFormDialog",
        "fileName",
        "showPreferencesDialog",
        "showAppFontDialog",
        "savePreviewImage",
        "printPreviewImage",
        "updateCloseAction",
        "formWindowCountChanged",
        "formWindowSettingsChanged",
        "fw"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'useBigIcons'
        QtMocHelpers::SignalData<void(bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'activeFormWindowChanged'
        QtMocHelpers::SlotData<void(QDesignerFormWindowInterface *)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 },
        }}),
        // Slot 'createForm'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotOpenForm'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'helpRequested'
        QtMocHelpers::SlotData<void(const QString &, const QString &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 9 }, { QMetaType::QString, 10 },
        }}),
        // Slot 'saveForm'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'saveFormAs'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'saveAllForms'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'saveFormAsTemplate'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'notImplementedYet'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'shutdown'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'editWidgetsSlot'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'openRecentForm'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'clearRecentFiles'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'closeForm'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'showDesignerHelp'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'aboutDesigner'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'showWidgetSpecificHelp'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'backupForms'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'showNewFormDialog'
        QtMocHelpers::SlotData<void(const QString &)>(25, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 26 },
        }}),
        // Slot 'showPreferencesDialog'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'showAppFontDialog'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'savePreviewImage'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'printPreviewImage'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateCloseAction'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'formWindowCountChanged'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'formWindowSettingsChanged'
        QtMocHelpers::SlotData<void(QDesignerFormWindowInterface *)>(33, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 4, 34 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QDesignerActions, qt_meta_tag_ZN16QDesignerActionsE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QDesignerActions::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16QDesignerActionsE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16QDesignerActionsE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16QDesignerActionsE_t>.metaTypes,
    nullptr
} };

void QDesignerActions::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QDesignerActions *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->useBigIcons((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->activeFormWindowChanged((*reinterpret_cast<std::add_pointer_t<QDesignerFormWindowInterface*>>(_a[1]))); break;
        case 2: _t->createForm(); break;
        case 3: _t->slotOpenForm(); break;
        case 4: _t->helpRequested((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 5: _t->saveForm(); break;
        case 6: _t->saveFormAs(); break;
        case 7: _t->saveAllForms(); break;
        case 8: _t->saveFormAsTemplate(); break;
        case 9: _t->notImplementedYet(); break;
        case 10: _t->shutdown(); break;
        case 11: _t->editWidgetsSlot(); break;
        case 12: _t->openRecentForm(); break;
        case 13: _t->clearRecentFiles(); break;
        case 14: _t->closeForm(); break;
        case 15: _t->showDesignerHelp(); break;
        case 16: _t->aboutDesigner(); break;
        case 17: _t->showWidgetSpecificHelp(); break;
        case 18: _t->backupForms(); break;
        case 19: _t->showNewFormDialog((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 20: _t->showPreferencesDialog(); break;
        case 21: _t->showAppFontDialog(); break;
        case 22: _t->savePreviewImage(); break;
        case 23: _t->printPreviewImage(); break;
        case 24: _t->updateCloseAction(); break;
        case 25: _t->formWindowCountChanged(); break;
        case 26: _t->formWindowSettingsChanged((*reinterpret_cast<std::add_pointer_t<QDesignerFormWindowInterface*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QDesignerActions::*)(bool )>(_a, &QDesignerActions::useBigIcons, 0))
            return;
    }
}

const QMetaObject *QDesignerActions::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QDesignerActions::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16QDesignerActionsE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QDesignerActions::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 27)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 27;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 27)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 27;
    }
    return _id;
}

// SIGNAL 0
void QDesignerActions::useBigIcons(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}
QT_WARNING_POP
