/********************************************************************************
** Form generated from reading UI file 'preferencesdialog.ui'
**
** Created by: Qt User Interface Compiler version 6.12.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PREFERENCESDIALOG_H
#define UI_PREFERENCESDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QStackedWidget>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "helpdocsettingswidget.h"
#include "qhelpfiltersettingswidget.h"

QT_BEGIN_NAMESPACE

class Ui_PreferencesDialogClass
{
public:
    QVBoxLayout *verticalLayout_6;
    QTabWidget *tabWidget;
    QWidget *fontsTab;
    QGridLayout *gridLayout;
    QHBoxLayout *hboxLayout;
    QLabel *fontLabel;
    QComboBox *comboBox;
    QStackedWidget *stackedWidget_2;
    QWidget *page_4;
    QWidget *filtersTab;
    QVBoxLayout *verticalLayout_5;
    QHelpFilterSettingsWidget *filterSettingsWidget;
    QWidget *docsTab;
    QVBoxLayout *verticalLayout_4;
    HelpDocSettingsWidget *docSettingsWidget;
    QWidget *optionsTab;
    QVBoxLayout *verticalLayout_3;
    QGroupBox *groupBox_2;
    QHBoxLayout *horizontalLayout_3;
    QLabel *label_5;
    QComboBox *helpStartComboBox;
    QSpacerItem *horizontalSpacer_3;
    QGroupBox *groupBox;
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout_2;
    QLabel *label_4;
    QLineEdit *homePageLineEdit;
    QHBoxLayout *horizontalLayout;
    QSpacerItem *horizontalSpacer;
    QPushButton *currentPageButton;
    QPushButton *blankPageButton;
    QPushButton *defaultPageButton;
    QGroupBox *groupBox_3;
    QVBoxLayout *verticalLayout_2;
    QCheckBox *showTabs;
    QSpacerItem *verticalSpacer_2;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *PreferencesDialogClass)
    {
        if (PreferencesDialogClass->objectName().isEmpty())
            PreferencesDialogClass->setObjectName("PreferencesDialogClass");
        PreferencesDialogClass->resize(395, 376);
        verticalLayout_6 = new QVBoxLayout(PreferencesDialogClass);
        verticalLayout_6->setSpacing(6);
        verticalLayout_6->setContentsMargins(11, 11, 11, 11);
        verticalLayout_6->setObjectName("verticalLayout_6");
        tabWidget = new QTabWidget(PreferencesDialogClass);
        tabWidget->setObjectName("tabWidget");
        fontsTab = new QWidget();
        fontsTab->setObjectName("fontsTab");
        gridLayout = new QGridLayout(fontsTab);
        gridLayout->setSpacing(6);
        gridLayout->setContentsMargins(11, 11, 11, 11);
        gridLayout->setObjectName("gridLayout");
        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(6);
        hboxLayout->setObjectName("hboxLayout");
        fontLabel = new QLabel(fontsTab);
        fontLabel->setObjectName("fontLabel");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Maximum, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(fontLabel->sizePolicy().hasHeightForWidth());
        fontLabel->setSizePolicy(sizePolicy);

        hboxLayout->addWidget(fontLabel);

        comboBox = new QComboBox(fontsTab);
        comboBox->addItem(QString());
        comboBox->addItem(QString());
        comboBox->setObjectName("comboBox");

        hboxLayout->addWidget(comboBox);


        gridLayout->addLayout(hboxLayout, 0, 0, 1, 1);

        stackedWidget_2 = new QStackedWidget(fontsTab);
        stackedWidget_2->setObjectName("stackedWidget_2");
        page_4 = new QWidget();
        page_4->setObjectName("page_4");
        stackedWidget_2->addWidget(page_4);

        gridLayout->addWidget(stackedWidget_2, 1, 0, 1, 1);

        tabWidget->addTab(fontsTab, QString());
        filtersTab = new QWidget();
        filtersTab->setObjectName("filtersTab");
        verticalLayout_5 = new QVBoxLayout(filtersTab);
        verticalLayout_5->setSpacing(6);
        verticalLayout_5->setContentsMargins(11, 11, 11, 11);
        verticalLayout_5->setObjectName("verticalLayout_5");
        verticalLayout_5->setContentsMargins(0, 0, 0, 0);
        filterSettingsWidget = new QHelpFilterSettingsWidget(filtersTab);
        filterSettingsWidget->setObjectName("filterSettingsWidget");

        verticalLayout_5->addWidget(filterSettingsWidget);

        tabWidget->addTab(filtersTab, QString());
        docsTab = new QWidget();
        docsTab->setObjectName("docsTab");
        verticalLayout_4 = new QVBoxLayout(docsTab);
        verticalLayout_4->setSpacing(6);
        verticalLayout_4->setContentsMargins(11, 11, 11, 11);
        verticalLayout_4->setObjectName("verticalLayout_4");
        verticalLayout_4->setContentsMargins(0, 0, 0, 0);
        docSettingsWidget = new HelpDocSettingsWidget(docsTab);
        docSettingsWidget->setObjectName("docSettingsWidget");

        verticalLayout_4->addWidget(docSettingsWidget);

        tabWidget->addTab(docsTab, QString());
        optionsTab = new QWidget();
        optionsTab->setObjectName("optionsTab");
        verticalLayout_3 = new QVBoxLayout(optionsTab);
        verticalLayout_3->setSpacing(6);
        verticalLayout_3->setContentsMargins(11, 11, 11, 11);
        verticalLayout_3->setObjectName("verticalLayout_3");
        groupBox_2 = new QGroupBox(optionsTab);
        groupBox_2->setObjectName("groupBox_2");
        horizontalLayout_3 = new QHBoxLayout(groupBox_2);
        horizontalLayout_3->setSpacing(6);
        horizontalLayout_3->setContentsMargins(11, 11, 11, 11);
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        label_5 = new QLabel(groupBox_2);
        label_5->setObjectName("label_5");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(label_5->sizePolicy().hasHeightForWidth());
        label_5->setSizePolicy(sizePolicy1);

        horizontalLayout_3->addWidget(label_5);

        helpStartComboBox = new QComboBox(groupBox_2);
        helpStartComboBox->addItem(QString());
        helpStartComboBox->addItem(QString());
        helpStartComboBox->addItem(QString());
        helpStartComboBox->setObjectName("helpStartComboBox");
        sizePolicy1.setHeightForWidth(helpStartComboBox->sizePolicy().hasHeightForWidth());
        helpStartComboBox->setSizePolicy(sizePolicy1);

        horizontalLayout_3->addWidget(helpStartComboBox);

        horizontalSpacer_3 = new QSpacerItem(54, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_3->addItem(horizontalSpacer_3);


        verticalLayout_3->addWidget(groupBox_2);

        groupBox = new QGroupBox(optionsTab);
        groupBox->setObjectName("groupBox");
        verticalLayout = new QVBoxLayout(groupBox);
        verticalLayout->setSpacing(6);
        verticalLayout->setContentsMargins(11, 11, 11, 11);
        verticalLayout->setObjectName("verticalLayout");
        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setSpacing(6);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        label_4 = new QLabel(groupBox);
        label_4->setObjectName("label_4");

        horizontalLayout_2->addWidget(label_4);

        homePageLineEdit = new QLineEdit(groupBox);
        homePageLineEdit->setObjectName("homePageLineEdit");

        horizontalLayout_2->addWidget(homePageLineEdit);


        verticalLayout->addLayout(horizontalLayout_2);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setSpacing(6);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        currentPageButton = new QPushButton(groupBox);
        currentPageButton->setObjectName("currentPageButton");

        horizontalLayout->addWidget(currentPageButton);

        blankPageButton = new QPushButton(groupBox);
        blankPageButton->setObjectName("blankPageButton");

        horizontalLayout->addWidget(blankPageButton);

        defaultPageButton = new QPushButton(groupBox);
        defaultPageButton->setObjectName("defaultPageButton");

        horizontalLayout->addWidget(defaultPageButton);


        verticalLayout->addLayout(horizontalLayout);


        verticalLayout_3->addWidget(groupBox);

        groupBox_3 = new QGroupBox(optionsTab);
        groupBox_3->setObjectName("groupBox_3");
        verticalLayout_2 = new QVBoxLayout(groupBox_3);
        verticalLayout_2->setSpacing(6);
        verticalLayout_2->setContentsMargins(11, 11, 11, 11);
        verticalLayout_2->setObjectName("verticalLayout_2");
        showTabs = new QCheckBox(groupBox_3);
        showTabs->setObjectName("showTabs");

        verticalLayout_2->addWidget(showTabs);


        verticalLayout_3->addWidget(groupBox_3);

        verticalSpacer_2 = new QSpacerItem(20, 72, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_3->addItem(verticalSpacer_2);

        tabWidget->addTab(optionsTab, QString());

        verticalLayout_6->addWidget(tabWidget);

        buttonBox = new QDialogButtonBox(PreferencesDialogClass);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setStandardButtons(QDialogButtonBox::Apply|QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout_6->addWidget(buttonBox);


        retranslateUi(PreferencesDialogClass);
        QObject::connect(comboBox, &QComboBox::currentIndexChanged, stackedWidget_2, &QStackedWidget::setCurrentIndex);

        tabWidget->setCurrentIndex(2);
        stackedWidget_2->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(PreferencesDialogClass);
    } // setupUi

    void retranslateUi(QDialog *PreferencesDialogClass)
    {
        PreferencesDialogClass->setWindowTitle(QCoreApplication::translate("PreferencesDialogClass", "Preferences", nullptr));
        fontLabel->setText(QCoreApplication::translate("PreferencesDialogClass", "Font settings:", nullptr));
        comboBox->setItemText(0, QCoreApplication::translate("PreferencesDialogClass", "Browser", nullptr));
        comboBox->setItemText(1, QCoreApplication::translate("PreferencesDialogClass", "Application", nullptr));

        tabWidget->setTabText(tabWidget->indexOf(fontsTab), QCoreApplication::translate("PreferencesDialogClass", "Fonts", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(filtersTab), QCoreApplication::translate("PreferencesDialogClass", "Filters", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(docsTab), QCoreApplication::translate("PreferencesDialogClass", "Documentation", nullptr));
        groupBox_2->setTitle(QString());
        label_5->setText(QCoreApplication::translate("PreferencesDialogClass", "On help start:", nullptr));
        helpStartComboBox->setItemText(0, QCoreApplication::translate("PreferencesDialogClass", "Show my home page", nullptr));
        helpStartComboBox->setItemText(1, QCoreApplication::translate("PreferencesDialogClass", "Show a blank page", nullptr));
        helpStartComboBox->setItemText(2, QCoreApplication::translate("PreferencesDialogClass", "Show my tabs from last session", nullptr));

        groupBox->setTitle(QString());
        label_4->setText(QCoreApplication::translate("PreferencesDialogClass", "Homepage", nullptr));
        currentPageButton->setText(QCoreApplication::translate("PreferencesDialogClass", "Current Page", nullptr));
        blankPageButton->setText(QCoreApplication::translate("PreferencesDialogClass", "Blank Page", nullptr));
        defaultPageButton->setText(QCoreApplication::translate("PreferencesDialogClass", "Restore to default", nullptr));
        groupBox_3->setTitle(QCoreApplication::translate("PreferencesDialogClass", "Appearance", nullptr));
        showTabs->setText(QCoreApplication::translate("PreferencesDialogClass", "Show tabs for each individual page", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(optionsTab), QCoreApplication::translate("PreferencesDialogClass", "Options", nullptr));
    } // retranslateUi

};

namespace Ui {
    class PreferencesDialogClass: public Ui_PreferencesDialogClass {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_PREFERENCESDIALOG_H
