/****************************************************************************
** Meta object code from reading C++ file 'designerpropertymanager.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../../../qttools/src/designer/src/components/propertyeditor/designerpropertymanager.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'designerpropertymanager.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN18qdesigner_internal23DesignerPropertyManagerE_t {};
} // unnamed namespace

template <> constexpr inline auto qdesigner_internal::DesignerPropertyManager::qt_create_metaobjectdata<qt_meta_tag_ZN18qdesigner_internal23DesignerPropertyManagerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "qdesigner_internal::DesignerPropertyManager",
        "valueChanged2",
        "",
        "QtProperty*",
        "property",
        "QVariant",
        "value",
        "enableSubPropertyHandling",
        "setAttribute",
        "attribute",
        "setValue",
        "slotValueChanged",
        "slotPropertyDestroyed"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'valueChanged2'
        QtMocHelpers::SignalData<void(QtProperty *, const QVariant &, bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 5, 6 }, { QMetaType::Bool, 7 },
        }}),
        // Slot 'setAttribute'
        QtMocHelpers::SlotData<void(QtProperty *, const QString &, const QVariant &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::QString, 9 }, { 0x80000000 | 5, 6 },
        }}),
        // Slot 'setValue'
        QtMocHelpers::SlotData<void(QtProperty *, const QVariant &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 5, 6 },
        }}),
        // Slot 'slotValueChanged'
        QtMocHelpers::SlotData<void(QtProperty *, const QVariant &)>(11, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 5, 6 },
        }}),
        // Slot 'slotPropertyDestroyed'
        QtMocHelpers::SlotData<void(QtProperty *)>(12, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DesignerPropertyManager, qt_meta_tag_ZN18qdesigner_internal23DesignerPropertyManagerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject qdesigner_internal::DesignerPropertyManager::staticMetaObject = { {
    QMetaObject::SuperData::link<QtVariantPropertyManager::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18qdesigner_internal23DesignerPropertyManagerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18qdesigner_internal23DesignerPropertyManagerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN18qdesigner_internal23DesignerPropertyManagerE_t>.metaTypes,
    nullptr
} };

void qdesigner_internal::DesignerPropertyManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DesignerPropertyManager *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->valueChanged2((*reinterpret_cast<std::add_pointer_t<QtProperty*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 1: _t->setAttribute((*reinterpret_cast<std::add_pointer_t<QtProperty*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[3]))); break;
        case 2: _t->setValue((*reinterpret_cast<std::add_pointer_t<QtProperty*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 3: _t->slotValueChanged((*reinterpret_cast<std::add_pointer_t<QtProperty*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 4: _t->slotPropertyDestroyed((*reinterpret_cast<std::add_pointer_t<QtProperty*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DesignerPropertyManager::*)(QtProperty * , const QVariant & , bool )>(_a, &DesignerPropertyManager::valueChanged2, 0))
            return;
    }
}

const QMetaObject *qdesigner_internal::DesignerPropertyManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *qdesigner_internal::DesignerPropertyManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18qdesigner_internal23DesignerPropertyManagerE_t>.strings))
        return static_cast<void*>(this);
    return QtVariantPropertyManager::qt_metacast(_clname);
}

int qdesigner_internal::DesignerPropertyManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QtVariantPropertyManager::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void qdesigner_internal::DesignerPropertyManager::valueChanged2(QtProperty * _t1, const QVariant & _t2, bool _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2, _t3);
}
namespace {
struct qt_meta_tag_ZN18qdesigner_internal21DesignerEditorFactoryE_t {};
} // unnamed namespace

template <> constexpr inline auto qdesigner_internal::DesignerEditorFactory::qt_create_metaobjectdata<qt_meta_tag_ZN18qdesigner_internal21DesignerEditorFactoryE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "qdesigner_internal::DesignerEditorFactory",
        "resetProperty",
        "",
        "QtProperty*",
        "property",
        "slotEditorDestroyed",
        "object",
        "slotAttributeChanged",
        "attribute",
        "QVariant",
        "value",
        "slotPropertyChanged",
        "slotValueChanged",
        "slotStringTextChanged",
        "prop",
        "slotKeySequenceChanged",
        "QKeySequence",
        "slotPaletteChanged",
        "QPalette",
        "slotPixmapChanged",
        "slotIconChanged",
        "slotIconThemeChanged",
        "slotIconThemeEnumChanged",
        "slotUintChanged",
        "slotIntChanged",
        "slotLongLongChanged",
        "slotULongLongChanged",
        "slotUrlChanged",
        "slotByteArrayChanged",
        "slotStringListChanged"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'resetProperty'
        QtMocHelpers::SignalData<void(QtProperty *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'slotEditorDestroyed'
        QtMocHelpers::SlotData<void(QtProperty *, QObject *)>(5, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::QObjectStar, 6 },
        }}),
        // Slot 'slotAttributeChanged'
        QtMocHelpers::SlotData<void(QtProperty *, const QString &, const QVariant &)>(7, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::QString, 8 }, { 0x80000000 | 9, 10 },
        }}),
        // Slot 'slotPropertyChanged'
        QtMocHelpers::SlotData<void(QtProperty *)>(11, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'slotValueChanged'
        QtMocHelpers::SlotData<void(QtProperty *, const QVariant &)>(12, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 9, 10 },
        }}),
        // Slot 'slotStringTextChanged'
        QtMocHelpers::SlotData<void(QtProperty *, const QString &)>(13, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 14 }, { QMetaType::QString, 10 },
        }}),
        // Slot 'slotKeySequenceChanged'
        QtMocHelpers::SlotData<void(QtProperty *, const QKeySequence &)>(15, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 14 }, { 0x80000000 | 16, 10 },
        }}),
        // Slot 'slotPaletteChanged'
        QtMocHelpers::SlotData<void(QtProperty *, const QPalette &)>(17, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 14 }, { 0x80000000 | 18, 10 },
        }}),
        // Slot 'slotPixmapChanged'
        QtMocHelpers::SlotData<void(QtProperty *, const QString &)>(19, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 14 }, { QMetaType::QString, 10 },
        }}),
        // Slot 'slotIconChanged'
        QtMocHelpers::SlotData<void(QtProperty *, const QString &)>(20, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 14 }, { QMetaType::QString, 10 },
        }}),
        // Slot 'slotIconThemeChanged'
        QtMocHelpers::SlotData<void(QtProperty *, const QString &)>(21, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 14 }, { QMetaType::QString, 10 },
        }}),
        // Slot 'slotIconThemeEnumChanged'
        QtMocHelpers::SlotData<void(QtProperty *, int)>(22, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 14 }, { QMetaType::Int, 10 },
        }}),
        // Slot 'slotUintChanged'
        QtMocHelpers::SlotData<void(QtProperty *, const QString &)>(23, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 14 }, { QMetaType::QString, 10 },
        }}),
        // Slot 'slotIntChanged'
        QtMocHelpers::SlotData<void(QtProperty *, int)>(24, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 14 }, { QMetaType::Int, 2 },
        }}),
        // Slot 'slotLongLongChanged'
        QtMocHelpers::SlotData<void(QtProperty *, const QString &)>(25, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 14 }, { QMetaType::QString, 10 },
        }}),
        // Slot 'slotULongLongChanged'
        QtMocHelpers::SlotData<void(QtProperty *, const QString &)>(26, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 14 }, { QMetaType::QString, 10 },
        }}),
        // Slot 'slotUrlChanged'
        QtMocHelpers::SlotData<void(QtProperty *, const QString &)>(27, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 14 }, { QMetaType::QString, 10 },
        }}),
        // Slot 'slotByteArrayChanged'
        QtMocHelpers::SlotData<void(QtProperty *, const QString &)>(28, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 14 }, { QMetaType::QString, 10 },
        }}),
        // Slot 'slotStringListChanged'
        QtMocHelpers::SlotData<void(QtProperty *, const QStringList &)>(29, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 14 }, { QMetaType::QStringList, 10 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DesignerEditorFactory, qt_meta_tag_ZN18qdesigner_internal21DesignerEditorFactoryE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject qdesigner_internal::DesignerEditorFactory::staticMetaObject = { {
    QMetaObject::SuperData::link<QtVariantEditorFactory::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18qdesigner_internal21DesignerEditorFactoryE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18qdesigner_internal21DesignerEditorFactoryE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN18qdesigner_internal21DesignerEditorFactoryE_t>.metaTypes,
    nullptr
} };

void qdesigner_internal::DesignerEditorFactory::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DesignerEditorFactory *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->resetProperty((*reinterpret_cast<std::add_pointer_t<QtProperty*>>(_a[1]))); break;
        case 1: _t->slotEditorDestroyed((*reinterpret_cast<std::add_pointer_t<QtProperty*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[2]))); break;
        case 2: _t->slotAttributeChanged((*reinterpret_cast<std::add_pointer_t<QtProperty*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[3]))); break;
        case 3: _t->slotPropertyChanged((*reinterpret_cast<std::add_pointer_t<QtProperty*>>(_a[1]))); break;
        case 4: _t->slotValueChanged((*reinterpret_cast<std::add_pointer_t<QtProperty*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 5: _t->slotStringTextChanged((*reinterpret_cast<std::add_pointer_t<QtProperty*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 6: _t->slotKeySequenceChanged((*reinterpret_cast<std::add_pointer_t<QtProperty*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QKeySequence>>(_a[2]))); break;
        case 7: _t->slotPaletteChanged((*reinterpret_cast<std::add_pointer_t<QtProperty*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPalette>>(_a[2]))); break;
        case 8: _t->slotPixmapChanged((*reinterpret_cast<std::add_pointer_t<QtProperty*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 9: _t->slotIconChanged((*reinterpret_cast<std::add_pointer_t<QtProperty*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 10: _t->slotIconThemeChanged((*reinterpret_cast<std::add_pointer_t<QtProperty*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 11: _t->slotIconThemeEnumChanged((*reinterpret_cast<std::add_pointer_t<QtProperty*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 12: _t->slotUintChanged((*reinterpret_cast<std::add_pointer_t<QtProperty*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 13: _t->slotIntChanged((*reinterpret_cast<std::add_pointer_t<QtProperty*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 14: _t->slotLongLongChanged((*reinterpret_cast<std::add_pointer_t<QtProperty*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 15: _t->slotULongLongChanged((*reinterpret_cast<std::add_pointer_t<QtProperty*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 16: _t->slotUrlChanged((*reinterpret_cast<std::add_pointer_t<QtProperty*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 17: _t->slotByteArrayChanged((*reinterpret_cast<std::add_pointer_t<QtProperty*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 18: _t->slotStringListChanged((*reinterpret_cast<std::add_pointer_t<QtProperty*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DesignerEditorFactory::*)(QtProperty * )>(_a, &DesignerEditorFactory::resetProperty, 0))
            return;
    }
}

const QMetaObject *qdesigner_internal::DesignerEditorFactory::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *qdesigner_internal::DesignerEditorFactory::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18qdesigner_internal21DesignerEditorFactoryE_t>.strings))
        return static_cast<void*>(this);
    return QtVariantEditorFactory::qt_metacast(_clname);
}

int qdesigner_internal::DesignerEditorFactory::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QtVariantEditorFactory::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 19)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 19;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 19)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 19;
    }
    return _id;
}

// SIGNAL 0
void qdesigner_internal::DesignerEditorFactory::resetProperty(QtProperty * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}
QT_WARNING_POP
