/****************************************************************************
** Meta object code from reading C++ file 'machinetranslator.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../qttools/src/linguist/shared/auto-translation/machinetranslator.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'machinetranslator.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN17MachineTranslatorE_t {};
} // unnamed namespace

template <> constexpr inline auto MachineTranslator::qt_create_metaobjectdata<qt_meta_tag_ZN17MachineTranslatorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "MachineTranslator",
        "batchTranslated",
        "",
        "QHash<const TranslatorMessage*,QStringList>",
        "translations",
        "translationFailed",
        "QList<const TranslatorMessage*>",
        "modelsReceived",
        "models",
        "debugLog",
        "message",
        "fromLlm"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'batchTranslated'
        QtMocHelpers::SignalData<void(QHash<const TranslatorMessage*,QStringList>)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'translationFailed'
        QtMocHelpers::SignalData<void(QList<const TranslatorMessage*>)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 2 },
        }}),
        // Signal 'modelsReceived'
        QtMocHelpers::SignalData<void(QStringList)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 8 },
        }}),
        // Signal 'debugLog'
        QtMocHelpers::SignalData<void(const QByteArray &, bool)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 10 }, { QMetaType::Bool, 11 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<MachineTranslator, qt_meta_tag_ZN17MachineTranslatorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject MachineTranslator::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17MachineTranslatorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17MachineTranslatorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17MachineTranslatorE_t>.metaTypes,
    nullptr
} };

void MachineTranslator::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<MachineTranslator *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->batchTranslated((*reinterpret_cast<std::add_pointer_t<QHash<const TranslatorMessage*,QStringList>>>(_a[1]))); break;
        case 1: _t->translationFailed((*reinterpret_cast<std::add_pointer_t<QList<const TranslatorMessage*>>>(_a[1]))); break;
        case 2: _t->modelsReceived((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 3: _t->debugLog((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (MachineTranslator::*)(QHash<const TranslatorMessage*,QStringList> )>(_a, &MachineTranslator::batchTranslated, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (MachineTranslator::*)(QList<const TranslatorMessage*> )>(_a, &MachineTranslator::translationFailed, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (MachineTranslator::*)(QStringList )>(_a, &MachineTranslator::modelsReceived, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (MachineTranslator::*)(const QByteArray & , bool )>(_a, &MachineTranslator::debugLog, 3))
            return;
    }
}

const QMetaObject *MachineTranslator::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MachineTranslator::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17MachineTranslatorE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int MachineTranslator::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void MachineTranslator::batchTranslated(QHash<const TranslatorMessage*,QStringList> _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void MachineTranslator::translationFailed(QList<const TranslatorMessage*> _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void MachineTranslator::modelsReceived(QStringList _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void MachineTranslator::debugLog(const QByteArray & _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}
QT_WARNING_POP
