/****************************************************************************
** Meta object code from reading C++ file 'tablewidgeteditor.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../../../qttools/src/designer/src/components/taskmenu/tablewidgeteditor.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'tablewidgeteditor.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN18qdesigner_internal17TableWidgetEditorE_t {};
} // unnamed namespace

template <> constexpr inline auto qdesigner_internal::TableWidgetEditor::qt_create_metaobjectdata<qt_meta_tag_ZN18qdesigner_internal17TableWidgetEditorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "qdesigner_internal::TableWidgetEditor",
        "tableWidgetCurrentCellChanged",
        "",
        "currentRow",
        "currentCol",
        "tableWidgetItemChanged",
        "QTableWidgetItem*",
        "item",
        "columnEditorIndexChanged",
        "idx",
        "columnEditorItemChanged",
        "role",
        "QVariant",
        "v",
        "columnEditorItemInserted",
        "columnEditorItemDeleted",
        "columnEditorItemMovedUp",
        "columnEditorItemMovedDown",
        "rowEditorIndexChanged",
        "rowEditorItemChanged",
        "rowEditorItemInserted",
        "rowEditorItemDeleted",
        "rowEditorItemMovedUp",
        "rowEditorItemMovedDown",
        "togglePropertyBrowser",
        "cacheReloaded"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'tableWidgetCurrentCellChanged'
        QtMocHelpers::SlotData<void(int, int)>(1, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 3 }, { QMetaType::Int, 4 },
        }}),
        // Slot 'tableWidgetItemChanged'
        QtMocHelpers::SlotData<void(QTableWidgetItem *)>(5, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Slot 'columnEditorIndexChanged'
        QtMocHelpers::SlotData<void(int)>(8, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 9 },
        }}),
        // Slot 'columnEditorItemChanged'
        QtMocHelpers::SlotData<void(int, int, const QVariant &)>(10, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 9 }, { QMetaType::Int, 11 }, { 0x80000000 | 12, 13 },
        }}),
        // Slot 'columnEditorItemInserted'
        QtMocHelpers::SlotData<void(int)>(14, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 9 },
        }}),
        // Slot 'columnEditorItemDeleted'
        QtMocHelpers::SlotData<void(int)>(15, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 9 },
        }}),
        // Slot 'columnEditorItemMovedUp'
        QtMocHelpers::SlotData<void(int)>(16, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 9 },
        }}),
        // Slot 'columnEditorItemMovedDown'
        QtMocHelpers::SlotData<void(int)>(17, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 9 },
        }}),
        // Slot 'rowEditorIndexChanged'
        QtMocHelpers::SlotData<void(int)>(18, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 9 },
        }}),
        // Slot 'rowEditorItemChanged'
        QtMocHelpers::SlotData<void(int, int, const QVariant &)>(19, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 9 }, { QMetaType::Int, 11 }, { 0x80000000 | 12, 13 },
        }}),
        // Slot 'rowEditorItemInserted'
        QtMocHelpers::SlotData<void(int)>(20, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 9 },
        }}),
        // Slot 'rowEditorItemDeleted'
        QtMocHelpers::SlotData<void(int)>(21, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 9 },
        }}),
        // Slot 'rowEditorItemMovedUp'
        QtMocHelpers::SlotData<void(int)>(22, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 9 },
        }}),
        // Slot 'rowEditorItemMovedDown'
        QtMocHelpers::SlotData<void(int)>(23, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 9 },
        }}),
        // Slot 'togglePropertyBrowser'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'cacheReloaded'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TableWidgetEditor, qt_meta_tag_ZN18qdesigner_internal17TableWidgetEditorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject qdesigner_internal::TableWidgetEditor::staticMetaObject = { {
    QMetaObject::SuperData::link<AbstractItemEditor::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18qdesigner_internal17TableWidgetEditorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18qdesigner_internal17TableWidgetEditorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN18qdesigner_internal17TableWidgetEditorE_t>.metaTypes,
    nullptr
} };

void qdesigner_internal::TableWidgetEditor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TableWidgetEditor *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->tableWidgetCurrentCellChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 1: _t->tableWidgetItemChanged((*reinterpret_cast<std::add_pointer_t<QTableWidgetItem*>>(_a[1]))); break;
        case 2: _t->columnEditorIndexChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 3: _t->columnEditorItemChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[3]))); break;
        case 4: _t->columnEditorItemInserted((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 5: _t->columnEditorItemDeleted((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 6: _t->columnEditorItemMovedUp((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 7: _t->columnEditorItemMovedDown((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 8: _t->rowEditorIndexChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 9: _t->rowEditorItemChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[3]))); break;
        case 10: _t->rowEditorItemInserted((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 11: _t->rowEditorItemDeleted((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 12: _t->rowEditorItemMovedUp((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 13: _t->rowEditorItemMovedDown((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 14: _t->togglePropertyBrowser(); break;
        case 15: _t->cacheReloaded(); break;
        default: ;
        }
    }
}

const QMetaObject *qdesigner_internal::TableWidgetEditor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *qdesigner_internal::TableWidgetEditor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18qdesigner_internal17TableWidgetEditorE_t>.strings))
        return static_cast<void*>(this);
    return AbstractItemEditor::qt_metacast(_clname);
}

int qdesigner_internal::TableWidgetEditor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = AbstractItemEditor::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 16)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 16;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN18qdesigner_internal23TableWidgetEditorDialogE_t {};
} // unnamed namespace

template <> constexpr inline auto qdesigner_internal::TableWidgetEditorDialog::qt_create_metaobjectdata<qt_meta_tag_ZN18qdesigner_internal23TableWidgetEditorDialogE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "qdesigner_internal::TableWidgetEditorDialog"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TableWidgetEditorDialog, qt_meta_tag_ZN18qdesigner_internal23TableWidgetEditorDialogE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject qdesigner_internal::TableWidgetEditorDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18qdesigner_internal23TableWidgetEditorDialogE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18qdesigner_internal23TableWidgetEditorDialogE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN18qdesigner_internal23TableWidgetEditorDialogE_t>.metaTypes,
    nullptr
} };

void qdesigner_internal::TableWidgetEditorDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TableWidgetEditorDialog *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *qdesigner_internal::TableWidgetEditorDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *qdesigner_internal::TableWidgetEditorDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18qdesigner_internal23TableWidgetEditorDialogE_t>.strings))
        return static_cast<void*>(this);
    return QDialog::qt_metacast(_clname);
}

int qdesigner_internal::TableWidgetEditorDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
