/****************************************************************************
** Meta object code from reading C++ file 'distancefieldmodelworker.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qttools/src/distancefieldgenerator/distancefieldmodelworker.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'distancefieldmodelworker.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN24DistanceFieldModelWorkerE_t {};
} // unnamed namespace

template <> constexpr inline auto DistanceFieldModelWorker::qt_create_metaobjectdata<qt_meta_tag_ZN24DistanceFieldModelWorkerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DistanceFieldModelWorker",
        "fontLoaded",
        "",
        "glyphCount",
        "doubleResolution",
        "pixelSize",
        "fontGenerated",
        "distanceFieldGenerated",
        "QImage",
        "distanceField",
        "QPainterPath",
        "path",
        "glyph_t",
        "glyphId",
        "cmapAssignment",
        "error",
        "errorString",
        "generateOneDistanceField",
        "loadFont",
        "fileName"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'fontLoaded'
        QtMocHelpers::SignalData<void(quint16, bool, qreal)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UShort, 3 }, { QMetaType::Bool, 4 }, { QMetaType::QReal, 5 },
        }}),
        // Signal 'fontGenerated'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'distanceFieldGenerated'
        QtMocHelpers::SignalData<void(const QImage &, const QPainterPath &, glyph_t, quint32)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 }, { 0x80000000 | 10, 11 }, { 0x80000000 | 12, 13 }, { QMetaType::UInt, 14 },
        }}),
        // Signal 'error'
        QtMocHelpers::SignalData<void(const QString &)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 16 },
        }}),
        // Method 'generateOneDistanceField'
        QtMocHelpers::MethodData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'loadFont'
        QtMocHelpers::MethodData<void(const QString &)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 19 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DistanceFieldModelWorker, qt_meta_tag_ZN24DistanceFieldModelWorkerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject DistanceFieldModelWorker::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24DistanceFieldModelWorkerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24DistanceFieldModelWorkerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN24DistanceFieldModelWorkerE_t>.metaTypes,
    nullptr
} };

void DistanceFieldModelWorker::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DistanceFieldModelWorker *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->fontLoaded((*reinterpret_cast<std::add_pointer_t<quint16>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<qreal>>(_a[3]))); break;
        case 1: _t->fontGenerated(); break;
        case 2: _t->distanceFieldGenerated((*reinterpret_cast<std::add_pointer_t<QImage>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPainterPath>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<glyph_t>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<quint32>>(_a[4]))); break;
        case 3: _t->error((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 4: _t->generateOneDistanceField(); break;
        case 5: _t->loadFont((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DistanceFieldModelWorker::*)(quint16 , bool , qreal )>(_a, &DistanceFieldModelWorker::fontLoaded, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DistanceFieldModelWorker::*)()>(_a, &DistanceFieldModelWorker::fontGenerated, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (DistanceFieldModelWorker::*)(const QImage & , const QPainterPath & , glyph_t , quint32 )>(_a, &DistanceFieldModelWorker::distanceFieldGenerated, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (DistanceFieldModelWorker::*)(const QString & )>(_a, &DistanceFieldModelWorker::error, 3))
            return;
    }
}

const QMetaObject *DistanceFieldModelWorker::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DistanceFieldModelWorker::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24DistanceFieldModelWorkerE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int DistanceFieldModelWorker::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void DistanceFieldModelWorker::fontLoaded(quint16 _t1, bool _t2, qreal _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2, _t3);
}

// SIGNAL 1
void DistanceFieldModelWorker::fontGenerated()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void DistanceFieldModelWorker::distanceFieldGenerated(const QImage & _t1, const QPainterPath & _t2, glyph_t _t3, quint32 _t4)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2, _t3, _t4);
}

// SIGNAL 3
void DistanceFieldModelWorker::error(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}
QT_WARNING_POP
