/********************************************************************************
** Form generated from reading UI file 'mainwindow.ui'
**
** Created by: Qt User Interface Compiler version 6.12.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_MAINWINDOW_H
#define UI_MAINWINDOW_H

#include <QtCore/QVariant>
#include <QtGui/QAction>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QListView>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QMainWindow>
#include <QtWidgets/QMenu>
#include <QtWidgets/QMenuBar>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QStatusBar>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_MainWindow
{
public:
    QAction *action_Open;
    QAction *action_Save_as;
    QAction *action_Save;
    QAction *actionE_xit;
    QAction *actionSelect_all;
    QAction *actionSelect_string;
    QAction *actionAbout_App;
    QAction *actionAbout_Qt;
    QAction *actionHelp;
    QWidget *centralwidget;
    QGridLayout *gridLayout;
    QListView *lvGlyphs;
    QListWidget *lwUnicodeRanges;
    QGroupBox *groupBox;
    QVBoxLayout *verticalLayout;
    QToolButton *tbSelectAll;
    QToolButton *tbSave;
    QHBoxLayout *horizontalLayout;
    QLabel *label;
    QSpinBox *sbMaximumTextureSize;
    QSpacerItem *verticalSpacer;
    QMenuBar *menubar;
    QMenu *menu_File;
    QMenu *menu_Select;
    QMenu *menu_Help;
    QStatusBar *statusbar;

    void setupUi(QMainWindow *MainWindow)
    {
        if (MainWindow->objectName().isEmpty())
            MainWindow->setObjectName("MainWindow");
        MainWindow->setEnabled(true);
        MainWindow->resize(800, 600);
        action_Open = new QAction(MainWindow);
        action_Open->setObjectName("action_Open");
        action_Save_as = new QAction(MainWindow);
        action_Save_as->setObjectName("action_Save_as");
        action_Save = new QAction(MainWindow);
        action_Save->setObjectName("action_Save");
        actionE_xit = new QAction(MainWindow);
        actionE_xit->setObjectName("actionE_xit");
        actionSelect_all = new QAction(MainWindow);
        actionSelect_all->setObjectName("actionSelect_all");
        actionSelect_string = new QAction(MainWindow);
        actionSelect_string->setObjectName("actionSelect_string");
        actionAbout_App = new QAction(MainWindow);
        actionAbout_App->setObjectName("actionAbout_App");
        actionAbout_Qt = new QAction(MainWindow);
        actionAbout_Qt->setObjectName("actionAbout_Qt");
        actionHelp = new QAction(MainWindow);
        actionHelp->setObjectName("actionHelp");
        centralwidget = new QWidget(MainWindow);
        centralwidget->setObjectName("centralwidget");
        gridLayout = new QGridLayout(centralwidget);
        gridLayout->setObjectName("gridLayout");
        lvGlyphs = new QListView(centralwidget);
        lvGlyphs->setObjectName("lvGlyphs");
        lvGlyphs->setSelectionMode(QAbstractItemView::MultiSelection);
        lvGlyphs->setFlow(QListView::LeftToRight);
        lvGlyphs->setProperty("isWrapping", QVariant(true));
        lvGlyphs->setGridSize(QSize(64, 64));
        lvGlyphs->setViewMode(QListView::ListMode);
        lvGlyphs->setUniformItemSizes(true);

        gridLayout->addWidget(lvGlyphs, 0, 0, 1, 2);

        lwUnicodeRanges = new QListWidget(centralwidget);
        lwUnicodeRanges->setObjectName("lwUnicodeRanges");
        lwUnicodeRanges->setSelectionMode(QAbstractItemView::MultiSelection);

        gridLayout->addWidget(lwUnicodeRanges, 1, 1, 1, 1);

        groupBox = new QGroupBox(centralwidget);
        groupBox->setObjectName("groupBox");
        verticalLayout = new QVBoxLayout(groupBox);
        verticalLayout->setObjectName("verticalLayout");
        tbSelectAll = new QToolButton(groupBox);
        tbSelectAll->setObjectName("tbSelectAll");

        verticalLayout->addWidget(tbSelectAll);

        tbSave = new QToolButton(groupBox);
        tbSave->setObjectName("tbSave");

        verticalLayout->addWidget(tbSave);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        label = new QLabel(groupBox);
        label->setObjectName("label");

        horizontalLayout->addWidget(label);

        sbMaximumTextureSize = new QSpinBox(groupBox);
        sbMaximumTextureSize->setObjectName("sbMaximumTextureSize");
        sbMaximumTextureSize->setMinimum(64);
        sbMaximumTextureSize->setMaximum(2147483647);
        sbMaximumTextureSize->setSingleStep(64);
        sbMaximumTextureSize->setValue(2048);

        horizontalLayout->addWidget(sbMaximumTextureSize);


        verticalLayout->addLayout(horizontalLayout);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer);


        gridLayout->addWidget(groupBox, 1, 0, 1, 1);

        MainWindow->setCentralWidget(centralwidget);
        menubar = new QMenuBar(MainWindow);
        menubar->setObjectName("menubar");
        menubar->setGeometry(QRect(0, 0, 800, 23));
        menu_File = new QMenu(menubar);
        menu_File->setObjectName("menu_File");
        menu_Select = new QMenu(menubar);
        menu_Select->setObjectName("menu_Select");
        menu_Help = new QMenu(menubar);
        menu_Help->setObjectName("menu_Help");
        MainWindow->setMenuBar(menubar);
        statusbar = new QStatusBar(MainWindow);
        statusbar->setObjectName("statusbar");
        MainWindow->setStatusBar(statusbar);

        menubar->addAction(menu_File->menuAction());
        menubar->addAction(menu_Select->menuAction());
        menubar->addAction(menu_Help->menuAction());
        menu_File->addAction(action_Open);
        menu_File->addAction(action_Save);
        menu_File->addAction(action_Save_as);
        menu_File->addSeparator();
        menu_File->addAction(actionE_xit);
        menu_Select->addAction(actionSelect_all);
        menu_Select->addAction(actionSelect_string);
        menu_Help->addAction(actionHelp);
        menu_Help->addAction(actionAbout_App);
        menu_Help->addAction(actionAbout_Qt);

        retranslateUi(MainWindow);

        QMetaObject::connectSlotsByName(MainWindow);
    } // setupUi

    void retranslateUi(QMainWindow *MainWindow)
    {
        MainWindow->setWindowTitle(QCoreApplication::translate("MainWindow", "Qt Distance Field Generator", nullptr));
        action_Open->setText(QCoreApplication::translate("MainWindow", "&Open Font...", nullptr));
#if QT_CONFIG(shortcut)
        action_Open->setShortcut(QCoreApplication::translate("MainWindow", "Ctrl+O", nullptr));
#endif // QT_CONFIG(shortcut)
        action_Save_as->setText(QCoreApplication::translate("MainWindow", "Save &As...", nullptr));
        action_Save->setText(QCoreApplication::translate("MainWindow", "&Save", nullptr));
#if QT_CONFIG(shortcut)
        action_Save->setShortcut(QCoreApplication::translate("MainWindow", "Ctrl+S", nullptr));
#endif // QT_CONFIG(shortcut)
        actionE_xit->setText(QCoreApplication::translate("MainWindow", "E&xit", nullptr));
#if QT_CONFIG(shortcut)
        actionE_xit->setShortcut(QCoreApplication::translate("MainWindow", "Ctrl+Q", nullptr));
#endif // QT_CONFIG(shortcut)
        actionSelect_all->setText(QCoreApplication::translate("MainWindow", "Select &All", nullptr));
#if QT_CONFIG(shortcut)
        actionSelect_all->setShortcut(QCoreApplication::translate("MainWindow", "Ctrl+A", nullptr));
#endif // QT_CONFIG(shortcut)
        actionSelect_string->setText(QCoreApplication::translate("MainWindow", "Select &String", nullptr));
        actionAbout_App->setText(QCoreApplication::translate("MainWindow", "About Qt Distance Field Generator", nullptr));
        actionAbout_Qt->setText(QCoreApplication::translate("MainWindow", "About Qt", nullptr));
        actionHelp->setText(QCoreApplication::translate("MainWindow", "Help", nullptr));
        groupBox->setTitle(QString());
        tbSelectAll->setText(QCoreApplication::translate("MainWindow", "Select &all", nullptr));
        tbSave->setText(QCoreApplication::translate("MainWindow", "&Save", nullptr));
        label->setText(QCoreApplication::translate("MainWindow", "Maximum texture size:", nullptr));
        menu_File->setTitle(QCoreApplication::translate("MainWindow", "&File", nullptr));
        menu_Select->setTitle(QCoreApplication::translate("MainWindow", "&Select", nullptr));
        menu_Help->setTitle(QCoreApplication::translate("MainWindow", "&Help", nullptr));
    } // retranslateUi

};

namespace Ui {
    class MainWindow: public Ui_MainWindow {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_MAINWINDOW_H
