/****************************************************************************
** Resource object code
**
** Created by: The Resource Compiler for Qt version 6.12.0
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#ifdef _MSC_VER
// disable informational message "function ... selected for automatic inline expansion"
#pragma warning (disable: 4711)
#endif

static const unsigned char qt_resource_data[] = {
  // exclusive_builds.prf
  0x0,0x0,0x0,0xf0,
  0x23,
  0x20,0x4c,0x75,0x70,0x64,0x61,0x74,0x65,0x20,0x64,0x6f,0x65,0x73,0x6e,0x27,0x74,
  0x20,0x65,0x78,0x65,0x63,0x75,0x74,0x65,0x20,0x74,0x68,0x65,0x20,0x62,0x75,0x69,
  0x6c,0x64,0x20,0x70,0x61,0x73,0x73,0x65,0x73,0x2c,0x20,0x73,0x6f,0x20,0x70,0x72,
  0x65,0x76,0x65,0x6e,0x74,0x20,0x74,0x68,0x61,0x74,0x20,0x74,0x68,0x65,0x79,0x20,
  0x61,0x72,0x65,0xa,0x23,0x20,0x63,0x72,0x65,0x61,0x74,0x65,0x64,0x20,0x69,0x6e,
  0x20,0x74,0x68,0x65,0x20,0x66,0x69,0x72,0x73,0x74,0x20,0x70,0x6c,0x61,0x63,0x65,
  0x2e,0x20,0x54,0x68,0x69,0x73,0x20,0x74,0x72,0x69,0x63,0x6b,0x73,0x20,0x66,0x6f,
  0x72,0x20,0x65,0x78,0x61,0x6d,0x70,0x6c,0x65,0x20,0x72,0x65,0x73,0x6f,0x75,0x72,
  0x63,0x65,0x73,0x2e,0x70,0x72,0x66,0xa,0x23,0x20,0x69,0x6e,0x74,0x6f,0x20,0x61,
  0x63,0x74,0x75,0x61,0x6c,0x6c,0x79,0x20,0x63,0x72,0x65,0x61,0x74,0x69,0x6e,0x67,
  0x20,0x74,0x68,0x65,0x20,0x51,0x4d,0x4c,0x20,0x72,0x65,0x73,0x6f,0x75,0x72,0x63,
  0x65,0x20,0x66,0x69,0x6c,0x65,0x2c,0x20,0x73,0x6f,0x20,0x77,0x65,0x20,0x63,0x61,
  0x6e,0x20,0x73,0x63,0x61,0x6e,0x20,0x69,0x74,0x2e,0xa,0xa,0x64,0x65,0x66,0x69,
  0x6e,0x65,0x54,0x65,0x73,0x74,0x28,0x61,0x64,0x64,0x45,0x78,0x63,0x6c,0x75,0x73,
  0x69,0x76,0x65,0x42,0x75,0x69,0x6c,0x64,0x73,0x29,0x20,0x7b,0xa,0x7d,0xa,
  
};

static const unsigned char qt_resource_name[] = {
  // qmake
  0x0,0x5,
  0x0,0x78,0x38,0x15,
  0x0,0x71,
  0x0,0x6d,0x0,0x61,0x0,0x6b,0x0,0x65,
    // override_features
  0x0,0x11,
  0xb,0xa5,0x91,0x23,
  0x0,0x6f,
  0x0,0x76,0x0,0x65,0x0,0x72,0x0,0x72,0x0,0x69,0x0,0x64,0x0,0x65,0x0,0x5f,0x0,0x66,0x0,0x65,0x0,0x61,0x0,0x74,0x0,0x75,0x0,0x72,0x0,0x65,0x0,0x73,
  
    // exclusive_builds.prf
  0x0,0x14,
  0x1,0x52,0x7e,0x46,
  0x0,0x65,
  0x0,0x78,0x0,0x63,0x0,0x6c,0x0,0x75,0x0,0x73,0x0,0x69,0x0,0x76,0x0,0x65,0x0,0x5f,0x0,0x62,0x0,0x75,0x0,0x69,0x0,0x6c,0x0,0x64,0x0,0x73,0x0,0x2e,
  0x0,0x70,0x0,0x72,0x0,0x66,
  
};

static const unsigned char qt_resource_struct[] = {
  // :
  0x0,0x0,0x0,0x0,0x0,0x2,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x1,
0x0,0x0,0x1,0x9e,0xc5,0x60,0x7a,0xd8,
  // :/qmake
  0x0,0x0,0x0,0x0,0x0,0x2,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x2,
0x0,0x0,0x1,0x9e,0xc5,0x60,0x7a,0xd8,
  // :/qmake/override_features
  0x0,0x0,0x0,0x10,0x0,0x2,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x3,
0x0,0x0,0x1,0x9e,0xc5,0x60,0x7a,0xd8,
  // :/qmake/override_features/exclusive_builds.prf
  0x0,0x0,0x0,0x38,0x0,0x0,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x0,
0x0,0x0,0x1,0x9e,0xc5,0x60,0x7a,0xd8,

};

#ifdef QT_NAMESPACE
#  define QT_RCC_PREPEND_NAMESPACE(name) ::QT_NAMESPACE::name
#  define QT_RCC_MANGLE_NAMESPACE0(x) x
#  define QT_RCC_MANGLE_NAMESPACE1(a, b) a##_##b
#  define QT_RCC_MANGLE_NAMESPACE2(a, b) QT_RCC_MANGLE_NAMESPACE1(a,b)
#  define QT_RCC_MANGLE_NAMESPACE(name) QT_RCC_MANGLE_NAMESPACE2( \
        QT_RCC_MANGLE_NAMESPACE0(name), QT_RCC_MANGLE_NAMESPACE0(QT_NAMESPACE))
#else
#   define QT_RCC_PREPEND_NAMESPACE(name) name
#   define QT_RCC_MANGLE_NAMESPACE(name) name
#endif

#if defined(QT_INLINE_NAMESPACE)
inline namespace QT_NAMESPACE {
#elif defined(QT_NAMESPACE)
namespace QT_NAMESPACE {
#endif

bool qRegisterResourceData(int, const unsigned char *, const unsigned char *, const unsigned char *);
bool qUnregisterResourceData(int, const unsigned char *, const unsigned char *, const unsigned char *);

#ifdef QT_NAMESPACE
}
#endif

int QT_RCC_MANGLE_NAMESPACE(qInitResources_proparser)();
int QT_RCC_MANGLE_NAMESPACE(qInitResources_proparser)()
{
    int version = 3;
    QT_RCC_PREPEND_NAMESPACE(qRegisterResourceData)
        (version, qt_resource_struct, qt_resource_name, qt_resource_data);
    return 1;
}

int QT_RCC_MANGLE_NAMESPACE(qCleanupResources_proparser)();
int QT_RCC_MANGLE_NAMESPACE(qCleanupResources_proparser)()
{
    int version = 3;
    QT_RCC_PREPEND_NAMESPACE(qUnregisterResourceData)
       (version, qt_resource_struct, qt_resource_name, qt_resource_data);
    return 1;
}

#ifdef __clang__
#   pragma clang diagnostic push
#   pragma clang diagnostic ignored "-Wexit-time-destructors"
#endif

namespace {
   struct initializer {
       initializer() { QT_RCC_MANGLE_NAMESPACE(qInitResources_proparser)(); }
       ~initializer() { QT_RCC_MANGLE_NAMESPACE(qCleanupResources_proparser)(); }
   } dummy;
}

#ifdef __clang__
#   pragma clang diagnostic pop
#endif
