/****************************************************************************
** Meta object code from reading C++ file 'signalsloteditorwindow.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../../../qttools/src/designer/src/components/signalsloteditor/signalsloteditorwindow.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'signalsloteditorwindow.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN18qdesigner_internal22SignalSlotEditorWindowE_t {};
} // unnamed namespace

template <> constexpr inline auto qdesigner_internal::SignalSlotEditorWindow::qt_create_metaobjectdata<qt_meta_tag_ZN18qdesigner_internal22SignalSlotEditorWindowE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "qdesigner_internal::SignalSlotEditorWindow",
        "setActiveFormWindow",
        "",
        "QDesignerFormWindowInterface*",
        "form",
        "updateDialogSelection",
        "qdesigner_internal::Connection*",
        "con",
        "updateEditorSelection",
        "QModelIndex",
        "index",
        "objectNameChanged",
        "formWindow",
        "object",
        "newName",
        "oldName",
        "addConnection",
        "removeConnection",
        "updateUi",
        "resizeColumns"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'setActiveFormWindow'
        QtMocHelpers::SlotData<void(QDesignerFormWindowInterface *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'updateDialogSelection'
        QtMocHelpers::SlotData<void(qdesigner_internal::Connection *)>(5, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Slot 'updateEditorSelection'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(8, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Slot 'objectNameChanged'
        QtMocHelpers::SlotData<void(QDesignerFormWindowInterface *, QObject *, const QString &, const QString &)>(11, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 12 }, { QMetaType::QObjectStar, 13 }, { QMetaType::QString, 14 }, { QMetaType::QString, 15 },
        }}),
        // Slot 'addConnection'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'removeConnection'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateUi'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'resizeColumns'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<SignalSlotEditorWindow, qt_meta_tag_ZN18qdesigner_internal22SignalSlotEditorWindowE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject qdesigner_internal::SignalSlotEditorWindow::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18qdesigner_internal22SignalSlotEditorWindowE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18qdesigner_internal22SignalSlotEditorWindowE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN18qdesigner_internal22SignalSlotEditorWindowE_t>.metaTypes,
    nullptr
} };

void qdesigner_internal::SignalSlotEditorWindow::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<SignalSlotEditorWindow *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->setActiveFormWindow((*reinterpret_cast<std::add_pointer_t<QDesignerFormWindowInterface*>>(_a[1]))); break;
        case 1: _t->updateDialogSelection((*reinterpret_cast<std::add_pointer_t<qdesigner_internal::Connection*>>(_a[1]))); break;
        case 2: _t->updateEditorSelection((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 3: _t->objectNameChanged((*reinterpret_cast<std::add_pointer_t<QDesignerFormWindowInterface*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4]))); break;
        case 4: _t->addConnection(); break;
        case 5: _t->removeConnection(); break;
        case 6: _t->updateUi(); break;
        case 7: _t->resizeColumns(); break;
        default: ;
        }
    }
}

const QMetaObject *qdesigner_internal::SignalSlotEditorWindow::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *qdesigner_internal::SignalSlotEditorWindow::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18qdesigner_internal22SignalSlotEditorWindowE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int qdesigner_internal::SignalSlotEditorWindow::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 8;
    }
    return _id;
}
QT_WARNING_POP
