/****************************************************************************
** Meta object code from reading C++ file 'formwindowmanager.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../../../qttools/src/designer/src/components/formeditor/formwindowmanager.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'formwindowmanager.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN18qdesigner_internal17FormWindowManagerE_t {};
} // unnamed namespace

template <> constexpr inline auto qdesigner_internal::FormWindowManager::qt_create_metaobjectdata<qt_meta_tag_ZN18qdesigner_internal17FormWindowManagerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "qdesigner_internal::FormWindowManager",
        "addFormWindow",
        "",
        "QDesignerFormWindowInterface*",
        "formWindow",
        "removeFormWindow",
        "setActiveFormWindow",
        "closeAllPreviews",
        "deviceProfilesChanged",
        "slotActionCutActivated",
        "slotActionCopyActivated",
        "slotActionPasteActivated",
        "slotActionDeleteActivated",
        "slotActionSelectAllActivated",
        "slotActionLowerActivated",
        "slotActionRaiseActivated",
        "createLayout",
        "slotActionBreakLayoutActivated",
        "slotActionAdjustSizeActivated",
        "slotActionSimplifyLayoutActivated",
        "showPreview",
        "slotActionGroupPreviewInStyle",
        "style",
        "deviceProfileIndex",
        "slotActionShowFormWindowSettingsDialog",
        "slotUpdateActions"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'addFormWindow'
        QtMocHelpers::SlotData<void(QDesignerFormWindowInterface *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'removeFormWindow'
        QtMocHelpers::SlotData<void(QDesignerFormWindowInterface *)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'setActiveFormWindow'
        QtMocHelpers::SlotData<void(QDesignerFormWindowInterface *)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'closeAllPreviews'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'deviceProfilesChanged'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotActionCutActivated'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotActionCopyActivated'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotActionPasteActivated'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotActionDeleteActivated'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotActionSelectAllActivated'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotActionLowerActivated'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotActionRaiseActivated'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'createLayout'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotActionBreakLayoutActivated'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotActionAdjustSizeActivated'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotActionSimplifyLayoutActivated'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'showPreview'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotActionGroupPreviewInStyle'
        QtMocHelpers::SlotData<void(const QString &, int)>(21, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 22 }, { QMetaType::Int, 23 },
        }}),
        // Slot 'slotActionShowFormWindowSettingsDialog'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotUpdateActions'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<FormWindowManager, qt_meta_tag_ZN18qdesigner_internal17FormWindowManagerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject qdesigner_internal::FormWindowManager::staticMetaObject = { {
    QMetaObject::SuperData::link<QDesignerFormWindowManager::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18qdesigner_internal17FormWindowManagerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18qdesigner_internal17FormWindowManagerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN18qdesigner_internal17FormWindowManagerE_t>.metaTypes,
    nullptr
} };

void qdesigner_internal::FormWindowManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<FormWindowManager *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->addFormWindow((*reinterpret_cast<std::add_pointer_t<QDesignerFormWindowInterface*>>(_a[1]))); break;
        case 1: _t->removeFormWindow((*reinterpret_cast<std::add_pointer_t<QDesignerFormWindowInterface*>>(_a[1]))); break;
        case 2: _t->setActiveFormWindow((*reinterpret_cast<std::add_pointer_t<QDesignerFormWindowInterface*>>(_a[1]))); break;
        case 3: _t->closeAllPreviews(); break;
        case 4: _t->deviceProfilesChanged(); break;
        case 5: _t->slotActionCutActivated(); break;
        case 6: _t->slotActionCopyActivated(); break;
        case 7: _t->slotActionPasteActivated(); break;
        case 8: _t->slotActionDeleteActivated(); break;
        case 9: _t->slotActionSelectAllActivated(); break;
        case 10: _t->slotActionLowerActivated(); break;
        case 11: _t->slotActionRaiseActivated(); break;
        case 12: _t->createLayout(); break;
        case 13: _t->slotActionBreakLayoutActivated(); break;
        case 14: _t->slotActionAdjustSizeActivated(); break;
        case 15: _t->slotActionSimplifyLayoutActivated(); break;
        case 16: _t->showPreview(); break;
        case 17: _t->slotActionGroupPreviewInStyle((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 18: _t->slotActionShowFormWindowSettingsDialog(); break;
        case 19: _t->slotUpdateActions(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QDesignerFormWindowInterface* >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QDesignerFormWindowInterface* >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QDesignerFormWindowInterface* >(); break;
            }
            break;
        }
    }
}

const QMetaObject *qdesigner_internal::FormWindowManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *qdesigner_internal::FormWindowManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18qdesigner_internal17FormWindowManagerE_t>.strings))
        return static_cast<void*>(this);
    return QDesignerFormWindowManager::qt_metacast(_clname);
}

int qdesigner_internal::FormWindowManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDesignerFormWindowManager::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 20)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 20;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 20)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 20;
    }
    return _id;
}
QT_WARNING_POP
