/********************************************************************************
** Form generated from reading UI file 'translationsettings.ui'
**
** Created by: Qt User Interface Compiler version 6.12.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_TRANSLATIONSETTINGS_H
#define UI_TRANSLATIONSETTINGS_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_TranslationSettingsDialog
{
public:
    QVBoxLayout *verticalLayout;
    QGroupBox *srcGroupBox;
    QGridLayout *_2;
    QComboBox *srcCbLanguageList;
    QLabel *srcLblLanguage;
    QComboBox *srcCbCountryList;
    QLabel *srcLblCountry;
    QGroupBox *tgtGroupBox;
    QGridLayout *gridLayout;
    QComboBox *tgtCbLanguageList;
    QLabel *tgtLblLanguage;
    QComboBox *tgtCbCountryList;
    QLabel *tgtLblCountry;
    QSpacerItem *verticalSpacer;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *TranslationSettingsDialog)
    {
        if (TranslationSettingsDialog->objectName().isEmpty())
            TranslationSettingsDialog->setObjectName("TranslationSettingsDialog");
        TranslationSettingsDialog->resize(416, 263);
        verticalLayout = new QVBoxLayout(TranslationSettingsDialog);
        verticalLayout->setObjectName("verticalLayout");
        srcGroupBox = new QGroupBox(TranslationSettingsDialog);
        srcGroupBox->setObjectName("srcGroupBox");
        _2 = new QGridLayout(srcGroupBox);
#ifndef Q_OS_MACOS
        _2->setSpacing(6);
#endif
#ifndef Q_OS_MACOS
        _2->setContentsMargins(9, 9, 9, 9);
#endif
        _2->setObjectName("_2");
        srcCbLanguageList = new QComboBox(srcGroupBox);
        srcCbLanguageList->setObjectName("srcCbLanguageList");

        _2->addWidget(srcCbLanguageList, 0, 1, 1, 1);

        srcLblLanguage = new QLabel(srcGroupBox);
        srcLblLanguage->setObjectName("srcLblLanguage");

        _2->addWidget(srcLblLanguage, 0, 0, 1, 1);

        srcCbCountryList = new QComboBox(srcGroupBox);
        srcCbCountryList->setObjectName("srcCbCountryList");

        _2->addWidget(srcCbCountryList, 1, 1, 1, 1);

        srcLblCountry = new QLabel(srcGroupBox);
        srcLblCountry->setObjectName("srcLblCountry");

        _2->addWidget(srcLblCountry, 1, 0, 1, 1);


        verticalLayout->addWidget(srcGroupBox);

        tgtGroupBox = new QGroupBox(TranslationSettingsDialog);
        tgtGroupBox->setObjectName("tgtGroupBox");
        gridLayout = new QGridLayout(tgtGroupBox);
#ifndef Q_OS_MACOS
        gridLayout->setSpacing(6);
#endif
#ifndef Q_OS_MACOS
        gridLayout->setContentsMargins(9, 9, 9, 9);
#endif
        gridLayout->setObjectName("gridLayout");
        tgtCbLanguageList = new QComboBox(tgtGroupBox);
        tgtCbLanguageList->setObjectName("tgtCbLanguageList");

        gridLayout->addWidget(tgtCbLanguageList, 0, 1, 1, 1);

        tgtLblLanguage = new QLabel(tgtGroupBox);
        tgtLblLanguage->setObjectName("tgtLblLanguage");

        gridLayout->addWidget(tgtLblLanguage, 0, 0, 1, 1);

        tgtCbCountryList = new QComboBox(tgtGroupBox);
        tgtCbCountryList->setObjectName("tgtCbCountryList");

        gridLayout->addWidget(tgtCbCountryList, 1, 1, 1, 1);

        tgtLblCountry = new QLabel(tgtGroupBox);
        tgtLblCountry->setObjectName("tgtLblCountry");

        gridLayout->addWidget(tgtLblCountry, 1, 0, 1, 1);


        verticalLayout->addWidget(tgtGroupBox);

        verticalSpacer = new QSpacerItem(20, 100, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer);

        buttonBox = new QDialogButtonBox(TranslationSettingsDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);

#if QT_CONFIG(shortcut)
        srcLblLanguage->setBuddy(tgtCbLanguageList);
        tgtLblLanguage->setBuddy(tgtCbLanguageList);
#endif // QT_CONFIG(shortcut)

        retranslateUi(TranslationSettingsDialog);
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, TranslationSettingsDialog, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(TranslationSettingsDialog);
    } // setupUi

    void retranslateUi(QDialog *TranslationSettingsDialog)
    {
        TranslationSettingsDialog->setWindowTitle(QString());
        srcGroupBox->setTitle(QCoreApplication::translate("TranslationSettingsDialog", "Source language", nullptr));
        srcLblLanguage->setText(QCoreApplication::translate("TranslationSettingsDialog", "Language", nullptr));
        srcLblCountry->setText(QCoreApplication::translate("TranslationSettingsDialog", "Country/Region", nullptr));
        tgtGroupBox->setTitle(QCoreApplication::translate("TranslationSettingsDialog", "Target language", nullptr));
        tgtLblLanguage->setText(QCoreApplication::translate("TranslationSettingsDialog", "Language", nullptr));
        tgtLblCountry->setText(QCoreApplication::translate("TranslationSettingsDialog", "Country/Region", nullptr));
    } // retranslateUi

};

namespace Ui {
    class TranslationSettingsDialog: public Ui_TranslationSettingsDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_TRANSLATIONSETTINGS_H
