/****************************************************************************
** Meta object code from reading C++ file 'connectionedit_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../../qttools/src/designer/src/lib/shared/connectionedit_p.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'connectionedit_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN18qdesigner_internal14ConnectionEditE_t {};
} // unnamed namespace

template <> constexpr inline auto qdesigner_internal::ConnectionEdit::qt_create_metaobjectdata<qt_meta_tag_ZN18qdesigner_internal14ConnectionEditE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "qdesigner_internal::ConnectionEdit",
        "aboutToAddConnection",
        "",
        "idx",
        "connectionAdded",
        "qdesigner_internal::Connection*",
        "con",
        "aboutToRemoveConnection",
        "connectionRemoved",
        "connectionSelected",
        "widgetActivated",
        "QWidget*",
        "wgt",
        "connectionChanged",
        "selectNone",
        "selectAll",
        "deleteSelected",
        "setBackground",
        "background",
        "updateBackground",
        "widgetRemoved",
        "w",
        "objectRemoved",
        "o",
        "updateLines",
        "enableUpdateBackground",
        "enable"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'aboutToAddConnection'
        QtMocHelpers::SignalData<void(int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Signal 'connectionAdded'
        QtMocHelpers::SignalData<void(qdesigner_internal::Connection *)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Signal 'aboutToRemoveConnection'
        QtMocHelpers::SignalData<void(qdesigner_internal::Connection *)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Signal 'connectionRemoved'
        QtMocHelpers::SignalData<void(int)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Signal 'connectionSelected'
        QtMocHelpers::SignalData<void(qdesigner_internal::Connection *)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Signal 'widgetActivated'
        QtMocHelpers::SignalData<void(QWidget *)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Signal 'connectionChanged'
        QtMocHelpers::SignalData<void(qdesigner_internal::Connection *)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Slot 'selectNone'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'selectAll'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'deleteSelected'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setBackground'
        QtMocHelpers::SlotData<void(QWidget *)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 18 },
        }}),
        // Slot 'updateBackground'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'widgetRemoved'
        QtMocHelpers::SlotData<void(QWidget *)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 21 },
        }}),
        // Slot 'objectRemoved'
        QtMocHelpers::SlotData<void(QObject *)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QObjectStar, 23 },
        }}),
        // Slot 'updateLines'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'enableUpdateBackground'
        QtMocHelpers::SlotData<void(bool)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 26 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ConnectionEdit, qt_meta_tag_ZN18qdesigner_internal14ConnectionEditE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject qdesigner_internal::ConnectionEdit::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18qdesigner_internal14ConnectionEditE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18qdesigner_internal14ConnectionEditE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN18qdesigner_internal14ConnectionEditE_t>.metaTypes,
    nullptr
} };

void qdesigner_internal::ConnectionEdit::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ConnectionEdit *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->aboutToAddConnection((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 1: _t->connectionAdded((*reinterpret_cast<std::add_pointer_t<qdesigner_internal::Connection*>>(_a[1]))); break;
        case 2: _t->aboutToRemoveConnection((*reinterpret_cast<std::add_pointer_t<qdesigner_internal::Connection*>>(_a[1]))); break;
        case 3: _t->connectionRemoved((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 4: _t->connectionSelected((*reinterpret_cast<std::add_pointer_t<qdesigner_internal::Connection*>>(_a[1]))); break;
        case 5: _t->widgetActivated((*reinterpret_cast<std::add_pointer_t<QWidget*>>(_a[1]))); break;
        case 6: _t->connectionChanged((*reinterpret_cast<std::add_pointer_t<qdesigner_internal::Connection*>>(_a[1]))); break;
        case 7: _t->selectNone(); break;
        case 8: _t->selectAll(); break;
        case 9: _t->deleteSelected(); break;
        case 10: _t->setBackground((*reinterpret_cast<std::add_pointer_t<QWidget*>>(_a[1]))); break;
        case 11: _t->updateBackground(); break;
        case 12: _t->widgetRemoved((*reinterpret_cast<std::add_pointer_t<QWidget*>>(_a[1]))); break;
        case 13: _t->objectRemoved((*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[1]))); break;
        case 14: _t->updateLines(); break;
        case 15: _t->enableUpdateBackground((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QWidget* >(); break;
            }
            break;
        case 10:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QWidget* >(); break;
            }
            break;
        case 12:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QWidget* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ConnectionEdit::*)(int )>(_a, &ConnectionEdit::aboutToAddConnection, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ConnectionEdit::*)(qdesigner_internal::Connection * )>(_a, &ConnectionEdit::connectionAdded, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ConnectionEdit::*)(qdesigner_internal::Connection * )>(_a, &ConnectionEdit::aboutToRemoveConnection, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (ConnectionEdit::*)(int )>(_a, &ConnectionEdit::connectionRemoved, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (ConnectionEdit::*)(qdesigner_internal::Connection * )>(_a, &ConnectionEdit::connectionSelected, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (ConnectionEdit::*)(QWidget * )>(_a, &ConnectionEdit::widgetActivated, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (ConnectionEdit::*)(qdesigner_internal::Connection * )>(_a, &ConnectionEdit::connectionChanged, 6))
            return;
    }
}

const QMetaObject *qdesigner_internal::ConnectionEdit::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *qdesigner_internal::ConnectionEdit::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18qdesigner_internal14ConnectionEditE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "CETypes"))
        return static_cast< CETypes*>(this);
    return QWidget::qt_metacast(_clname);
}

int qdesigner_internal::ConnectionEdit::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    }
    return _id;
}

// SIGNAL 0
void qdesigner_internal::ConnectionEdit::aboutToAddConnection(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void qdesigner_internal::ConnectionEdit::connectionAdded(qdesigner_internal::Connection * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void qdesigner_internal::ConnectionEdit::aboutToRemoveConnection(qdesigner_internal::Connection * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void qdesigner_internal::ConnectionEdit::connectionRemoved(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void qdesigner_internal::ConnectionEdit::connectionSelected(qdesigner_internal::Connection * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void qdesigner_internal::ConnectionEdit::widgetActivated(QWidget * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void qdesigner_internal::ConnectionEdit::connectionChanged(qdesigner_internal::Connection * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}
QT_WARNING_POP
