/****************************************************************************
** Meta object code from reading C++ file 'phraseview.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../qttools/src/linguist/linguist/phraseview.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'phraseview.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10PhraseViewE_t {};
} // unnamed namespace

template <> constexpr inline auto PhraseView::qt_create_metaobjectdata<qt_meta_tag_ZN10PhraseViewE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "PhraseView",
        "phraseSelected",
        "",
        "latestModel",
        "phrase",
        "showFewerGuessesAvailable",
        "canShow",
        "setCurrentMessageFromGuess",
        "modelIndex",
        "Candidate",
        "cand",
        "toggleGuessing",
        "update",
        "getMaxCandidates",
        "setMaxCandidates",
        "max",
        "getDefaultMaxCandidates",
        "moreGuesses",
        "fewerGuesses",
        "resetNumGuesses",
        "guessShortcut",
        "nkey",
        "selectPhrase",
        "QModelIndex",
        "index",
        "selectCurrentPhrase",
        "editPhrase",
        "gotoMessageFromGuess"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'phraseSelected'
        QtMocHelpers::SignalData<void(int, const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 }, { QMetaType::QString, 4 },
        }}),
        // Signal 'showFewerGuessesAvailable'
        QtMocHelpers::SignalData<void(bool)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 6 },
        }}),
        // Signal 'setCurrentMessageFromGuess'
        QtMocHelpers::SignalData<void(int, const Candidate &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 8 }, { 0x80000000 | 9, 10 },
        }}),
        // Slot 'toggleGuessing'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'update'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'getMaxCandidates'
        QtMocHelpers::SlotData<int() const>(13, 2, QMC::AccessPublic, QMetaType::Int),
        // Slot 'setMaxCandidates'
        QtMocHelpers::SlotData<void(const int)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 15 },
        }}),
        // Slot 'getDefaultMaxCandidates'
        QtMocHelpers::SlotData<int()>(16, 2, QMC::AccessPublic, QMetaType::Int),
        // Slot 'moreGuesses'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'fewerGuesses'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'resetNumGuesses'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'guessShortcut'
        QtMocHelpers::SlotData<void(int)>(20, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 21 },
        }}),
        // Slot 'selectPhrase'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(22, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 23, 24 },
        }}),
        // Slot 'selectCurrentPhrase'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'editPhrase'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'gotoMessageFromGuess'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<PhraseView, qt_meta_tag_ZN10PhraseViewE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject PhraseView::staticMetaObject = { {
    QMetaObject::SuperData::link<QTreeView::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10PhraseViewE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10PhraseViewE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10PhraseViewE_t>.metaTypes,
    nullptr
} };

void PhraseView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PhraseView *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->phraseSelected((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 1: _t->showFewerGuessesAvailable((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 2: _t->setCurrentMessageFromGuess((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Candidate>>(_a[2]))); break;
        case 3: _t->toggleGuessing(); break;
        case 4: _t->update(); break;
        case 5: { int _r = _t->getMaxCandidates();
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 6: _t->setMaxCandidates((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 7: { int _r = _t->getDefaultMaxCandidates();
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 8: _t->moreGuesses(); break;
        case 9: _t->fewerGuesses(); break;
        case 10: _t->resetNumGuesses(); break;
        case 11: _t->guessShortcut((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 12: _t->selectPhrase((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 13: _t->selectCurrentPhrase(); break;
        case 14: _t->editPhrase(); break;
        case 15: _t->gotoMessageFromGuess(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (PhraseView::*)(int , const QString & )>(_a, &PhraseView::phraseSelected, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (PhraseView::*)(bool )>(_a, &PhraseView::showFewerGuessesAvailable, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (PhraseView::*)(int , const Candidate & )>(_a, &PhraseView::setCurrentMessageFromGuess, 2))
            return;
    }
}

const QMetaObject *PhraseView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PhraseView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10PhraseViewE_t>.strings))
        return static_cast<void*>(this);
    return QTreeView::qt_metacast(_clname);
}

int PhraseView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QTreeView::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 16)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 16;
    }
    return _id;
}

// SIGNAL 0
void PhraseView::phraseSelected(int _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void PhraseView::showFewerGuessesAvailable(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void PhraseView::setCurrentMessageFromGuess(int _t1, const Candidate & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}
QT_WARNING_POP
