/****************************************************************************
** Meta object code from reading C++ file 'machinetranslationdialog.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../qttools/src/linguist/linguist/machinetranslationdialog.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'machinetranslationdialog.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN24MachineTranslationDialogE_t {};
} // unnamed namespace

template <> constexpr inline auto MachineTranslationDialog::qt_create_metaobjectdata<qt_meta_tag_ZN24MachineTranslationDialogE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "MachineTranslationDialog",
        "stop",
        "",
        "translateSelection",
        "onBatchTranslated",
        "QHash<const TranslatorMessage*,QStringList>",
        "translations",
        "onNewDebugMessage",
        "message",
        "fromLlm",
        "onFilterChanged",
        "id",
        "applyTranslations",
        "onTranslationFailed",
        "QList<const TranslatorMessage*>",
        "connectToServer",
        "onApiTypeChanged",
        "index",
        "onServerUrlChanged",
        "onModelsReceived",
        "models",
        "updateToolBoxTexts",
        "toggleAdvancedSettings",
        "checked",
        "loadAdvancedSettings",
        "saveAdvancedSettings",
        "resetAdvancedSettings",
        "validateAdvancedSettings"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'stop'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'translateSelection'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onBatchTranslated'
        QtMocHelpers::SlotData<void(QHash<const TranslatorMessage*,QStringList>)>(4, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Slot 'onNewDebugMessage'
        QtMocHelpers::SlotData<void(const QByteArray &, bool)>(7, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QByteArray, 8 }, { QMetaType::Bool, 9 },
        }}),
        // Slot 'onFilterChanged'
        QtMocHelpers::SlotData<void(int)>(10, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 11 },
        }}),
        // Slot 'applyTranslations'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onTranslationFailed'
        QtMocHelpers::SlotData<void(QList<const TranslatorMessage*>)>(13, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 14, 2 },
        }}),
        // Slot 'connectToServer'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onApiTypeChanged'
        QtMocHelpers::SlotData<void(int)>(16, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 17 },
        }}),
        // Slot 'onServerUrlChanged'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onModelsReceived'
        QtMocHelpers::SlotData<void(const QStringList &)>(19, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QStringList, 20 },
        }}),
        // Slot 'updateToolBoxTexts'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'toggleAdvancedSettings'
        QtMocHelpers::SlotData<void(bool)>(22, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 23 },
        }}),
        // Slot 'loadAdvancedSettings'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'saveAdvancedSettings'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'resetAdvancedSettings'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'validateAdvancedSettings'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<MachineTranslationDialog, qt_meta_tag_ZN24MachineTranslationDialogE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject MachineTranslationDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24MachineTranslationDialogE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24MachineTranslationDialogE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN24MachineTranslationDialogE_t>.metaTypes,
    nullptr
} };

void MachineTranslationDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<MachineTranslationDialog *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->stop(); break;
        case 1: _t->translateSelection(); break;
        case 2: _t->onBatchTranslated((*reinterpret_cast<std::add_pointer_t<QHash<const TranslatorMessage*,QStringList>>>(_a[1]))); break;
        case 3: _t->onNewDebugMessage((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 4: _t->onFilterChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 5: _t->applyTranslations(); break;
        case 6: _t->onTranslationFailed((*reinterpret_cast<std::add_pointer_t<QList<const TranslatorMessage*>>>(_a[1]))); break;
        case 7: _t->connectToServer(); break;
        case 8: _t->onApiTypeChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 9: _t->onServerUrlChanged(); break;
        case 10: _t->onModelsReceived((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 11: _t->updateToolBoxTexts(); break;
        case 12: _t->toggleAdvancedSettings((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 13: _t->loadAdvancedSettings(); break;
        case 14: _t->saveAdvancedSettings(); break;
        case 15: _t->resetAdvancedSettings(); break;
        case 16: _t->validateAdvancedSettings(); break;
        default: ;
        }
    }
}

const QMetaObject *MachineTranslationDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MachineTranslationDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24MachineTranslationDialogE_t>.strings))
        return static_cast<void*>(this);
    return QDialog::qt_metacast(_clname);
}

int MachineTranslationDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 17)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 17;
    }
    return _id;
}
QT_WARNING_POP
