/****************************************************************************
** Meta object code from reading C++ file 'qdesigner_workbench.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../../qttools/src/designer/src/designer/qdesigner_workbench.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qdesigner_workbench.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN18QDesignerWorkbenchE_t {};
} // unnamed namespace

template <> constexpr inline auto QDesignerWorkbench::qt_create_metaobjectdata<qt_meta_tag_ZN18QDesignerWorkbenchE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QDesignerWorkbench",
        "modeChanged",
        "",
        "UIMode",
        "mode",
        "initialized",
        "addFormWindow",
        "QDesignerFormWindow*",
        "formWindow",
        "removeFormWindow",
        "bringAllToFront",
        "toggleFormMinimizationState",
        "showNewForm",
        "requestActivate",
        "switchToNeutralMode",
        "switchToDockedMode",
        "switchToTopLevelMode",
        "initializeCorePlugins",
        "handleCloseEvent",
        "QCloseEvent*",
        "slotFormWindowActivated",
        "fw",
        "updateWindowMenu",
        "QDesignerFormWindowInterface*",
        "formWindowActionTriggered",
        "QAction*",
        "a",
        "adjustMDIFormPositions",
        "minimizationStateChanged",
        "minimized",
        "restoreUISettings",
        "notifyUISettingsChanged",
        "slotFileDropped",
        "f"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'modeChanged'
        QtMocHelpers::SignalData<void(UIMode)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'initialized'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'addFormWindow'
        QtMocHelpers::SlotData<void(QDesignerFormWindow *)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Slot 'removeFormWindow'
        QtMocHelpers::SlotData<void(QDesignerFormWindow *)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Slot 'bringAllToFront'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'toggleFormMinimizationState'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showNewForm'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'requestActivate'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'switchToNeutralMode'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'switchToDockedMode'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'switchToTopLevelMode'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'initializeCorePlugins'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'handleCloseEvent'
        QtMocHelpers::SlotData<void(QCloseEvent *)>(18, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 19, 2 },
        }}),
        // Slot 'slotFormWindowActivated'
        QtMocHelpers::SlotData<void(QDesignerFormWindow *)>(20, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 7, 21 },
        }}),
        // Slot 'updateWindowMenu'
        QtMocHelpers::SlotData<void(QDesignerFormWindowInterface *)>(22, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 23, 21 },
        }}),
        // Slot 'formWindowActionTriggered'
        QtMocHelpers::SlotData<void(QAction *)>(24, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 25, 26 },
        }}),
        // Slot 'adjustMDIFormPositions'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'minimizationStateChanged'
        QtMocHelpers::SlotData<void(QDesignerFormWindowInterface *, bool)>(28, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 23, 8 }, { QMetaType::Bool, 29 },
        }}),
        // Slot 'restoreUISettings'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'notifyUISettingsChanged'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotFileDropped'
        QtMocHelpers::SlotData<void(const QString &)>(32, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 33 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QDesignerWorkbench, qt_meta_tag_ZN18QDesignerWorkbenchE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QDesignerWorkbench::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18QDesignerWorkbenchE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18QDesignerWorkbenchE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN18QDesignerWorkbenchE_t>.metaTypes,
    nullptr
} };

void QDesignerWorkbench::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QDesignerWorkbench *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->modeChanged((*reinterpret_cast<std::add_pointer_t<UIMode>>(_a[1]))); break;
        case 1: _t->initialized(); break;
        case 2: _t->addFormWindow((*reinterpret_cast<std::add_pointer_t<QDesignerFormWindow*>>(_a[1]))); break;
        case 3: _t->removeFormWindow((*reinterpret_cast<std::add_pointer_t<QDesignerFormWindow*>>(_a[1]))); break;
        case 4: _t->bringAllToFront(); break;
        case 5: _t->toggleFormMinimizationState(); break;
        case 6: _t->showNewForm(); break;
        case 7: _t->requestActivate(); break;
        case 8: _t->switchToNeutralMode(); break;
        case 9: _t->switchToDockedMode(); break;
        case 10: _t->switchToTopLevelMode(); break;
        case 11: _t->initializeCorePlugins(); break;
        case 12: _t->handleCloseEvent((*reinterpret_cast<std::add_pointer_t<QCloseEvent*>>(_a[1]))); break;
        case 13: _t->slotFormWindowActivated((*reinterpret_cast<std::add_pointer_t<QDesignerFormWindow*>>(_a[1]))); break;
        case 14: _t->updateWindowMenu((*reinterpret_cast<std::add_pointer_t<QDesignerFormWindowInterface*>>(_a[1]))); break;
        case 15: _t->formWindowActionTriggered((*reinterpret_cast<std::add_pointer_t<QAction*>>(_a[1]))); break;
        case 16: _t->adjustMDIFormPositions(); break;
        case 17: _t->minimizationStateChanged((*reinterpret_cast<std::add_pointer_t<QDesignerFormWindowInterface*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 18: _t->restoreUISettings(); break;
        case 19: _t->notifyUISettingsChanged(); break;
        case 20: _t->slotFileDropped((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QDesignerWorkbench::*)(UIMode )>(_a, &QDesignerWorkbench::modeChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDesignerWorkbench::*)()>(_a, &QDesignerWorkbench::initialized, 1))
            return;
    }
}

const QMetaObject *QDesignerWorkbench::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QDesignerWorkbench::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18QDesignerWorkbenchE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QDesignerWorkbench::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 21)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 21;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 21)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 21;
    }
    return _id;
}

// SIGNAL 0
void QDesignerWorkbench::modeChanged(UIMode _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void QDesignerWorkbench::initialized()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
QT_WARNING_POP
