// Copyright (C) 2026 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0

#ifndef QDOCLOGGING_H
#define QDOCLOGGING_H

#include <QtCore/qloggingcategory.h>

QT_BEGIN_NAMESPACE

Q_DECLARE_LOGGING_CATEGORY(lcQdoc)
Q_DECLARE_LOGGING_CATEGORY(lcQdocClang)

QT_END_NAMESPACE

#endif // QDOCLOGGING_H
