/*

* Copyright (C) 2016 The Qt Company Ltd.
* SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0

*/

/********************************************************************************
** Form generated from reading UI file 'batchtranslation.ui'
**
** Created by: Qt User Interface Compiler version 6.12.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_BATCHTRANSLATION_H
#define UI_BATCHTRANSLATION_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QListView>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_BatchTranslationDialog
{
public:
    QVBoxLayout *vboxLayout;
    QGroupBox *groupBox;
    QVBoxLayout *vboxLayout1;
    QCheckBox *ckMarkFinished;
    QCheckBox *ckTranslateTranslated;
    QCheckBox *ckTranslateFinished;
    QGroupBox *groupBox_2;
    QVBoxLayout *vboxLayout2;
    QHBoxLayout *hboxLayout;
    QListView *phrasebookList;
    QVBoxLayout *vboxLayout3;
    QPushButton *moveUpButton;
    QPushButton *moveDownButton;
    QSpacerItem *spacerItem;
    QLabel *label;
    QHBoxLayout *hboxLayout1;
    QSpacerItem *spacerItem1;
    QPushButton *runButton;
    QPushButton *cancelButton;

    void setupUi(QDialog *BatchTranslationDialog)
    {
        if (BatchTranslationDialog->objectName().isEmpty())
            BatchTranslationDialog->setObjectName("BatchTranslationDialog");
        BatchTranslationDialog->resize(437, 492);
        vboxLayout = new QVBoxLayout(BatchTranslationDialog);
#ifndef Q_OS_MACOS
        vboxLayout->setSpacing(6);
#endif
#ifndef Q_OS_MACOS
        vboxLayout->setContentsMargins(9, 9, 9, 9);
#endif
        vboxLayout->setObjectName("vboxLayout");
        groupBox = new QGroupBox(BatchTranslationDialog);
        groupBox->setObjectName("groupBox");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Maximum);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(groupBox->sizePolicy().hasHeightForWidth());
        groupBox->setSizePolicy(sizePolicy);
        vboxLayout1 = new QVBoxLayout(groupBox);
#ifndef Q_OS_MACOS
        vboxLayout1->setSpacing(6);
#endif
#ifndef Q_OS_MACOS
        vboxLayout1->setContentsMargins(9, 9, 9, 9);
#endif
        vboxLayout1->setObjectName("vboxLayout1");
        ckMarkFinished = new QCheckBox(groupBox);
        ckMarkFinished->setObjectName("ckMarkFinished");
        ckMarkFinished->setChecked(true);

        vboxLayout1->addWidget(ckMarkFinished);

        ckTranslateTranslated = new QCheckBox(groupBox);
        ckTranslateTranslated->setObjectName("ckTranslateTranslated");
        ckTranslateTranslated->setChecked(false);

        vboxLayout1->addWidget(ckTranslateTranslated);

        ckTranslateFinished = new QCheckBox(groupBox);
        ckTranslateFinished->setObjectName("ckTranslateFinished");

        vboxLayout1->addWidget(ckTranslateFinished);


        vboxLayout->addWidget(groupBox);

        groupBox_2 = new QGroupBox(BatchTranslationDialog);
        groupBox_2->setObjectName("groupBox_2");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Minimum);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(groupBox_2->sizePolicy().hasHeightForWidth());
        groupBox_2->setSizePolicy(sizePolicy1);
        vboxLayout2 = new QVBoxLayout(groupBox_2);
#ifndef Q_OS_MACOS
        vboxLayout2->setSpacing(6);
#endif
#ifndef Q_OS_MACOS
        vboxLayout2->setContentsMargins(9, 9, 9, 9);
#endif
        vboxLayout2->setObjectName("vboxLayout2");
        hboxLayout = new QHBoxLayout();
#ifndef Q_OS_MACOS
        hboxLayout->setSpacing(6);
#endif
#ifndef Q_OS_MACOS
        hboxLayout->setContentsMargins(0, 0, 0, 0);
#endif
        hboxLayout->setObjectName("hboxLayout");
        phrasebookList = new QListView(groupBox_2);
        phrasebookList->setObjectName("phrasebookList");
        phrasebookList->setUniformItemSizes(true);

        hboxLayout->addWidget(phrasebookList);

        vboxLayout3 = new QVBoxLayout();
#ifndef Q_OS_MACOS
        vboxLayout3->setSpacing(6);
#endif
        vboxLayout3->setContentsMargins(0, 0, 0, 0);
        vboxLayout3->setObjectName("vboxLayout3");
        moveUpButton = new QPushButton(groupBox_2);
        moveUpButton->setObjectName("moveUpButton");

        vboxLayout3->addWidget(moveUpButton);

        moveDownButton = new QPushButton(groupBox_2);
        moveDownButton->setObjectName("moveDownButton");

        vboxLayout3->addWidget(moveDownButton);

        spacerItem = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout3->addItem(spacerItem);


        hboxLayout->addLayout(vboxLayout3);


        vboxLayout2->addLayout(hboxLayout);

        label = new QLabel(groupBox_2);
        label->setObjectName("label");
        label->setWordWrap(true);

        vboxLayout2->addWidget(label);


        vboxLayout->addWidget(groupBox_2);

        hboxLayout1 = new QHBoxLayout();
#ifndef Q_OS_MACOS
        hboxLayout1->setSpacing(6);
#endif
        hboxLayout1->setContentsMargins(0, 0, 0, 0);
        hboxLayout1->setObjectName("hboxLayout1");
        spacerItem1 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout1->addItem(spacerItem1);

        runButton = new QPushButton(BatchTranslationDialog);
        runButton->setObjectName("runButton");

        hboxLayout1->addWidget(runButton);

        cancelButton = new QPushButton(BatchTranslationDialog);
        cancelButton->setObjectName("cancelButton");

        hboxLayout1->addWidget(cancelButton);


        vboxLayout->addLayout(hboxLayout1);


        retranslateUi(BatchTranslationDialog);
        QObject::connect(cancelButton, &QPushButton::clicked, BatchTranslationDialog, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(BatchTranslationDialog);
    } // setupUi

    void retranslateUi(QDialog *BatchTranslationDialog)
    {
        BatchTranslationDialog->setWindowTitle(QCoreApplication::translate("BatchTranslationDialog", "Qt Linguist - Batch Translation", nullptr));
        groupBox->setTitle(QCoreApplication::translate("BatchTranslationDialog", "Options", nullptr));
#if QT_CONFIG(tooltip)
        ckMarkFinished->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
        ckMarkFinished->setText(QCoreApplication::translate("BatchTranslationDialog", "Set translated entries to finished", nullptr));
        ckTranslateTranslated->setText(QCoreApplication::translate("BatchTranslationDialog", "Retranslate entries with existing translation", nullptr));
#if QT_CONFIG(tooltip)
        ckTranslateFinished->setToolTip(QCoreApplication::translate("BatchTranslationDialog", "Note that the modified entries will be reset to unfinished if 'Set translated entries to finished' above is unchecked", nullptr));
#endif // QT_CONFIG(tooltip)
        ckTranslateFinished->setText(QCoreApplication::translate("BatchTranslationDialog", "Translate also finished entries", nullptr));
        groupBox_2->setTitle(QCoreApplication::translate("BatchTranslationDialog", "Phrase book preference", nullptr));
        moveUpButton->setText(QCoreApplication::translate("BatchTranslationDialog", "Move up", nullptr));
        moveDownButton->setText(QCoreApplication::translate("BatchTranslationDialog", "Move down", nullptr));
        label->setText(QCoreApplication::translate("BatchTranslationDialog", "The batch translator will search through the selected phrase books in the order given above", nullptr));
        runButton->setText(QCoreApplication::translate("BatchTranslationDialog", "&Run", nullptr));
        cancelButton->setText(QCoreApplication::translate("BatchTranslationDialog", "Cancel", nullptr));
    } // retranslateUi

};

namespace Ui {
    class BatchTranslationDialog: public Ui_BatchTranslationDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_BATCHTRANSLATION_H
