/*

* Copyright (C) 2016 The Qt Company Ltd.
* SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0

*/

/********************************************************************************
** Form generated from reading UI file 'statistics.ui'
**
** Created by: Qt User Interface Compiler version 6.12.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_STATISTICS_H
#define UI_STATISTICS_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>

QT_BEGIN_NAMESPACE

class Ui_Statistics
{
public:
    QGridLayout *gridLayout;
    QHBoxLayout *hboxLayout;
    QSpacerItem *spacer4_2;
    QPushButton *closeBtn;
    QSpacerItem *spacer4;
    QFrame *frame4;
    QGridLayout *gridLayout1;
    QLabel *textLabel4;
    QLabel *textLabel5;
    QLabel *textLabel7;
    QLabel *wordsSourceTextbox;
    QLabel *wordsFinishedTextbox;
    QLabel *wordsUnfinishedTextbox;
    QLabel *textLabel1;
    QLabel *charsFinishedTextbox;
    QLabel *charsUnfinishedTextbox;
    QLabel *charsSourceTextbox;
    QLabel *textLabel3;
    QLabel *textLabel6;
    QLabel *charsSpacesFinishedTextbox;
    QLabel *charsSpacesSourceTextbox;
    QLabel *charsSpacesUnfinishedTextbox;
    QFrame *frame5;
    QGridLayout *gridLayout2;
    QLabel *textLabelTot;
    QLabel *textLabelTotFinished;
    QLabel *textLabelFinishedNoWarn;
    QLabel *textLabelFinishedWarn;
    QLabel *textLabelUnfinished;
    QLabel *textLabelUnfinishedNoWarn;
    QLabel *textLabelUnfinishedWarn;
    QLabel *textLabelTotWithObs;
    QLabel *totalMessagesTextbox;
    QLabel *totalFinishedTextbox;
    QLabel *finishedWithoutWarningsTextbox;
    QLabel *finishedWithWarningsTextbox;
    QLabel *unfinishedNoObsTextbox;
    QLabel *unfinishedNoWarningsTextbox;
    QLabel *unfinishedWithWarningsTextbox;
    QLabel *totalWithObsoleteTextbox;

    void setupUi(QDialog *Statistics)
    {
        if (Statistics->objectName().isEmpty())
            Statistics->setObjectName("Statistics");
        Statistics->resize(422, 386);
        gridLayout = new QGridLayout(Statistics);
        gridLayout->setSpacing(6);
        gridLayout->setContentsMargins(11, 11, 11, 11);
        gridLayout->setObjectName("gridLayout");
        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(6);
        hboxLayout->setObjectName("hboxLayout");
        spacer4_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(spacer4_2);

        closeBtn = new QPushButton(Statistics);
        closeBtn->setObjectName("closeBtn");

        hboxLayout->addWidget(closeBtn);

        spacer4 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(spacer4);


        gridLayout->addLayout(hboxLayout, 2, 0, 1, 1);

        frame4 = new QFrame(Statistics);
        frame4->setObjectName("frame4");
        frame4->setFrameShape(QFrame::StyledPanel);
        frame4->setFrameShadow(QFrame::Raised);
        gridLayout1 = new QGridLayout(frame4);
        gridLayout1->setSpacing(6);
        gridLayout1->setContentsMargins(11, 11, 11, 11);
        gridLayout1->setObjectName("gridLayout1");
        textLabel4 = new QLabel(frame4);
        textLabel4->setObjectName("textLabel4");

        gridLayout1->addWidget(textLabel4, 0, 2, 1, 1);

        textLabel5 = new QLabel(frame4);
        textLabel5->setObjectName("textLabel5");

        gridLayout1->addWidget(textLabel5, 0, 1, 1, 1);

        textLabel7 = new QLabel(frame4);
        textLabel7->setObjectName("textLabel7");

        gridLayout1->addWidget(textLabel7, 0, 3, 1, 1);

        wordsSourceTextbox = new QLabel(frame4);
        wordsSourceTextbox->setObjectName("wordsSourceTextbox");

        gridLayout1->addWidget(wordsSourceTextbox, 3, 1, 1, 1);

        wordsFinishedTextbox = new QLabel(frame4);
        wordsFinishedTextbox->setObjectName("wordsFinishedTextbox");

        gridLayout1->addWidget(wordsFinishedTextbox, 3, 2, 1, 1);

        wordsUnfinishedTextbox = new QLabel(frame4);
        wordsUnfinishedTextbox->setObjectName("wordsUnfinishedTextbox");

        gridLayout1->addWidget(wordsUnfinishedTextbox, 3, 3, 1, 1);

        textLabel1 = new QLabel(frame4);
        textLabel1->setObjectName("textLabel1");

        gridLayout1->addWidget(textLabel1, 3, 0, 1, 1);

        charsFinishedTextbox = new QLabel(frame4);
        charsFinishedTextbox->setObjectName("charsFinishedTextbox");

        gridLayout1->addWidget(charsFinishedTextbox, 1, 2, 1, 1);

        charsUnfinishedTextbox = new QLabel(frame4);
        charsUnfinishedTextbox->setObjectName("charsUnfinishedTextbox");

        gridLayout1->addWidget(charsUnfinishedTextbox, 1, 3, 1, 1);

        charsSourceTextbox = new QLabel(frame4);
        charsSourceTextbox->setObjectName("charsSourceTextbox");

        gridLayout1->addWidget(charsSourceTextbox, 1, 1, 1, 1);

        textLabel3 = new QLabel(frame4);
        textLabel3->setObjectName("textLabel3");

        gridLayout1->addWidget(textLabel3, 1, 0, 1, 1);

        textLabel6 = new QLabel(frame4);
        textLabel6->setObjectName("textLabel6");

        gridLayout1->addWidget(textLabel6, 2, 0, 1, 1);

        charsSpacesFinishedTextbox = new QLabel(frame4);
        charsSpacesFinishedTextbox->setObjectName("charsSpacesFinishedTextbox");

        gridLayout1->addWidget(charsSpacesFinishedTextbox, 2, 2, 1, 1);

        charsSpacesSourceTextbox = new QLabel(frame4);
        charsSpacesSourceTextbox->setObjectName("charsSpacesSourceTextbox");

        gridLayout1->addWidget(charsSpacesSourceTextbox, 2, 1, 1, 1);

        charsSpacesUnfinishedTextbox = new QLabel(frame4);
        charsSpacesUnfinishedTextbox->setObjectName("charsSpacesUnfinishedTextbox");

        gridLayout1->addWidget(charsSpacesUnfinishedTextbox, 2, 3, 1, 1);


        gridLayout->addWidget(frame4, 0, 0, 1, 1);

        frame5 = new QFrame(Statistics);
        frame5->setObjectName("frame5");
        frame5->setFrameShape(QFrame::StyledPanel);
        frame5->setFrameShadow(QFrame::Raised);
        gridLayout2 = new QGridLayout(frame5);
        gridLayout2->setSpacing(6);
        gridLayout2->setContentsMargins(11, 11, 11, 11);
        gridLayout2->setObjectName("gridLayout2");
        textLabelTot = new QLabel(frame5);
        textLabelTot->setObjectName("textLabelTot");

        gridLayout2->addWidget(textLabelTot, 0, 0, 1, 1);

        textLabelTotFinished = new QLabel(frame5);
        textLabelTotFinished->setObjectName("textLabelTotFinished");
        textLabelTotFinished->setIndent(15);

        gridLayout2->addWidget(textLabelTotFinished, 1, 0, 1, 1);

        textLabelFinishedNoWarn = new QLabel(frame5);
        textLabelFinishedNoWarn->setObjectName("textLabelFinishedNoWarn");
        textLabelFinishedNoWarn->setIndent(30);

        gridLayout2->addWidget(textLabelFinishedNoWarn, 2, 0, 1, 1);

        textLabelFinishedWarn = new QLabel(frame5);
        textLabelFinishedWarn->setObjectName("textLabelFinishedWarn");
        textLabelFinishedWarn->setIndent(30);

        gridLayout2->addWidget(textLabelFinishedWarn, 3, 0, 1, 1);

        textLabelUnfinished = new QLabel(frame5);
        textLabelUnfinished->setObjectName("textLabelUnfinished");
        textLabelUnfinished->setIndent(15);

        gridLayout2->addWidget(textLabelUnfinished, 4, 0, 1, 1);

        textLabelUnfinishedNoWarn = new QLabel(frame5);
        textLabelUnfinishedNoWarn->setObjectName("textLabelUnfinishedNoWarn");
        textLabelUnfinishedNoWarn->setIndent(30);

        gridLayout2->addWidget(textLabelUnfinishedNoWarn, 5, 0, 1, 1);

        textLabelUnfinishedWarn = new QLabel(frame5);
        textLabelUnfinishedWarn->setObjectName("textLabelUnfinishedWarn");
        textLabelUnfinishedWarn->setIndent(30);

        gridLayout2->addWidget(textLabelUnfinishedWarn, 6, 0, 1, 1);

        textLabelTotWithObs = new QLabel(frame5);
        textLabelTotWithObs->setObjectName("textLabelTotWithObs");

        gridLayout2->addWidget(textLabelTotWithObs, 7, 0, 1, 1);

        totalMessagesTextbox = new QLabel(frame5);
        totalMessagesTextbox->setObjectName("totalMessagesTextbox");

        gridLayout2->addWidget(totalMessagesTextbox, 0, 1, 1, 1);

        totalFinishedTextbox = new QLabel(frame5);
        totalFinishedTextbox->setObjectName("totalFinishedTextbox");

        gridLayout2->addWidget(totalFinishedTextbox, 1, 1, 1, 1);

        finishedWithoutWarningsTextbox = new QLabel(frame5);
        finishedWithoutWarningsTextbox->setObjectName("finishedWithoutWarningsTextbox");

        gridLayout2->addWidget(finishedWithoutWarningsTextbox, 2, 1, 1, 1);

        finishedWithWarningsTextbox = new QLabel(frame5);
        finishedWithWarningsTextbox->setObjectName("finishedWithWarningsTextbox");

        gridLayout2->addWidget(finishedWithWarningsTextbox, 3, 1, 1, 1);

        unfinishedNoObsTextbox = new QLabel(frame5);
        unfinishedNoObsTextbox->setObjectName("unfinishedNoObsTextbox");

        gridLayout2->addWidget(unfinishedNoObsTextbox, 4, 1, 1, 1);

        unfinishedNoWarningsTextbox = new QLabel(frame5);
        unfinishedNoWarningsTextbox->setObjectName("unfinishedNoWarningsTextbox");

        gridLayout2->addWidget(unfinishedNoWarningsTextbox, 5, 1, 1, 1);

        unfinishedWithWarningsTextbox = new QLabel(frame5);
        unfinishedWithWarningsTextbox->setObjectName("unfinishedWithWarningsTextbox");

        gridLayout2->addWidget(unfinishedWithWarningsTextbox, 6, 1, 1, 1);

        totalWithObsoleteTextbox = new QLabel(frame5);
        totalWithObsoleteTextbox->setObjectName("totalWithObsoleteTextbox");

        gridLayout2->addWidget(totalWithObsoleteTextbox, 7, 1, 1, 1);


        gridLayout->addWidget(frame5, 1, 0, 1, 1);


        retranslateUi(Statistics);
        QObject::connect(closeBtn, &QPushButton::clicked, Statistics, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(Statistics);
    } // setupUi

    void retranslateUi(QDialog *Statistics)
    {
        Statistics->setWindowTitle(QCoreApplication::translate("Statistics", "Statistics", nullptr));
        closeBtn->setText(QCoreApplication::translate("Statistics", "Close", nullptr));
        textLabel4->setText(QCoreApplication::translate("Statistics", "Translation", nullptr));
        textLabel5->setText(QCoreApplication::translate("Statistics", "Source", nullptr));
        textLabel7->setText(QCoreApplication::translate("Statistics", "Unfinished", nullptr));
        wordsSourceTextbox->setText(QCoreApplication::translate("Statistics", "0", nullptr));
        wordsFinishedTextbox->setText(QCoreApplication::translate("Statistics", "0", nullptr));
        wordsUnfinishedTextbox->setText(QCoreApplication::translate("Statistics", "0", nullptr));
        textLabel1->setText(QCoreApplication::translate("Statistics", "Words:", nullptr));
        charsFinishedTextbox->setText(QCoreApplication::translate("Statistics", "0", nullptr));
        charsUnfinishedTextbox->setText(QCoreApplication::translate("Statistics", "0", nullptr));
        charsSourceTextbox->setText(QCoreApplication::translate("Statistics", "0", nullptr));
        textLabel3->setText(QCoreApplication::translate("Statistics", "Characters:", nullptr));
        textLabel6->setText(QCoreApplication::translate("Statistics", "Characters (with spaces):", nullptr));
        charsSpacesFinishedTextbox->setText(QCoreApplication::translate("Statistics", "0", nullptr));
        charsSpacesSourceTextbox->setText(QCoreApplication::translate("Statistics", "0", nullptr));
        charsSpacesUnfinishedTextbox->setText(QCoreApplication::translate("Statistics", "0", nullptr));
        textLabelTot->setText(QCoreApplication::translate("Statistics", "Total translatable messages:", nullptr));
        textLabelTotFinished->setText(QCoreApplication::translate("Statistics", "Total finished:", nullptr));
        textLabelFinishedNoWarn->setText(QCoreApplication::translate("Statistics", "Without warnings:", nullptr));
        textLabelFinishedWarn->setText(QCoreApplication::translate("Statistics", "With warnings:", nullptr));
        textLabelUnfinished->setText(QCoreApplication::translate("Statistics", "Unfinished:", nullptr));
        textLabelUnfinishedNoWarn->setText(QCoreApplication::translate("Statistics", "Without warnings:", nullptr));
        textLabelUnfinishedWarn->setText(QCoreApplication::translate("Statistics", "With warnings:", nullptr));
        textLabelTotWithObs->setText(QCoreApplication::translate("Statistics", "Total messages including obsolete:", nullptr));
        totalMessagesTextbox->setText(QCoreApplication::translate("Statistics", "0", nullptr));
        totalFinishedTextbox->setText(QCoreApplication::translate("Statistics", "0", nullptr));
        finishedWithoutWarningsTextbox->setText(QCoreApplication::translate("Statistics", "0", nullptr));
        finishedWithWarningsTextbox->setText(QCoreApplication::translate("Statistics", "0", nullptr));
        unfinishedNoObsTextbox->setText(QCoreApplication::translate("Statistics", "0", nullptr));
        unfinishedNoWarningsTextbox->setText(QCoreApplication::translate("Statistics", "0", nullptr));
        unfinishedWithWarningsTextbox->setText(QCoreApplication::translate("Statistics", "0", nullptr));
        totalWithObsoleteTextbox->setText(QCoreApplication::translate("Statistics", "0", nullptr));
    } // retranslateUi

};

namespace Ui {
    class Statistics: public Ui_Statistics {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_STATISTICS_H
