// Copyright (C) 2016 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0

#include "abstractresourcebrowser.h"

QT_BEGIN_NAMESPACE

QDesignerResourceBrowserInterface::QDesignerResourceBrowserInterface(QWidget *parent)
    : QWidget(parent)
{

}

QDesignerResourceBrowserInterface::~QDesignerResourceBrowserInterface() = default;

QT_END_NAMESPACE
